@@ -81,7 +81,7 @@ public:
 		}
 #endif /*VIPS_DEBUG_VERBOSE*/
 
-		if( !steal ) {
+		if( !steal && vobject ) {
 #ifdef VIPS_DEBUG_VERBOSE
 			printf( "   reffing object\n" ); 
 #endif /*VIPS_DEBUG_VERBOSE*/
@@ -98,22 +98,21 @@ public:
 	VObject( const VObject &a ) : 
 		vobject( a.vobject )
 	{
-		g_assert( VIPS_IS_OBJECT( a.vobject ) ); 
+		g_assert( !vobject ||
+			VIPS_IS_OBJECT( vobject ) );
 
 #ifdef VIPS_DEBUG_VERBOSE
 		printf( "VObject copy constructor, obj = %p\n", 
 			vobject ); 
 		printf( "   reffing object\n" ); 
 #endif /*VIPS_DEBUG_VERBOSE*/
-		g_object_ref( vobject );
+		if( vobject )
+			g_object_ref( vobject );
 	}
 
 	// assignment ... we must delete the old ref
-	// old can be NULL, new must not be NULL
 	VObject &operator=( const VObject &a )
 	{
-		VipsObject *old_vobject;
-
 #ifdef VIPS_DEBUG_VERBOSE
 		printf( "VObject assignment\n" );  
 		printf( "   reffing %p\n", a.vobject ); 
@@ -122,16 +121,16 @@ public:
 
 		g_assert( !vobject ||
 			VIPS_IS_OBJECT( vobject ) ); 
-		g_assert( a.vobject &&
+		g_assert( !a.vobject ||
 			VIPS_IS_OBJECT( a.vobject ) ); 
 
 		// delete the old ref at the end ... otherwise "a = a;" could
 		// unref before reffing again 
-		old_vobject = vobject;
+		if( a.vobject )
+			g_object_ref( a.vobject );
+		if( vobject )
+			g_object_unref( vobject );
 		vobject = a.vobject;
-		g_object_ref( vobject ); 
-		if( old_vobject )
-			g_object_unref( old_vobject );
 
 		return( *this ); 
 	}
@@ -169,6 +168,8 @@ public:
 
 class VIPS_CPLUSPLUS_API VImage;
 class VIPS_CPLUSPLUS_API VInterpolate;
+class VIPS_CPLUSPLUS_API VSource;
+class VIPS_CPLUSPLUS_API VTarget;
 class VIPS_CPLUSPLUS_API VOption;
 
 class VOption
@@ -220,8 +221,10 @@ public:
 	VOption *set( const char *name, int value );
 	VOption *set( const char *name, double value );
 	VOption *set( const char *name, const char *value );
-	VOption *set( const char *name, VImage value );
-	VOption *set( const char *name, VInterpolate value ); 
+	VOption *set( const char *name, const VImage value );
+	VOption *set( const char *name, const VInterpolate value ); 
+	VOption *set( const char *name, const VSource value );
+	VOption *set( const char *name, const VTarget value );
 	VOption *set( const char *name, std::vector<VImage> value );
 	VOption *set( const char *name, std::vector<double> value );
 	VOption *set( const char *name, std::vector<int> value );
@@ -351,6 +354,19 @@ public:
 		vips_image_set_int( this->get_image(), field, value ); 
 	}
 
+	void 
+	set( const char *field, int *value, int n )
+	{
+		vips_image_set_array_int( this->get_image(), field, value, n ); 
+	}
+
+	void 
+	set( const char *field, std::vector<int> value )
+	{
+		vips_image_set_array_int( this->get_image(), field, &value[0],
+			static_cast<int>( value.size() ) );
+	}
+
 	void 
 	set( const char *field, double value )
 	{
@@ -388,6 +404,27 @@ public:
 		return( value ); 
 	}
 
+	void
+	get_array_int( const char *field, int **out, int *n ) const
+	{
+		if( vips_image_get_array_int( this->get_image(), field, out, n ) )
+			throw( VError() ); 
+	}
+
+	std::vector<int> 
+	get_array_int( const char *field ) const
+	{
+		int length;
+		int *array;
+
+		if( vips_image_get_array_int( this->get_image(), field, &array, &length ) )
+			throw( VError() ); 
+
+		std::vector<int> vector( array, array + length );
+
+		return( vector );
+	}
+
 	double 
 	get_double( const char *field ) const
 	{
@@ -476,6 +513,9 @@ public:
 	static VImage new_from_buffer( const std::string &buf,
 		const char *option_string, VOption *options = 0 );
 
+	static VImage new_from_source( VSource source, 
+		const char *option_string, VOption *options = 0 );
+
 	static VImage new_matrix( int width, int height );
 
 	static VImage 
@@ -528,6 +568,9 @@ public:
 	void write_to_buffer( const char *suffix, void **buf, size_t *size, 
 		VOption *options = 0 ) const;
 
+	void write_to_target( const char *suffix, VTarget target, 
+		VOption *options = 0 ) const;
+
 	void *
 	write_to_memory( size_t *size ) const
 	{
