@@ -73,6 +73,7 @@
 #include <stdlib.h>
 
 #include <vips/vips.h>
+#include <vips/internal.h>
 
 #include "binary.h"
 #include "unaryconst.h"
@@ -140,6 +141,15 @@ vips_boolean_build( VipsObject *object )
 		g_assert_not_reached(); \
 	} 
 
+#define FNLOOP( TYPE, FN ) { \
+	TYPE * restrict left = (TYPE *) in[0]; \
+	TYPE * restrict right = (TYPE *) in[1]; \
+	int * restrict q = (int *) out; \
+	\
+	for( x = 0; x < sz; x++ ) \
+		q[x] = FN( left[x], right[x] ); \
+}
+
 static void
 vips_boolean_buffer( VipsArithmetic *arithmetic, 
 	VipsPel *out, VipsPel **in, int width )
@@ -163,8 +173,30 @@ vips_boolean_buffer( VipsArithmetic *arithmetic,
 		SWITCH( LOOP, FLOOP, ^ ); 
 		break;
 
+	/* Special case: we need to be able to use VIPS_LSHIFT_INT().
+	 */
 	case VIPS_OPERATION_BOOLEAN_LSHIFT: 	
-		SWITCH( LOOP, FLOOP, << ); 
+		switch( vips_image_get_format( im ) ) { 
+		case VIPS_FORMAT_UCHAR:	
+			LOOP( unsigned char, << ); break; 
+		case VIPS_FORMAT_CHAR:
+			FNLOOP( signed char, VIPS_LSHIFT_INT ); break; 
+		case VIPS_FORMAT_USHORT:
+			LOOP( unsigned short, << ); break; 
+		case VIPS_FORMAT_SHORT:
+			FNLOOP( signed short, VIPS_LSHIFT_INT ); break; 
+		case VIPS_FORMAT_UINT:
+			LOOP( unsigned int, << ); break; 
+		case VIPS_FORMAT_INT:
+			FNLOOP( signed int, VIPS_LSHIFT_INT ); break; 
+		case VIPS_FORMAT_FLOAT:
+			FLOOP( float, << ); break; 
+		case VIPS_FORMAT_DOUBLE:
+			FLOOP( double, << ); break;
+		
+		default: 
+			g_assert_not_reached(); 
+		} 
 		break;
 
 	case VIPS_OPERATION_BOOLEAN_RSHIFT: 	
@@ -426,14 +458,11 @@ vips_boolean_const_build( VipsObject *object )
 {
 	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( object );
 	VipsUnary *unary = (VipsUnary *) object;
-	VipsUnaryConst *uconst = (VipsUnaryConst *) object;
 
 	if( unary->in &&
 		vips_check_noncomplex( class->nickname, unary->in ) )
 		return( -1 );
 
-	uconst->const_format = VIPS_FORMAT_INT;
-
 	if( VIPS_OBJECT_CLASS( vips_boolean_const_parent_class )->
 		build( object ) )
 		return( -1 );
@@ -442,9 +471,9 @@ vips_boolean_const_build( VipsObject *object )
 }
 
 #define LOOPC( TYPE, OP ) { \
-	TYPE *p = (TYPE *) in[0]; \
-	TYPE *q = (TYPE *) out; \
-	int *c = (int *) uconst->c_ready; \
+	TYPE * restrict p = (TYPE *) in[0]; \
+	TYPE * restrict q = (TYPE *) out; \
+	int * restrict c = uconst->c_int; \
  	\
 	for( i = 0, x = 0; x < width; x++ ) \
 		for( b = 0; b < bands; b++, i++ ) \
@@ -452,9 +481,9 @@ vips_boolean_const_build( VipsObject *object )
 }
 
 #define FLOOPC( TYPE, OP ) { \
-	TYPE *p = (TYPE *) in[0]; \
-	int *q = (int *) out; \
-	int *c = (int *) uconst->c_ready; \
+	TYPE * restrict p = (TYPE *) in[0]; \
+	int * restrict q = (int *) out; \
+	int * restrict c = uconst->c_int; \
  	\
 	for( i = 0, x = 0; x < width; x++ ) \
 		for( b = 0; b < bands; b++, i++ ) \
@@ -533,7 +562,7 @@ vips_boolean_const_init( VipsBooleanConst *boolean_const )
 
 static int
 vips_boolean_constv( VipsImage *in, VipsImage **out, 
-	VipsOperationBoolean operation, double *c, int n, va_list ap )
+	VipsOperationBoolean operation, const double *c, int n, va_list ap )
 {
 	VipsArea *area_c;
 	double *array; 
@@ -580,7 +609,7 @@ vips_boolean_constv( VipsImage *in, VipsImage **out,
  */
 int
 vips_boolean_const( VipsImage *in, VipsImage **out, 
-	VipsOperationBoolean boolean, double *c, int n, ... )
+	VipsOperationBoolean boolean, const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -608,7 +637,8 @@ vips_boolean_const( VipsImage *in, VipsImage **out,
  * Returns: 0 on success, -1 on error
  */
 int
-vips_andimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_andimage_const( VipsImage *in, VipsImage **out, 
+	const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -637,7 +667,8 @@ vips_andimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
  * Returns: 0 on success, -1 on error
  */
 int
-vips_orimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_orimage_const( VipsImage *in, VipsImage **out, 
+	const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -666,7 +697,8 @@ vips_orimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
  * Returns: 0 on success, -1 on error
  */
 int
-vips_eorimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_eorimage_const( VipsImage *in, VipsImage **out, 
+	const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -695,7 +727,7 @@ vips_eorimage_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
  * Returns: 0 on success, -1 on error
  */
 int
-vips_lshift_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_lshift_const( VipsImage *in, VipsImage **out, const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -724,7 +756,7 @@ vips_lshift_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
  * Returns: 0 on success, -1 on error
  */
 int
-vips_rshift_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_rshift_const( VipsImage *in, VipsImage **out, const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
