@@ -136,6 +136,8 @@ vips_complex_hypot( double a, double b )
 {
 	double d;
 
+	/* hypot() is less sensitive to overflow. Use it if we can.
+	 */
 #ifdef HAVE_HYPOT
 	d = hypot( a, b ); 
 #else
@@ -150,6 +152,9 @@ vips_complex_atan2( double a, double b )
 {
 	double h;
 
+	/* atan2() is very slow, but is better behaved when a is near 0. Use
+	 * it in preference when we can.
+	 */
 #ifdef HAVE_ATAN2
 	h = VIPS_DEG( atan2( b, a ) );
 	if( h < 0.0 )
@@ -338,7 +343,7 @@ vips_rect( VipsImage *in, VipsImage **out, ... )
 	int result;
 
 	va_start( ap, out );
-	result = vips_complexv( in, out, VIPS_OPERATION_COMPLEX_POLAR, ap );
+	result = vips_complexv( in, out, VIPS_OPERATION_COMPLEX_RECT, ap );
 	va_end( ap );
 
 	return( result );
