@@ -40,10 +40,11 @@
  * 	- removed the 1-ary constant path, no faster
  * 30/11/13
  * 	- 1ary is back, faster with gcc 4.8
- * 3/12/13
- * 	- try an ORC path with the band loop unrolled
  * 14/1/14
  * 	- add uchar output option
+ * 30/9/17
+ * 	- squash constants with all elements equal so we use 1ary path more
+ * 	  often
  */
 
 /*
@@ -124,7 +125,7 @@ vips_linear_build( VipsObject *object )
 
 	int i;
 
-	/* If we have a three-element vector we need to bandup the image to
+	/* If we have a three-element vector, we need to bandup the image to
 	 * match.
 	 */
 	linear->n = 1;
@@ -150,6 +151,38 @@ vips_linear_build( VipsObject *object )
 		return( -1 );
 	}
 
+	/* If all elements of the constants are equal, we can shrink them down
+	 * to a single element.
+	 */
+	if( linear->a ) {
+		double *ary = (double *) linear->a->data;
+		gboolean all_equal;
+
+		all_equal = TRUE;
+		for( i = 1; i < linear->a->n; i++ )
+			if( ary[i] != ary[0] ) {
+				all_equal = FALSE;
+				break;
+			}
+
+		if( all_equal )
+			linear->a->n = 1;
+	}
+	if( linear->b ) {
+		double *ary = (double *) linear->b->data;
+		gboolean all_equal;
+
+		all_equal = TRUE;
+		for( i = 1; i < linear->b->n; i++ )
+			if( ary[i] != ary[0] ) {
+				all_equal = FALSE;
+				break;
+			}
+
+		if( all_equal )
+			linear->b->n = 1;
+	}
+
 	/* Make up-banded versions of our constants.
 	 */
 	linear->a_ready = VIPS_ARRAY( linear, linear->n, double );
@@ -420,14 +453,14 @@ vips_linear_init( VipsLinear *linear )
 
 static int
 vips_linearv( VipsImage *in, VipsImage **out, 
-	double *a, double *b, int n, va_list ap )
+	const double *a, const double *b, int n, va_list ap )
 {
 	VipsArea *area_a;
 	VipsArea *area_b;
 	int result;
 
-	area_a = (VipsArea *) vips_array_double_new( a, n );
-	area_b = (VipsArea *) vips_array_double_new( b, n );
+	area_a = VIPS_AREA( vips_array_double_new( a, n ) );
+	area_b = VIPS_AREA( vips_array_double_new( b, n ) );
 
 	result = vips_call_split( "linear", ap, in, out, area_a, area_b );
 
@@ -438,9 +471,9 @@ vips_linearv( VipsImage *in, VipsImage **out,
 }
 
 /**
- * vips_linear:
+ * vips_linear: (method)
  * @in: image to transform
- * @out: output image
+ * @out: (out): output image
  * @a: (array length=n): array of constants for multiplication
  * @b: (array length=n): array of constants for addition
  * @n: length of constant arrays
@@ -467,7 +500,8 @@ vips_linearv( VipsImage *in, VipsImage **out,
  * Returns: 0 on success, -1 on error
  */
 int
-vips_linear( VipsImage *in, VipsImage **out, double *a, double *b, int n, ... )
+vips_linear( VipsImage *in, VipsImage **out, 
+	const double *a, const double *b, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -480,9 +514,9 @@ vips_linear( VipsImage *in, VipsImage **out, double *a, double *b, int n, ... )
 }
 
 /**
- * vips_linear1:
+ * vips_linear1: (method)
  * @in: image to transform
- * @out: output image
+ * @out: (out): output image
  * @a: constant for multiplication
  * @b: constant for addition
  * @...: %NULL-terminated list of optional named arguments
