@@ -338,14 +338,11 @@ vips_math2_const_build( VipsObject *object )
 {
 	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( object );
 	VipsUnary *unary = (VipsUnary *) object;
-	VipsUnaryConst *uconst = (VipsUnaryConst *) object;
 
 	if( unary->in &&
 		vips_check_noncomplex( class->nickname, unary->in ) )
 		return( -1 );
 
-	uconst->const_format = VIPS_FORMAT_DOUBLE;
-
 	if( VIPS_OBJECT_CLASS( vips_math2_const_parent_class )->
 		build( object ) )
 		return( -1 );
@@ -356,7 +353,7 @@ vips_math2_const_build( VipsObject *object )
 #define LOOPC( IN, OUT, OP ) { \
 	IN * restrict p = (IN *) in[0]; \
 	OUT * restrict q = (OUT *) out; \
-	double * restrict c = (double *) uconst->c_ready; \
+	double * restrict c = uconst->c_double; \
 	\
 	for( i = 0, x = 0; x < width; x++ ) \
 		for( b = 0; b < bands; b++, i++ ) \
@@ -423,7 +420,7 @@ vips_math2_const_init( VipsMath2Const *math2_const )
 
 static int
 vips_math2_constv( VipsImage *in, VipsImage **out, 
-	VipsOperationMath2 math2, double *c, int n, va_list ap )
+	VipsOperationMath2 math2, const double *c, int n, va_list ap )
 {
 	VipsArea *area_c;
 	double *array; 
@@ -473,7 +470,7 @@ vips_math2_constv( VipsImage *in, VipsImage **out,
  */
 int
 vips_math2_const( VipsImage *in, VipsImage **out, 
-	VipsOperationMath2 math2, double *c, int n, ... )
+	VipsOperationMath2 math2, const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -499,7 +496,7 @@ vips_math2_const( VipsImage *in, VipsImage **out,
  * Returns: 0 on success, -1 on error
  */
 int
-vips_pow_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_pow_const( VipsImage *in, VipsImage **out, const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
@@ -526,7 +523,7 @@ vips_pow_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
  * Returns: 0 on success, -1 on error
  */
 int
-vips_wop_const( VipsImage *in, VipsImage **out, double *c, int n, ... )
+vips_wop_const( VipsImage *in, VipsImage **out, const double *c, int n, ... )
 {
 	va_list ap;
 	int result;
