@@ -3,19 +3,19 @@
 
 /*
 
-    Copyright (C) 1991-2005 The National Gallery
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
+    This file is part of VIPS.
+    
+    VIPS is free software; you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
+    You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
     02110-1301  USA
@@ -216,6 +216,13 @@ extern float vips_v2Y_16[65536];
 void vips_col_make_tables_RGB_8( void );
 void vips_col_make_tables_RGB_16( void );
 
+/* A colour-transforming function.
+ */
+typedef int (*VipsColourTransformFn)( VipsImage *in, VipsImage **out, ... );
+
+int vips__colourspace_process_n( const char *domain, 
+	VipsImage *in, VipsImage **out, int n, VipsColourTransformFn fn );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
