@@ -343,6 +343,13 @@ vips_composite_base_max_band( VipsCompositeBase *composite, double *max_band )
 		max_band[2] = 255;
 		break;
 
+	case VIPS_INTERPRETATION_CMYK:
+		max_band[0] = 255;
+		max_band[1] = 255;
+		max_band[2] = 255;
+		max_band[3] = 255;
+		break;
+
 	case VIPS_INTERPRETATION_RGB16:
 		max_band[0] = 65535;
 		max_band[1] = 65535;
@@ -621,7 +628,7 @@ vips_composite_base_blend( VipsCompositeBase *composite,
 
 		case VIPS_BLEND_MODE_DIFFERENCE:
 			for( int b = 0; b < bands; b++ ) 
-				f[b] = abs( B[b] - A[b] );
+				f[b] = fabs( B[b] - A[b] );
 			break;
 
 		case VIPS_BLEND_MODE_EXCLUSION:
@@ -1721,7 +1728,18 @@ vips_composite2( VipsImage *base, VipsImage *overlay, VipsImage **out,
 	va_list ap;
 	int result;
 
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wvarargs"
+
+	/* Triggers a clang compiler warning because mode might not be an int.
+	 * I think the warning is harmless for all platforms we care about.
+	 */
 	va_start( ap, mode );
+
+	g_assert( sizeof( mode ) == sizeof( int ) );
+
+#pragma clang diagnostic pop
+
 	result = vips_call_split( "composite2", ap, base, overlay, out, mode );
 	va_end( ap );
 
