@@ -214,6 +214,7 @@
  * @VIPS_INTERESTING_ATTENTION: look for features likely to draw human attention
  * @VIPS_INTERESTING_LOW: position the crop towards the low coordinate
  * @VIPS_INTERESTING_HIGH: position the crop towards the high coordinate
+ * @VIPS_INTERESTING_ALL: everything is interesting
  *
  * Pick the algorithm vips uses to decide image "interestingness". This is used
  * by vips_smartcrop(), for example, to decide what parts of the image to
@@ -383,6 +384,7 @@ vips_conversion_operation_init( void )
 	extern GType vips_rot45_get_type( void ); 
 	extern GType vips_autorot_get_type( void ); 
 	extern GType vips_ifthenelse_get_type( void ); 
+	extern GType vips_switch_get_type( void ); 
 	extern GType vips_recomb_get_type( void ); 
 	extern GType vips_bandmean_get_type( void ); 
 	extern GType vips_bandfold_get_type( void ); 
@@ -434,6 +436,7 @@ vips_conversion_operation_init( void )
 	vips_rot45_get_type();
 	vips_autorot_get_type();
 	vips_ifthenelse_get_type();
+	vips_switch_get_type(); 
 	vips_recomb_get_type(); 
 	vips_bandmean_get_type(); 
 	vips_bandfold_get_type(); 
