@@ -129,12 +129,13 @@ vips_gaussmat_build( VipsObject *object )
 		vips_error( class->nickname, "%s", _( "mask too large" ) );
 		return( -1 );
 	}
-	width = 2 * x - 1;
+	width = 2 * VIPS_MAX( x - 1, 0 ) + 1;
 	height = gaussmat->separable ? 1 : width; 
 
 	vips_image_init_fields( create->out,
 		width, height, 1, 
-		VIPS_FORMAT_DOUBLE, VIPS_CODING_NONE, VIPS_INTERPRETATION_B_W,
+		VIPS_FORMAT_DOUBLE, VIPS_CODING_NONE, 
+		VIPS_INTERPRETATION_MULTIBAND,
 		1.0, 1.0 ); 
 	vips_image_pipelinev( create->out, 
 		VIPS_DEMAND_STYLE_ANY, NULL );
@@ -157,6 +158,11 @@ vips_gaussmat_build( VipsObject *object )
 		}
 	}
 
+	/* Make sure we can't make sum == 0: it'd certainly cause /0 later. 
+	 */
+	if( sum == 0 )
+		sum = 1;
+
 	vips_image_set_double( create->out, "scale", sum ); 
 	vips_image_set_double( create->out, "offset", 0.0 ); 
 
