@@ -30,6 +30,8 @@
  * 	- set Xoffset/Yoffset to ink left/top
  * 27/6/19
  * 	- fitting could occasionally terminate early [levmorozov]
+ * 16/5/20 [keiviv]
+ * 	- don't add fontfiles repeatedly
  */
 
 /*
@@ -103,6 +105,10 @@ typedef VipsCreateClass VipsTextClass;
 
 G_DEFINE_TYPE( VipsText, vips_text, VIPS_TYPE_CREATE );
 
+/* ... single-thread the body of vips_text() with this.
+ */
+static GMutex *vips_text_lock = NULL; 
+
 /* Just have one of these and reuse it.
  *
  * This does not unref cleanly on many platforms, so we will leak horribly
@@ -111,9 +117,10 @@ G_DEFINE_TYPE( VipsText, vips_text, VIPS_TYPE_CREATE );
  */
 static PangoFontMap *vips_text_fontmap = NULL;
 
-/* ... single-thread the body of vips_text() with this.
+/* All the fontfiles we've loaded. fontconfig lets you add a fontfile
+ * repeatedly, and we obviously don't want that.
  */
-static GMutex *vips_text_lock = NULL; 
+static GHashTable *vips_text_fontfiles = NULL;
 
 static void
 vips_text_dispose( GObject *gobject )
@@ -182,6 +189,7 @@ vips_text_get_extents( VipsText *text, VipsRect *extents )
 	pango_ft2_font_map_set_resolution( 
 		PANGO_FT2_FONT_MAP( vips_text_fontmap ), text->dpi, text->dpi );
 
+	VIPS_UNREF( text->layout );
 	if( !(text->layout = text_layout_new( text->context, 
 		text->text, text->font, 
 		text->width, text->spacing, text->align, text->justify )) ) 
@@ -239,6 +247,7 @@ vips_text_autofit( VipsText *text )
 	target.width = text->width;
 	target.height = text->height;
 	previous_dpi = -1;
+	previous_difference = 0;
 
 #ifdef DEBUG
 	printf( "vips_text_autofit: "
@@ -348,17 +357,26 @@ vips_text_build( VipsObject *object )
 
 	if( !vips_text_fontmap )
 		vips_text_fontmap = pango_ft2_font_map_new();
+	if( !vips_text_fontfiles )
+		vips_text_fontfiles = 
+			g_hash_table_new( g_str_hash, g_str_equal );
 
 	text->context = pango_font_map_create_context( 
 		PANGO_FONT_MAP( vips_text_fontmap ) );
 
 	if( text->fontfile &&
-		!FcConfigAppFontAddFile( NULL, 
+		!g_hash_table_lookup( vips_text_fontfiles, text->fontfile ) ) {
+		if( !FcConfigAppFontAddFile( NULL, 
 			(const FcChar8 *) text->fontfile ) ) {
-		vips_error( class->nickname, 
-			_( "unable to load font \"%s\"" ), text->fontfile );
-		g_mutex_unlock( vips_text_lock ); 
-		return( -1 );
+			vips_error( class->nickname, 
+				_( "unable to load font \"%s\"" ), 
+				text->fontfile );
+			g_mutex_unlock( vips_text_lock ); 
+			return( -1 );
+		}
+		g_hash_table_insert( vips_text_fontfiles, 
+			text->fontfile,
+			g_strdup( text->fontfile ) );
 	}
 
 	/* If our caller set height and not dpi, we adjust dpi until 
@@ -541,7 +559,7 @@ vips_text_init( VipsText *text )
  * * @fontfile: %gchararray, load this font file
  * * @width: %gint, image should be no wider than this many pixels
  * * @height: %gint, image should be no higher than this many pixels
- * * @align: #VipsAlign, left/centre/right alignment
+ * * @align: #VipsAlign, set justification alignment
  * * @justify: %gboolean, justify lines
  * * @dpi: %gint, render at this resolution
  * * @autofit_dpi: %gint, read out auto-fitted DPI 
@@ -562,11 +580,12 @@ vips_text_init( VipsText *text )
  *
  * @width is the number of pixels to word-wrap at. Lines of text wider than
  * this will be broken at word boundaries. 
- * @align can be used to set the alignment style for multi-line
- * text. Note that the output image can be wider than @width if there are no
- * word breaks, or narrower if the lines don't break exactly at @width. 
  *
  * Set @justify to turn on line justification.
+ * @align can be used to set the alignment style for multi-line
+ * text to the low (left) edge centre, or high (right) edge. Note that the 
+ * output image can be wider than @width if there are no
+ * word breaks, or narrower if the lines don't break exactly at @width. 
  *
  * @height is the maximum number of pixels high the generated text can be. This
  * only takes effect when @dpi is not set, and @width is set, making a box. 
