@@ -38,6 +38,7 @@
 #include <string.h>
 
 #include <vips/vips.h>
+#include <vips/vips7compat.h>
 #include <vips/internal.h>
 
 int 
@@ -800,3 +801,25 @@ vips_warn( const char *domain, const char *fmt, ... )
 	va_end( ap );
 }
 
+/**
+ * vips_autorot_get_angle:
+ * @image: image to fetch orientation from
+ *
+ * This function is deprecated. Use vips_autorot() instead.
+ */
+VipsAngle
+vips_autorot_get_angle( VipsImage *im )
+{
+	return( VIPS_ANGLE_D0 );
+}
+
+/* The old vips_free(), now replaced by g_free() and vips_area_free_cb().
+ */
+int
+vips_free( void *buf )
+{
+	g_free( buf );
+
+	return( 0 );
+}
+
