@@ -77,6 +77,7 @@
  * @stability: Stable
  * @see_also: <link linkend="libvips-image">image</link>
  * @include: vips/vips.h
+ * @title: VipsForeign
  *
  * This set of operations load and save images in a variety of formats. 
  *
@@ -389,7 +390,7 @@ file_add_class( VipsForeignClass *class, GSList **files )
 }
 
 static gint
-file_compare( VipsForeignClass *a, VipsForeignClass *b )
+file_compare( VipsForeignClass *a, VipsForeignClass *b, void *user_data )
 {
         return( b->priority - a->priority );
 }
@@ -457,6 +458,8 @@ vips_foreign_load_summary_class( VipsObjectClass *object_class, VipsBuf *buf )
 			vips_buf_appends( buf, ", is_a" );
 		if( class->is_a_buffer )
 			vips_buf_appends( buf, ", is_a_buffer" );
+		if( class->is_a_source )
+			vips_buf_appends( buf, ", is_a_source" );
 		if( class->get_flags )
 			vips_buf_appends( buf, ", get_flags" );
 		if( class->get_flags_filename )
@@ -476,8 +479,9 @@ vips_foreign_load_summary_class( VipsObjectClass *object_class, VipsBuf *buf )
  */
 static void *
 vips_foreign_find_load_sub( VipsForeignLoadClass *load_class, 
-	const char *filename )
+	const char *filename, void *b )
 {
+	VipsObjectClass *object_class = VIPS_OBJECT_CLASS( load_class );
 	VipsForeignClass *class = VIPS_FOREIGN_CLASS( load_class );
 
 #ifdef DEBUG
@@ -485,7 +489,9 @@ vips_foreign_find_load_sub( VipsForeignLoadClass *load_class,
 		VIPS_OBJECT_CLASS( class )->nickname );
 #endif /*DEBUG*/
 
-	if( load_class->is_a ) {
+	if( load_class->is_a &&
+		!vips_ispostfix( object_class->nickname, "_buffer" ) &&
+		!vips_ispostfix( object_class->nickname, "_source" ) ) {
 		if( load_class->is_a( filename ) ) 
 			return( load_class );
 
@@ -525,9 +531,16 @@ vips_foreign_find_load( const char *name )
 
 	vips__filename_split8( name, filename, option_string );
 
+	/* Very common, so make a better error message for this case.
+	 */
 	if( !vips_existsf( "%s", filename ) ) {
 		vips_error( "VipsForeignLoad", 
-			_( "file \"%s\" not found" ), name );
+			_( "file \"%s\" does not exist" ), name );
+		return( NULL );
+	}
+	if( vips_isdirf( "%s", filename ) ) {
+		vips_error( "VipsForeignLoad", 
+			_( "\"%s\" is a directory" ), name );
 		return( NULL );
 	}
 
@@ -579,7 +592,10 @@ static void *
 vips_foreign_find_load_buffer_sub( VipsForeignLoadClass *load_class, 
 	const void **buf, size_t *len )
 {
+	VipsObjectClass *object_class = VIPS_OBJECT_CLASS( load_class );
+
 	if( load_class->is_a_buffer &&
+		vips_ispostfix( object_class->nickname, "_buffer" ) &&
 		load_class->is_a_buffer( *buf, *len ) ) 
 		return( load_class );
 
@@ -619,6 +635,60 @@ vips_foreign_find_load_buffer( const void *data, size_t size )
 	return( G_OBJECT_CLASS_NAME( load_class ) );
 }
 
+/* Can this VipsForeign open this source?
+ */
+static void *
+vips_foreign_find_load_source_sub( void *item, void *a, void *b )
+{
+	VipsObjectClass *object_class = VIPS_OBJECT_CLASS( item );
+	VipsForeignLoadClass *load_class = VIPS_FOREIGN_LOAD_CLASS( item );
+	VipsSource *source = VIPS_SOURCE( a );
+
+	if( load_class->is_a_source &&
+		vips_ispostfix( object_class->nickname, "_source" ) ) {
+		/* We may have done a read() rather than a sniff() in one of
+		 * the is_a testers. Always rewind.
+		 */
+		(void) vips_source_rewind( source );
+
+		if( load_class->is_a_source( source ) ) 
+			return( load_class );
+	}
+
+	return( NULL );
+}
+
+/**
+ * vips_foreign_find_load_source:
+ * @source: source to load from
+ *
+ * Searches for an operation you could use to load a source. To see the
+ * range of source loaders supported by your vips, try something like:
+ * 
+ * 	vips -l | grep load_source
+ *
+ * See also: vips_image_new_from_source().
+ *
+ * Returns: (transfer none): the name of an operation on success, %NULL on 
+ * error.
+ */
+const char *
+vips_foreign_find_load_source( VipsSource *source )
+{
+	VipsForeignLoadClass *load_class;
+
+	if( !(load_class = (VipsForeignLoadClass *) vips_foreign_map( 
+		"VipsForeignLoad",
+		vips_foreign_find_load_source_sub, 
+		source, NULL )) ) {
+		vips_error( "VipsForeignLoad", 
+			"%s", _( "source is not in a known format" ) ); 
+		return( NULL );
+	}
+
+	return( G_OBJECT_CLASS_NAME( load_class ) );
+}
+
 /**
  * vips_foreign_is_a:
  * @loader: name of loader to use for test
@@ -672,6 +742,32 @@ vips_foreign_is_a_buffer( const char *loader, const void *data, size_t size )
 	return( FALSE );
 }
 
+/**
+ * vips_foreign_is_a_source:
+ * @loader: name of loader to use for test
+ * @source: source to test
+ *
+ * Return %TRUE if @source can be loaded by @loader. @loader is something
+ * like "tiffload_source" or "VipsForeignLoadTiffSource".
+ *
+ * Returns: %TRUE if @data can be loaded by @source.
+ */
+gboolean
+vips_foreign_is_a_source( const char *loader, VipsSource *source )
+{
+	const VipsObjectClass *class;
+	VipsForeignLoadClass *load_class;
+
+	if( !(class = vips_class_find( "VipsForeignLoad", loader )) )
+		return( FALSE );
+	load_class = VIPS_FOREIGN_LOAD_CLASS( class );
+	if( load_class->is_a_source &&
+		load_class->is_a_source( source ) )
+		return( TRUE );
+
+	return( FALSE );
+}
+
 /**
  * vips_foreign_flags:
  * @loader: name of loader to use for test
@@ -838,7 +934,7 @@ vips_foreign_load_start( VipsImage *out, void *a, void *b )
 
 		/* Load the image and check the result.
 		 *
-		 * ->header() read the header into @out, load has read the
+		 * ->header() read the header into @out, load will read the
 		 * image into @real. They must match exactly in size, bands,
 		 * format and coding for the copy to work.  
 		 *
@@ -1240,8 +1336,7 @@ vips__foreign_convert_saveable( VipsImage *in, VipsImage **ready,
 	}
 
 	/* If this image is CMYK and the saver is RGB-only, use lcms to try to
-	 * import to XYZ. This will only work if the image has an embedded
-	 * profile. 
+	 * import to XYZ. 
 	 */
 	if( in->Type == VIPS_INTERPRETATION_CMYK &&
 		in->Bands >= 4 &&
@@ -1252,6 +1347,8 @@ vips__foreign_convert_saveable( VipsImage *in, VipsImage **ready,
 
 		if( vips_icc_import( in, &out, 
 			"pcs", VIPS_PCS_XYZ,
+			"embedded", TRUE,
+			"input_profile", "cmyk",
 			NULL ) ) {
 			g_object_unref( in );
 			return( -1 );
@@ -1514,8 +1611,19 @@ vips__foreign_convert_saveable( VipsImage *in, VipsImage **ready,
 
 		if( !vips_image_get_blob( in, VIPS_META_ICC_NAME, 
 			&data, &length ) &&
-			!vips_icc_is_compatible_profile( in, data, length ) ) 
+			!vips_icc_is_compatible_profile( in, data, length ) ) {
+			VipsImage *out;
+
+			if( vips_copy( in, &out, NULL ) ) {
+				g_object_unref( in );
+				return( -1 );
+			}
+			g_object_unref( in );
+
+			in = out;
+
 			vips_image_remove( in, VIPS_META_ICC_NAME );
+		}
 	}
 
 	*ready = in;
@@ -1538,9 +1646,19 @@ vips_foreign_save_build( VipsObject *object )
 			save->background ) )
 			return( -1 );
 
-		if( save->page_height )
+		if( save->page_height ) {
+			VipsImage *x;
+
+			if( vips_copy( ready, &x, NULL ) ) {
+				VIPS_UNREF( ready );
+				return( -1 );
+			}
+			VIPS_UNREF( ready );
+			ready = x;
+
 			vips_image_set_int( ready, 
 				VIPS_META_PAGE_HEIGHT, save->page_height );
+		}
 
 		VIPS_UNREF( save->ready );
 		save->ready = ready;
@@ -1628,7 +1746,7 @@ vips_foreign_save_class_init( VipsForeignSaveClass *class )
 		G_STRUCT_OFFSET( VipsForeignSave, background ),
 		VIPS_TYPE_ARRAY_DOUBLE );
 
-	VIPS_ARG_INT( class, "page_height", 8, 
+	VIPS_ARG_INT( class, "page_height", 102, 
 		_( "Page height" ), 
 		_( "Set page height for multipage save" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
@@ -1642,19 +1760,26 @@ vips_foreign_save_init( VipsForeignSave *save )
 	save->background = vips_array_double_newv( 1, 0.0 );
 }
 
-/* Can we write this filename with this file? 
+/* Can we write this filename with this class? 
  */
 static void *
 vips_foreign_find_save_sub( VipsForeignSaveClass *save_class, 
-	const char *filename )
+	const char *filename, void *b )
 {
+	VipsObjectClass *object_class = VIPS_OBJECT_CLASS( save_class );
 	VipsForeignClass *class = VIPS_FOREIGN_CLASS( save_class );
 
-	/* The suffs might be defined on an abstract base class, make sure we
-	 * don't pick that.
+	/* All concrete savers needs suffs, since we use the suff to pick the 
+	 * saver.
 	 */
+	if( !G_TYPE_IS_ABSTRACT( G_TYPE_FROM_CLASS( class ) ) &&
+		!class->suffs )
+		g_warning( "no suffix defined for %s", object_class->nickname );
+
 	if( !G_TYPE_IS_ABSTRACT( G_TYPE_FROM_CLASS( class ) ) &&
 		class->suffs &&
+		!vips_ispostfix( object_class->nickname, "_buffer" ) &&
+		!vips_ispostfix( object_class->nickname, "_target" ) &&
 		vips_filename_suffix_match( filename, class->suffs ) )
 		return( save_class );
 
@@ -1791,16 +1916,82 @@ vips_foreign_save( VipsImage *in, const char *name, ... )
 	return( result );
 }
 
-/* Can we write this buffer with this file type?
+/* Can this class write this filetype to a target?
  */
 static void *
-vips_foreign_find_save_buffer_sub( VipsForeignSaveClass *save_class, 
-	const char *suffix )
+vips_foreign_find_save_target_sub( VipsForeignSaveClass *save_class, 
+	const char *suffix, void *b )
 {
 	VipsObjectClass *object_class = VIPS_OBJECT_CLASS( save_class );
 	VipsForeignClass *class = VIPS_FOREIGN_CLASS( save_class );
 
-	if( class->suffs &&
+	/* All concrete savers needs suffs, since we use the suff to pick the 
+	 * saver.
+	 */
+	if( !G_TYPE_IS_ABSTRACT( G_TYPE_FROM_CLASS( class ) ) &&
+		!class->suffs )
+		g_warning( "no suffix defined for %s", object_class->nickname );
+
+	if( !G_TYPE_IS_ABSTRACT( G_TYPE_FROM_CLASS( class ) ) &&
+		class->suffs &&
+		vips_ispostfix( object_class->nickname, "_target" ) &&
+		vips_filename_suffix_match( suffix, class->suffs ) )
+		return( save_class );
+
+	return( NULL );
+}
+
+/**
+ * vips_foreign_find_save_target:
+ * @suffix: format to find a saver for
+ *
+ * Searches for an operation you could use to write to a target in @suffix
+ * format. 
+ *
+ * See also: vips_image_write_to_buffer().
+ *
+ * Returns: the name of an operation on success, %NULL on error
+ */
+const char *
+vips_foreign_find_save_target( const char *name )
+{
+	char suffix[VIPS_PATH_MAX];
+	char option_string[VIPS_PATH_MAX];
+	VipsForeignSaveClass *save_class;
+
+	vips__filename_split8( name, suffix, option_string );
+
+	if( !(save_class = (VipsForeignSaveClass *) vips_foreign_map( 
+		"VipsForeignSave",
+		(VipsSListMap2Fn) vips_foreign_find_save_target_sub, 
+		(void *) suffix, NULL )) ) {
+		vips_error( "VipsForeignSave",
+			_( "\"%s\" is not a known target format" ), name );
+
+		return( NULL );
+	}
+
+	return( G_OBJECT_CLASS_NAME( save_class ) );
+}
+
+/* Can we write this buffer with this file type?
+ */
+static void *
+vips_foreign_find_save_buffer_sub( VipsForeignSaveClass *save_class, 
+	const char *suffix, void *b )
+{
+	VipsObjectClass *object_class = VIPS_OBJECT_CLASS( save_class );
+	VipsForeignClass *class = VIPS_FOREIGN_CLASS( save_class );
+
+	/* All concrete savers needs suffs, since we use the suff to pick the 
+	 * saver.
+	 */
+	if( !G_TYPE_IS_ABSTRACT( G_TYPE_FROM_CLASS( class ) ) &&
+		!class->suffs )
+		g_warning( "no suffix defined for %s", object_class->nickname );
+
+	if( !G_TYPE_IS_ABSTRACT( G_TYPE_FROM_CLASS( class ) ) &&
+		class->suffs &&
 		vips_ispostfix( object_class->nickname, "_buffer" ) &&
 		vips_filename_suffix_match( suffix, class->suffs ) )
 		return( save_class );
@@ -1847,74 +2038,115 @@ vips_foreign_find_save_buffer( const char *name )
 void
 vips_foreign_operation_init( void )
 {
-	extern GType vips_foreign_load_rad_get_type( void ); 
+	extern GType vips_foreign_load_rad_file_get_type( void ); 
+	extern GType vips_foreign_load_rad_buffer_get_type( void ); 
+	extern GType vips_foreign_load_rad_source_get_type( void ); 
 	extern GType vips_foreign_save_rad_file_get_type( void ); 
 	extern GType vips_foreign_save_rad_buffer_get_type( void ); 
+	extern GType vips_foreign_save_rad_target_get_type( void ); 
+
 	extern GType vips_foreign_load_mat_get_type( void ); 
-	extern GType vips_foreign_load_ppm_get_type( void ); 
-	extern GType vips_foreign_save_ppm_get_type( void ); 
-	extern GType vips_foreign_load_png_get_type( void ); 
+
+	extern GType vips_foreign_load_ppm_file_get_type( void ); 
+	extern GType vips_foreign_load_ppm_source_get_type( void ); 
+	extern GType vips_foreign_save_ppm_file_get_type( void ); 
+	extern GType vips_foreign_save_ppm_target_get_type( void ); 
+
+	extern GType vips_foreign_load_png_file_get_type( void ); 
 	extern GType vips_foreign_load_png_buffer_get_type( void ); 
+	extern GType vips_foreign_load_png_source_get_type( void ); 
 	extern GType vips_foreign_save_png_file_get_type( void ); 
 	extern GType vips_foreign_save_png_buffer_get_type( void ); 
-	extern GType vips_foreign_load_csv_get_type( void ); 
-	extern GType vips_foreign_save_csv_get_type( void ); 
-	extern GType vips_foreign_load_matrix_get_type( void ); 
-	extern GType vips_foreign_save_matrix_get_type( void ); 
+	extern GType vips_foreign_save_png_target_get_type( void ); 
+
+	extern GType vips_foreign_load_csv_file_get_type( void ); 
+	extern GType vips_foreign_load_csv_source_get_type( void ); 
+	extern GType vips_foreign_save_csv_file_get_type( void ); 
+	extern GType vips_foreign_save_csv_target_get_type( void ); 
+
+	extern GType vips_foreign_load_matrix_file_get_type( void ); 
+	extern GType vips_foreign_load_matrix_source_get_type( void ); 
+	extern GType vips_foreign_save_matrix_file_get_type( void ); 
+	extern GType vips_foreign_save_matrix_target_get_type( void ); 
 	extern GType vips_foreign_print_matrix_get_type( void ); 
+
 	extern GType vips_foreign_load_fits_get_type( void ); 
 	extern GType vips_foreign_save_fits_get_type( void ); 
+
 	extern GType vips_foreign_load_analyze_get_type( void ); 
+
 	extern GType vips_foreign_load_openexr_get_type( void ); 
+
 	extern GType vips_foreign_load_openslide_get_type( void ); 
+
 	extern GType vips_foreign_load_jpeg_file_get_type( void ); 
 	extern GType vips_foreign_load_jpeg_buffer_get_type( void ); 
+	extern GType vips_foreign_load_jpeg_source_get_type( void ); 
 	extern GType vips_foreign_save_jpeg_file_get_type( void ); 
 	extern GType vips_foreign_save_jpeg_buffer_get_type( void ); 
+	extern GType vips_foreign_save_jpeg_target_get_type( void ); 
 	extern GType vips_foreign_save_jpeg_mime_get_type( void ); 
+
 	extern GType vips_foreign_load_tiff_file_get_type( void ); 
 	extern GType vips_foreign_load_tiff_buffer_get_type( void ); 
+	extern GType vips_foreign_load_tiff_source_get_type( void ); 
 	extern GType vips_foreign_save_tiff_file_get_type( void ); 
 	extern GType vips_foreign_save_tiff_buffer_get_type( void ); 
+
 	extern GType vips_foreign_load_vips_get_type( void ); 
 	extern GType vips_foreign_save_vips_get_type( void ); 
+
 	extern GType vips_foreign_load_raw_get_type( void ); 
 	extern GType vips_foreign_save_raw_get_type( void ); 
 	extern GType vips_foreign_save_raw_fd_get_type( void ); 
+
 	extern GType vips_foreign_load_magick_file_get_type( void ); 
 	extern GType vips_foreign_load_magick_buffer_get_type( void ); 
 	extern GType vips_foreign_load_magick7_file_get_type( void ); 
 	extern GType vips_foreign_load_magick7_buffer_get_type( void ); 
 	extern GType vips_foreign_save_magick_file_get_type( void );
 	extern GType vips_foreign_save_magick_buffer_get_type( void );
+
 	extern GType vips_foreign_save_dz_file_get_type( void ); 
 	extern GType vips_foreign_save_dz_buffer_get_type( void ); 
+
 	extern GType vips_foreign_load_webp_file_get_type( void ); 
 	extern GType vips_foreign_load_webp_buffer_get_type( void ); 
+	extern GType vips_foreign_load_webp_source_get_type( void ); 
 	extern GType vips_foreign_save_webp_file_get_type( void ); 
 	extern GType vips_foreign_save_webp_buffer_get_type( void ); 
-	extern GType vips_foreign_load_pdf_get_type( void ); 
+	extern GType vips_foreign_save_webp_target_get_type( void ); 
+
 	extern GType vips_foreign_load_pdf_file_get_type( void ); 
 	extern GType vips_foreign_load_pdf_buffer_get_type( void ); 
-	extern GType vips_foreign_load_svg_get_type( void ); 
+	extern GType vips_foreign_load_pdf_source_get_type( void ); 
+
 	extern GType vips_foreign_load_svg_file_get_type( void ); 
 	extern GType vips_foreign_load_svg_buffer_get_type( void ); 
-	extern GType vips_foreign_load_heif_get_type( void ); 
+	extern GType vips_foreign_load_svg_source_get_type( void ); 
+
 	extern GType vips_foreign_load_heif_file_get_type( void ); 
 	extern GType vips_foreign_load_heif_buffer_get_type( void ); 
-	extern GType vips_foreign_save_heif_get_type( void ); 
+	extern GType vips_foreign_load_heif_source_get_type( void ); 
 	extern GType vips_foreign_save_heif_file_get_type( void ); 
 	extern GType vips_foreign_save_heif_buffer_get_type( void ); 
+	extern GType vips_foreign_save_heif_target_get_type( void ); 
+
 	extern GType vips_foreign_load_nifti_get_type( void ); 
 	extern GType vips_foreign_save_nifti_get_type( void ); 
-	extern GType vips_foreign_load_gif_get_type( void ); 
+
 	extern GType vips_foreign_load_gif_file_get_type( void ); 
 	extern GType vips_foreign_load_gif_buffer_get_type( void ); 
+	extern GType vips_foreign_load_gif_source_get_type( void ); 
 
-	vips_foreign_load_csv_get_type(); 
-	vips_foreign_save_csv_get_type(); 
-	vips_foreign_load_matrix_get_type(); 
-	vips_foreign_save_matrix_get_type(); 
+	vips_foreign_load_csv_file_get_type(); 
+	vips_foreign_load_csv_source_get_type(); 
+	vips_foreign_save_csv_file_get_type(); 
+	vips_foreign_save_csv_target_get_type(); 
+	vips_foreign_load_matrix_file_get_type(); 
+	vips_foreign_load_matrix_source_get_type(); 
+	vips_foreign_save_matrix_file_get_type(); 
+	vips_foreign_save_matrix_target_get_type(); 
 	vips_foreign_print_matrix_get_type(); 
 	vips_foreign_load_raw_get_type(); 
 	vips_foreign_save_raw_get_type(); 
@@ -1927,38 +2159,42 @@ vips_foreign_operation_init( void )
 #endif /*HAVE_ANALYZE*/
 
 #ifdef HAVE_PPM
-	vips_foreign_load_ppm_get_type(); 
-	vips_foreign_save_ppm_get_type(); 
+	vips_foreign_load_ppm_file_get_type(); 
+	vips_foreign_load_ppm_source_get_type(); 
+	vips_foreign_save_ppm_file_get_type(); 
+	vips_foreign_save_ppm_target_get_type(); 
 #endif /*HAVE_PPM*/
 
 #ifdef HAVE_RADIANCE
-	vips_foreign_load_rad_get_type(); 
+	vips_foreign_load_rad_file_get_type(); 
+	vips_foreign_load_rad_buffer_get_type(); 
+	vips_foreign_load_rad_source_get_type(); 
 	vips_foreign_save_rad_file_get_type(); 
 	vips_foreign_save_rad_buffer_get_type(); 
+	vips_foreign_save_rad_target_get_type(); 
 #endif /*HAVE_RADIANCE*/
 
-#ifdef HAVE_POPPLER
-	vips_foreign_load_pdf_get_type(); 
+#if defined(HAVE_POPPLER)
 	vips_foreign_load_pdf_file_get_type(); 
 	vips_foreign_load_pdf_buffer_get_type(); 
+	vips_foreign_load_pdf_source_get_type(); 
 #endif /*HAVE_POPPLER*/
 
 #ifdef HAVE_PDFIUM
-	vips_foreign_load_pdf_get_type(); 
 	vips_foreign_load_pdf_file_get_type(); 
 	vips_foreign_load_pdf_buffer_get_type(); 
 #endif /*HAVE_PDFIUM*/
 
 #ifdef HAVE_RSVG
-	vips_foreign_load_svg_get_type(); 
 	vips_foreign_load_svg_file_get_type(); 
 	vips_foreign_load_svg_buffer_get_type(); 
+	vips_foreign_load_svg_source_get_type(); 
 #endif /*HAVE_RSVG*/
 
 #ifdef HAVE_GIFLIB
-	vips_foreign_load_gif_get_type(); 
 	vips_foreign_load_gif_file_get_type(); 
 	vips_foreign_load_gif_buffer_get_type(); 
+	vips_foreign_load_gif_source_get_type(); 
 #endif /*HAVE_GIFLIB*/
 
 #ifdef HAVE_GSF
@@ -1967,12 +2203,20 @@ vips_foreign_operation_init( void )
 #endif /*HAVE_GSF*/
 
 #ifdef HAVE_PNG
-	vips_foreign_load_png_get_type(); 
+	vips_foreign_load_png_file_get_type(); 
 	vips_foreign_load_png_buffer_get_type(); 
+	vips_foreign_load_png_source_get_type(); 
 	vips_foreign_save_png_file_get_type(); 
 	vips_foreign_save_png_buffer_get_type(); 
+	vips_foreign_save_png_target_get_type(); 
 #endif /*HAVE_PNG*/
 
+#ifdef HAVE_SPNG
+	vips_foreign_load_png_file_get_type(); 
+	vips_foreign_load_png_buffer_get_type(); 
+	vips_foreign_load_png_source_get_type(); 
+#endif /*HAVE_SPNG*/
+
 #ifdef HAVE_MATIO
 	vips_foreign_load_mat_get_type(); 
 #endif /*HAVE_MATIO*/
@@ -1980,21 +2224,26 @@ vips_foreign_operation_init( void )
 #ifdef HAVE_JPEG
 	vips_foreign_load_jpeg_file_get_type(); 
 	vips_foreign_load_jpeg_buffer_get_type(); 
+	vips_foreign_load_jpeg_source_get_type(); 
 	vips_foreign_save_jpeg_file_get_type(); 
 	vips_foreign_save_jpeg_buffer_get_type(); 
+	vips_foreign_save_jpeg_target_get_type(); 
 	vips_foreign_save_jpeg_mime_get_type(); 
 #endif /*HAVE_JPEG*/
 
 #ifdef HAVE_LIBWEBP
 	vips_foreign_load_webp_file_get_type(); 
 	vips_foreign_load_webp_buffer_get_type(); 
+	vips_foreign_load_webp_source_get_type(); 
 	vips_foreign_save_webp_file_get_type(); 
 	vips_foreign_save_webp_buffer_get_type(); 
+	vips_foreign_save_webp_target_get_type(); 
 #endif /*HAVE_LIBWEBP*/
 
 #ifdef HAVE_TIFF
 	vips_foreign_load_tiff_file_get_type(); 
 	vips_foreign_load_tiff_buffer_get_type(); 
+	vips_foreign_load_tiff_source_get_type(); 
 	vips_foreign_save_tiff_file_get_type(); 
 	vips_foreign_save_tiff_buffer_get_type(); 
 #endif /*HAVE_TIFF*/
@@ -2035,15 +2284,15 @@ vips_foreign_operation_init( void )
 #endif /*HAVE_NIFTI*/
 
 #ifdef HAVE_HEIF_DECODER
-	vips_foreign_load_heif_get_type(); 
 	vips_foreign_load_heif_file_get_type(); 
 	vips_foreign_load_heif_buffer_get_type(); 
+	vips_foreign_load_heif_source_get_type(); 
 #endif /*HAVE_HEIF_DECODER*/
 
 #ifdef HAVE_HEIF_ENCODER
-	vips_foreign_save_heif_get_type(); 
 	vips_foreign_save_heif_file_get_type(); 
 	vips_foreign_save_heif_buffer_get_type(); 
+	vips_foreign_save_heif_target_get_type(); 
 #endif /*HAVE_HEIF_ENCODER*/
 
 	vips__foreign_load_operation = 
