@@ -4,6 +4,22 @@
  * 	- from niftiload.c
  * 24/7/19 [zhoux2016]
  * 	- always fetch metadata from the main image (thumbs don't have it)
+ * 24/7/19
+ * 	- close early on minimise 
+ * 	- close early on error
+ * 1/9/19 [meyermarcel]
+ * 	- handle alpha
+ * 30/9/19
+ * 	- much faster handling of thumbnail=TRUE and missing thumbnail ... we
+ * 	  were reselecting the image for each scanline
+ * 3/10/19
+ * 	- restart after minimise
+ * 15/3/20
+ * 	- revise for new VipsSource API
+ * 10/5/20
+ * 	- deprecate autorotate -- it's too difficult to support properly
+ * 31/7/20
+ * 	- block broken thumbnails, if we can
  */
 
 /*
@@ -52,12 +68,48 @@
 #include <vips/debug.h>
 #include <vips/internal.h>
 
-#ifdef HAVE_HEIF_DECODER
+/* These are shared with the encoder.
+ */
+#if defined(HAVE_HEIF_DECODER) || defined(HAVE_HEIF_ENCODER)
 
 #include <libheif/heif.h>
 
 #include "pforeign.h"
 
+void
+vips__heif_error( struct heif_error *error )
+{
+	if( error->code ) 
+		vips_error( "heif", "%s (%d.%d)", error->message, error->code,
+			error->subcode );
+}
+
+const char *vips__heif_suffs[] = { 
+	".heic",
+	".heif",
+	".avif",
+	NULL 
+};
+
+#endif /*defined(DECODE) || defined(ENCODE)*/
+
+#ifdef HAVE_HEIF_DECODER
+
+#define VIPS_TYPE_FOREIGN_LOAD_HEIF (vips_foreign_load_heif_get_type())
+#define VIPS_FOREIGN_LOAD_HEIF( obj ) \
+	(G_TYPE_CHECK_INSTANCE_CAST( (obj), \
+	VIPS_TYPE_FOREIGN_LOAD_HEIF, VipsForeignLoadHeif ))
+#define VIPS_FOREIGN_LOAD_HEIF_CLASS( klass ) \
+	(G_TYPE_CHECK_CLASS_CAST( (klass), \
+	VIPS_TYPE_FOREIGN_LOAD_HEIF, VipsForeignLoadHeifClass))
+#define VIPS_IS_FOREIGN_LOAD_HEIF( obj ) \
+	(G_TYPE_CHECK_INSTANCE_TYPE( (obj), VIPS_TYPE_FOREIGN_LOAD_HEIF ))
+#define VIPS_IS_FOREIGN_LOAD_HEIF_CLASS( klass ) \
+	(G_TYPE_CHECK_CLASS_TYPE( (klass), VIPS_TYPE_FOREIGN_LOAD_HEIF ))
+#define VIPS_FOREIGN_LOAD_HEIF_GET_CLASS( obj ) \
+	(G_TYPE_INSTANCE_GET_CLASS( (obj), \
+	VIPS_TYPE_FOREIGN_LOAD_HEIF, VipsForeignLoadHeifClass ))
+
 typedef struct _VipsForeignLoadHeif {
 	VipsForeignLoad parent_object;
 
@@ -72,6 +124,11 @@ typedef struct _VipsForeignLoadHeif {
 	gboolean thumbnail;
 
 	/* Apply any orientation tags in the header.
+	 *
+	 * This is deprecated and does nothing. Non-autorotated reads from
+	 * libheif are surprisingly hard to support well, since orientation can
+	 * be represented in several different ways in HEIC files and devices
+	 * vary in how they do this.
 	 */
 	gboolean autorotate;
 
@@ -83,6 +140,10 @@ typedef struct _VipsForeignLoadHeif {
 	 */
 	int n_top;
 
+	/* TRUE for RGBA ... otherwise, RGB.
+	 */
+	gboolean has_alpha;
+
 	/* Size of final output image. 
 	 */
 	int width;
@@ -123,9 +184,22 @@ typedef struct _VipsForeignLoadHeif {
 	int stride;
 	const uint8_t *data;
 
+	/* Set from subclasses.
+	 */
+	VipsSource *source;
+
+	/* The reader struct. We use this to attach to our VipsSource. This
+	 * has to be alloced rather than in our struct, since it may change
+	 * size in libheif API versions.
+	 */
+	struct heif_reader *reader;
+
 } VipsForeignLoadHeif;
 
-typedef VipsForeignLoadClass VipsForeignLoadHeifClass;
+typedef struct _VipsForeignLoadHeifClass {
+	VipsForeignLoadClass parent_class;
+
+} VipsForeignLoadHeifClass;
 
 G_DEFINE_ABSTRACT_TYPE( VipsForeignLoadHeif, vips_foreign_load_heif, 
 	VIPS_TYPE_FOREIGN_LOAD );
@@ -135,22 +209,51 @@ vips_foreign_load_heif_dispose( GObject *gobject )
 {
 	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) gobject;
 
+	heif->data = NULL;
 	VIPS_FREEF( heif_image_release, heif->img );
 	VIPS_FREEF( heif_image_handle_release, heif->handle );
 	VIPS_FREEF( heif_context_free, heif->ctx );
 	VIPS_FREE( heif->id );
+	VIPS_FREE( heif->reader );
+	VIPS_UNREF( heif->source );
 
 	G_OBJECT_CLASS( vips_foreign_load_heif_parent_class )->
 		dispose( gobject );
 }
 
-void
-vips__heif_error( struct heif_error *error )
+static int
+vips_foreign_load_heif_build( VipsObject *object )
 {
-	if( error->code ) 
-		vips_error( "heifload", "%s", error->message ); 
+	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) object;
+
+#ifdef DEBUG
+	printf( "vips_foreign_load_heif_build:\n" );
+#endif /*DEBUG*/
+
+	if( vips_source_rewind( heif->source ) )
+		return( -1 );
+
+	if( !heif->ctx ) {
+		struct heif_error error;
+
+		heif->ctx = heif_context_alloc();
+		error = heif_context_read_from_reader( heif->ctx, 
+			heif->reader, heif, NULL );
+		if( error.code ) {
+			vips__heif_error( &error );
+			return( -1 );
+		}
+	}
+
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_load_heif_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	return( 0 );
 }
 
+
 static const char *heif_magic[] = {
 	"ftypheic",	/* A regular heif image */
 	"ftypheix",	/* Extended range (>8 bit) image */
@@ -160,22 +263,32 @@ static const char *heif_magic[] = {
 	"ftyphevm",	/* Multiview sequence */
 	"ftyphevs",	/* Scaleable sequence */
 	"ftypmif1",	/* Nokia alpha_ image */
-	"ftypmsf1"	/* Nokia animation image */
+	"ftypmsf1",	/* Nokia animation image */
+	"ftypavif"	/* AV1 image format */
 };
 
-/* THe API has:
+/* The API has:
  *
  *	enum heif_filetype_result result = heif_check_filetype( buf, 12 );
  *
- * but it's very conservative and seems to be missing some of the Noka hief
+ * but it's very conservative and seems to be missing some of the Nokia heif
  * types.
  */
 static int
 vips_foreign_load_heif_is_a( const char *buf, int len )
 {
 	if( len >= 12 ) {
+		const guint chunk_len = GUINT_FROM_BE( *((guint32 *) buf) );
+
 		int i;
 
+                /* We've seen real files with 36 here, so 64 should be
+                 * plenty.
+                 */
+		if( chunk_len > 64 || 
+			chunk_len % 4 != 0 )
+			return( 0 );
+
 		for( i = 0; i < VIPS_NUMBER( heif_magic ); i++ )
 			if( strncmp( buf + 4, heif_magic[i], 8 ) == 0 )
 				return( 1 );
@@ -192,6 +305,75 @@ vips_foreign_load_heif_get_flags( VipsForeignLoad *load )
 	return( VIPS_FOREIGN_SEQUENTIAL );
 }
 
+/* We've selected the page. Try to select the associated thumbnail instead, 
+ * if we can.
+ */
+static int
+vips_foreign_load_heif_set_thumbnail( VipsForeignLoadHeif *heif )
+{
+	heif_item_id thumb_ids[1];
+	int n_thumbs;
+	struct heif_image_handle *thumb_handle;
+	struct heif_image *thumb_img;
+	struct heif_error error;
+	double main_aspect;
+	double thumb_aspect;
+
+#ifdef DEBUG
+	printf( "vips_foreign_load_heif_set_thumbnail:\n" );
+#endif /*DEBUG*/
+
+	n_thumbs = heif_image_handle_get_list_of_thumbnail_IDs( 
+		heif->handle, thumb_ids, 1 );
+	if( n_thumbs == 0 )
+		return( 0 );
+
+	error = heif_image_handle_get_thumbnail( heif->handle,
+		thumb_ids[0], &thumb_handle );
+	if( error.code ) {
+		vips__heif_error( &error );
+		return( -1 );
+	}
+
+	/* Just checking the width and height of the handle isn't
+	 * enough -- we have to experimentally decode it and test the 
+	 * decoded dimensions. 
+	 */
+	error = heif_decode_image( thumb_handle, &thumb_img,
+		heif_colorspace_RGB, 
+		heif_chroma_interleaved_RGB,
+		NULL );
+	if( error.code ) {
+		VIPS_FREEF( heif_image_handle_release, thumb_handle );
+		vips__heif_error( &error );
+		return( -1 );
+	}
+
+	thumb_aspect = (double) 
+		heif_image_get_width( thumb_img, heif_channel_interleaved ) /
+		heif_image_get_height( thumb_img, heif_channel_interleaved );
+
+	VIPS_FREEF( heif_image_release, thumb_img );
+
+	main_aspect = (double) 
+		heif_image_handle_get_width( heif->handle ) /
+		heif_image_handle_get_height( heif->handle );
+
+	/* The bug we are working around has decoded thumbs as 512x512 
+	 * with the main image as 6kx4k, so a 0.1 threshold is more 
+	 * than tight enough to spot the error.
+	 */
+	if( fabs( main_aspect - thumb_aspect ) > 0.1 ) {
+		VIPS_FREEF( heif_image_handle_release, thumb_handle );
+		return( 0 );
+	}
+
+	VIPS_FREEF( heif_image_handle_release, heif->handle );
+	heif->handle = thumb_handle;
+
+	return( 0 );
+}
+
 /* Select a page. If thumbnail is set, select the thumbnail for that page, if
  * there is one.
  */
@@ -204,6 +386,11 @@ vips_foreign_load_heif_set_page( VipsForeignLoadHeif *heif,
 		thumbnail != heif->thumbnail_set ) {
 		struct heif_error error;
 
+#ifdef DEBUG
+		printf( "vips_foreign_load_heif_set_page: %d, thumbnail = %d\n",
+			page_no, thumbnail );
+#endif /*DEBUG*/
+
 		VIPS_FREEF( heif_image_handle_release, heif->handle );
 		VIPS_FREEF( heif_image_release, heif->img );
 		heif->data = NULL;
@@ -217,27 +404,18 @@ vips_foreign_load_heif_set_page( VipsForeignLoadHeif *heif,
 		}
 
 		if( thumbnail ) {
-			heif_item_id thumb_ids[1];
-			int n_thumbs;
-			struct heif_image_handle *thumb_handle;
+			if( vips_foreign_load_heif_set_thumbnail( heif ) )
+				return( -1 );
 
-			n_thumbs = heif_image_handle_get_list_of_thumbnail_IDs( 
-				heif->handle, thumb_ids, 1 );
-
-			if( n_thumbs > 0 ) {
-				error = heif_image_handle_get_thumbnail( 
-					heif->handle,
-					thumb_ids[0], &thumb_handle );
-				if( error.code ) {
-					vips__heif_error( &error );
-					return( -1 );
-				}
-
-				VIPS_FREEF( heif_image_handle_release, 
-					heif->handle );
-				heif->handle = thumb_handle;
-				heif->thumbnail_set = TRUE;
-			}
+			/* If we were asked to select the thumbnail, say we
+			 * did, even if there are no thumbnails and we just
+			 * selected the main image. 
+			 *
+			 * If we don't do this, next time around in _generate
+			 * we'll try to select the thumbnail again, which will
+			 * be horribly slow.
+			 */
+			heif->thumbnail_set = TRUE;
 		}
 
 		heif->page_no = page_no;
@@ -249,7 +427,8 @@ vips_foreign_load_heif_set_page( VipsForeignLoadHeif *heif,
 static int
 vips_foreign_load_heif_set_header( VipsForeignLoadHeif *heif, VipsImage *out )
 {
-	gboolean has_alpha;
+	VipsForeignLoad *load = (VipsForeignLoad *) heif;
+
 	int bands;
 	int i;
 	/* Surely, 16 metadata items will be enough for anyone.
@@ -264,12 +443,14 @@ vips_foreign_load_heif_set_header( VipsForeignLoadHeif *heif, VipsImage *out )
 	if( vips_foreign_load_heif_set_page( heif, heif->page, FALSE ) )
 		return( -1 );
 
-	/* FIXME ... never seen this return TRUE on any image, strangely.
-	 */
-	has_alpha = heif_image_handle_has_alpha_channel( heif->handle );
-	bands = has_alpha ? 4 : 3;
+	heif->has_alpha = heif_image_handle_has_alpha_channel( heif->handle );
+#ifdef DEBUG
+	printf( "heif_image_handle_has_alpha_channel() = %d\n", 
+		heif->has_alpha );
+#endif /*DEBUG*/
+	bands = heif->has_alpha ? 4 : 3;
 
-	/* FIXME .. need to test XMP and IPCT.
+	/* FIXME .. IPTC as well?
 	 */
 	n_metadata = heif_image_handle_get_list_of_metadata_block_IDs( 
 		heif->handle, NULL, id, VIPS_NUMBER( id ) );
@@ -283,9 +464,11 @@ vips_foreign_load_heif_set_header( VipsForeignLoadHeif *heif, VipsImage *out )
 		char name[256];
 
 #ifdef DEBUG
-		printf( "metadata type = %s, length = %zd\n", type, length ); 
+		printf( "metadata type = %s, length = %zu\n", type, length ); 
 #endif /*DEBUG*/
 
+		if( !length )
+			continue;
 		if( !(data = VIPS_ARRAY( out, length, unsigned char )) )
 			return( -1 );
 		error = heif_image_handle_get_metadata( 
@@ -298,7 +481,8 @@ vips_foreign_load_heif_set_header( VipsForeignLoadHeif *heif, VipsImage *out )
 		/* We need to skip the first four bytes of EXIF, they just
 		 * contain the offset.
 		 */
-		if( g_ascii_strcasecmp( type, "exif" ) == 0 ) {
+		if( length > 4 &&
+			g_ascii_strcasecmp( type, "exif" ) == 0 ) {
 			data += 4;
 			length -= 4;
 		}
@@ -311,24 +495,35 @@ vips_foreign_load_heif_set_header( VipsForeignLoadHeif *heif, VipsImage *out )
 		if( g_ascii_strcasecmp( type, "exif" ) == 0 )
 			vips_snprintf( name, 256, VIPS_META_EXIF_NAME );
 		else if( g_ascii_strcasecmp( type, "mime" ) == 0 &&
+			length > 10 &&
 			vips_isprefix( "<x:xmpmeta", (const char *) data ) ) 
-			snprintf( name, 256, VIPS_META_XMP_NAME ); 
+			vips_snprintf( name, 256, VIPS_META_XMP_NAME );
 		else
 			vips_snprintf( name, 256, "heif-%s-%d", type, i );
 
 		vips_image_set_blob( out, name, 
 			(VipsCallbackFn) NULL, data, length );
 
-		if( g_ascii_strcasecmp( type, "exif" ) == 0 ) 
-			(void) vips__exif_parse( out );
+		/* image_set will automatically parse EXIF, if necessary.
+		 */
 	}
 
+	/* We use libheif's autorotate, so we need to remove any EXIF
+	 * orientaion tags.
+	 *
+	 * According to the HEIF standard, EXIF orientation tags are only
+	 * informational and images should not be rotated because of them.
+	 * Unless we strip these tags, there's a danger downstream processing
+	 * could double-rotate.
+	 */
+	vips_autorot_remove_angle( out );
+
 #ifdef HAVE_HEIF_COLOR_PROFILE
-#ifdef DEBUG
-{
 	enum heif_color_profile_type profile_type = 
 		heif_image_handle_get_color_profile_type( heif->handle );
 
+#ifdef DEBUG
+{
 	printf( "profile type = " ); 
 	switch( profile_type ) {
 	case heif_color_profile_type_not_present: 
@@ -355,10 +550,10 @@ vips_foreign_load_heif_set_header( VipsForeignLoadHeif *heif, VipsImage *out )
 }
 #endif /*DEBUG*/
 
-	/* FIXME should probably check the profile type ... lcms seems to be
-	 * able to load at least rICC and prof.
+	/* lcms can load standard (prof) and reduced (rICC) profiles
 	 */
-	if( heif_image_handle_get_color_profile_type( heif->handle ) ) {
+	if( profile_type == heif_color_profile_type_prof ||
+		profile_type == heif_color_profile_type_rICC ) {
 		size_t length = heif_image_handle_get_raw_color_profile_size( 
 			heif->handle );
 
@@ -380,15 +575,11 @@ vips_foreign_load_heif_set_header( VipsForeignLoadHeif *heif, VipsImage *out )
 		vips_image_set_blob( out, VIPS_META_ICC_NAME, 
 			(VipsCallbackFn) NULL, data, length );
 	}
+	else if( profile_type == heif_color_profile_type_nclx ) {
+		g_warning( "heifload: ignoring nclx profile" );
+	}
 #endif /*HAVE_HEIF_COLOR_PROFILE*/
 
-	/* If we are using libheif's autorotate, remove the exif one. 
-	 */
-#ifdef HAVE_HEIF_IMAGE_HANDLE_GET_ISPE_WIDTH
-	if( heif->autorotate )
-		vips_autorot_remove_angle( out );
-#endif /*HAVE_HEIF_IMAGE_HANDLE_GET_ISPE_WIDTH*/
-
 	vips_image_set_int( out, "heif-primary", heif->primary_page );
 	vips_image_set_int( out, "n-pages", heif->n_top );
 	if( vips_object_argument_isset( VIPS_OBJECT( heif ), "n" ) )
@@ -404,43 +595,12 @@ vips_foreign_load_heif_set_header( VipsForeignLoadHeif *heif, VipsImage *out )
 		VIPS_FORMAT_UCHAR, VIPS_CODING_NONE, VIPS_INTERPRETATION_sRGB, 
 		1.0, 1.0 );
 
+	VIPS_SETSTR( load->out->filename, 
+		vips_connection_filename( VIPS_CONNECTION( heif->source ) ) );
+
 	return( 0 );
 }
 
-static int
-vips_foreign_load_heif_get_width( VipsForeignLoadHeif *heif, 
-	struct heif_image_handle *handle )
-{
-	int width;
-
-	/* _get_ipse_width() fetches the untransformed dimension, but was only
-	 * added in 1.3.4. Without it, we just use the transformed dimension
-	 * and have to autorotate.
-	 */
-	width = heif_image_handle_get_width( handle );
-#ifdef HAVE_HEIF_IMAGE_HANDLE_GET_ISPE_WIDTH
-	if( !heif->autorotate ) 
-		width = heif_image_handle_get_ispe_width( handle );
-#endif /*HAVE_HEIF_IMAGE_HANDLE_GET_ISPE_WIDTH*/
-
-	return( width );
-}
-
-static int
-vips_foreign_load_heif_get_height( VipsForeignLoadHeif *heif,
-	struct heif_image_handle *handle )
-{
-	int height;
-
-	height = heif_image_handle_get_height( handle );
-#ifdef HAVE_HEIF_IMAGE_HANDLE_GET_ISPE_WIDTH
-	if( !heif->autorotate )
-		height = heif_image_handle_get_ispe_height( handle );
-#endif /*HAVE_HEIF_IMAGE_HANDLE_GET_ISPE_WIDTH*/
-
-	return( height );
-}
-
 static int
 vips_foreign_load_heif_header( VipsForeignLoad *load )
 {
@@ -451,6 +611,10 @@ vips_foreign_load_heif_header( VipsForeignLoad *load )
 	heif_item_id primary_id;
 	int i;
 
+#ifdef DEBUG
+	printf( "vips_foreign_load_heif_header:\n" );
+#endif /*DEBUG*/
+
 	heif->n_top = heif_context_get_number_of_top_level_images( heif->ctx );
 	heif->id = VIPS_ARRAY( NULL, heif->n_top, heif_item_id );
 	heif_context_get_list_of_top_level_image_IDs( heif->ctx, 
@@ -484,10 +648,6 @@ vips_foreign_load_heif_header( VipsForeignLoad *load )
 	}
 
 #ifdef DEBUG
-#ifdef HAVE_HEIF_IMAGE_HANDLE_GET_ISPE_WIDTH
-	if( !heif->autorotate )
-		printf( "using _get_ispe_width() / _height()\n" );
-#endif /*HAVE_HEIF_IMAGE_HANDLE_GET_ISPE_WIDTH*/
 	for( i = heif->page; i < heif->page + heif->n; i++ ) {
 		heif_item_id thumb_ids[1];
 		int n_items;
@@ -518,11 +678,9 @@ vips_foreign_load_heif_header( VipsForeignLoad *load )
 
 			printf( "  thumb %d\n", j );
 			printf( "    width = %d\n", 
-				vips_foreign_load_heif_get_width( heif, 
-					thumb_handle ) );
+				heif_image_handle_get_width( thumb_handle ) );
 			printf( "    height = %d\n", 
-				vips_foreign_load_heif_get_height( heif, 
-					thumb_handle ) );
+				heif_image_handle_get_height( thumb_handle ) );
 		}
 	}
 #endif /*DEBUG*/
@@ -532,18 +690,16 @@ vips_foreign_load_heif_header( VipsForeignLoad *load )
 	if( vips_foreign_load_heif_set_page( heif, 
 		heif->page, heif->thumbnail ) )
 		return( -1 );
-	heif->page_width = vips_foreign_load_heif_get_width( heif, 
-		heif->handle );
-	heif->page_height = vips_foreign_load_heif_get_height( heif, 
-		heif->handle );
+	heif->page_width = heif_image_handle_get_width( heif->handle );
+	heif->page_height = heif_image_handle_get_height( heif->handle );
 	for( i = heif->page + 1; i < heif->page + heif->n; i++ ) {
 		if( vips_foreign_load_heif_set_page( heif, 
 			i, heif->thumbnail ) )
 			return( -1 );
-		if( vips_foreign_load_heif_get_width( heif, 
-				heif->handle ) != heif->page_width ||
-			vips_foreign_load_heif_get_height( heif, 
-				heif->handle ) != heif->page_height ) {
+		if( heif_image_handle_get_width( heif->handle ) 
+				!= heif->page_width ||
+			heif_image_handle_get_height( heif->handle ) 
+				!= heif->page_height ) {
 			vips_error( class->nickname, "%s", 
 				_( "not all pages are the same size" ) ); 
 			return( -1 ); 
@@ -557,11 +713,9 @@ vips_foreign_load_heif_header( VipsForeignLoad *load )
 		if( vips_foreign_load_heif_set_page( heif, i, FALSE ) )
 			return( -1 );
 		printf( "    width = %d\n", 
-			vips_foreign_load_heif_get_width( heif, 
-				heif->handle ) );
+			heif_image_handle_get_width( heif->handle ) );
 		printf( "    height = %d\n", 
-			vips_foreign_load_heif_get_height( heif, 
-				heif->handle ) );
+			heif_image_handle_get_height( heif->handle ) );
 		printf( "    has_depth = %d\n", 
 			heif_image_handle_has_depth_image( heif->handle ) );
 		printf( "    has_alpha = %d\n", 
@@ -580,6 +734,8 @@ vips_foreign_load_heif_header( VipsForeignLoad *load )
 	if( vips_foreign_load_heif_set_header( heif, load->out ) )
 		return( -1 );
 
+	vips_source_minimise( heif->source );
+
 	return( 0 );
 }
 
@@ -606,21 +762,18 @@ vips_foreign_load_heif_generate( VipsRegion *or,
 	if( !heif->img ) {
 		struct heif_error error;
 		struct heif_decoding_options *options;
+		enum heif_chroma chroma = heif->has_alpha ? 
+			heif_chroma_interleaved_RGBA :
+			heif_chroma_interleaved_RGB;
 
-		/* Decode the image to 24bit interleaved. 
-		 *
-		 * FIXME What will this do for RGBA? Or is alpha always 
-		 * separate?
-		 *
-		 * Only disable transforms if we have been able to fetch the
-		 * untransformed dimensions.
-		 */
 		options = heif_decoding_options_alloc();
-#ifdef HAVE_HEIF_IMAGE_HANDLE_GET_ISPE_WIDTH
-		options->ignore_transformations = !heif->autorotate;
-#endif /*HAVE_HEIF_IMAGE_HANDLE_GET_ISPE_WIDTH*/
+#ifdef HAVE_HEIF_DECODING_OPTIONS_CONVERT_HDR_TO_8BIT
+		/* VIPS_FORMAT_UCHAR is assumed so downsample HDR to 8bpc
+		 */
+		options->convert_hdr_to_8bit = TRUE;
+#endif /*HAVE_HEIF_DECODING_OPTIONS_CONVERT_HDR_TO_8BIT*/
 		error = heif_decode_image( heif->handle, &heif->img, 
-			heif_colorspace_RGB, heif_chroma_interleaved_RGB, 
+			heif_colorspace_RGB, chroma, 
 			options );
 		heif_decoding_options_free( options );
 		if( error.code ) {
@@ -706,6 +859,12 @@ vips_foreign_load_heif_generate( VipsRegion *or,
 	return( 0 );
 }
 
+static void
+vips_foreign_load_heif_minimise( VipsObject *object, VipsForeignLoadHeif *heif )
+{
+	vips_source_minimise( heif->source );
+}
+
 static int
 vips_foreign_load_heif_load( VipsForeignLoad *load )
 {
@@ -721,12 +880,21 @@ vips_foreign_load_heif_load( VipsForeignLoad *load )
 	t[0] = vips_image_new();
 	if( vips_foreign_load_heif_set_header( heif, t[0] ) )
 		return( -1 );
+
+	/* CLose input immediately at end of read.
+	 */
+	g_signal_connect( t[0], "minimise", 
+		G_CALLBACK( vips_foreign_load_heif_minimise ), heif ); 
+
 	if( vips_image_generate( t[0],
 		NULL, vips_foreign_load_heif_generate, NULL, heif, NULL ) ||
 		vips_sequential( t[0], &t[1], NULL ) ||
 		vips_image_write( t[1], load->real ) )
 		return( -1 );
 
+	if( vips_source_decode( heif->source ) )
+		return( -1 );
+
 	return( 0 );
 }
 
@@ -743,6 +911,7 @@ vips_foreign_load_heif_class_init( VipsForeignLoadHeifClass *class )
 
 	object_class->nickname = "heifload_base";
 	object_class->description = _( "load a HEIF image" );
+	object_class->build = vips_foreign_load_heif_build;
 
 	load_class->get_flags = vips_foreign_load_heif_get_flags;
 	load_class->header = vips_foreign_load_heif_header;
@@ -772,17 +941,100 @@ vips_foreign_load_heif_class_init( VipsForeignLoadHeifClass *class )
 	VIPS_ARG_BOOL( class, "autorotate", 21, 
 		_( "Autorotate" ), 
 		_( "Rotate image using exif orientation" ),
-		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
 		G_STRUCT_OFFSET( VipsForeignLoadHeif, autorotate ),
 		FALSE );
 
 }
 
+static gint64
+vips_foreign_load_heif_get_position( void *userdata )
+{
+	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) userdata;
+
+	return( vips_source_seek( heif->source, 0L, SEEK_CUR ) );
+}
+
+/* libheif read() does not work like unix read(). 
+ *
+ * This method is cannot return EOF. Instead, the separate wait_for_file_size() 
+ * is called beforehand to make sure that there's enough data there.
+ */
+static int
+vips_foreign_load_heif_read( void *data, size_t size, void *userdata )
+{
+	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) userdata;
+
+	while( size > 0 ) {
+		gint64 bytes_read;
+
+		bytes_read = vips_source_read( heif->source, data, size );
+		if( bytes_read <= 0 ) 
+			return( -1 );
+
+		size -= bytes_read;
+		data += bytes_read;
+	}
+
+	return( 0 );
+}
+
+static int
+vips_foreign_load_heif_seek( gint64 position, void *userdata )
+{
+	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) userdata;
+
+	/* Return 0 on success.
+	 */
+	return( vips_source_seek( heif->source, position, SEEK_SET ) == -1 );
+}
+
+/* libheif calls this to mean "I intend to read() to this position, please
+ * check it is OK".
+ */
+static enum heif_reader_grow_status 
+vips_foreign_load_heif_wait_for_file_size( gint64 target_size, void *userdata )
+{
+	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) userdata;
+
+	gint64 old_position;
+	gint64 result;
+	enum heif_reader_grow_status status;
+
+	/* We seek the VipsSource to the position and check for errors. 
+	 */
+	old_position = vips_source_seek( heif->source, 0L, SEEK_CUR );
+	result = vips_source_seek( heif->source, target_size, SEEK_SET );
+	vips_source_seek( heif->source, old_position, SEEK_SET );
+
+	if( result < 0 )
+		/* Unable to seek to this point, so it's beyond EOF.
+		 */
+		status = heif_reader_grow_status_size_beyond_eof;
+	else
+		/* Successfully read to the requested point, but the requested
+		 * point is not necessarily EOF.
+		 */
+		status = heif_reader_grow_status_size_reached;
+
+	return( status );
+}
+
 static void
 vips_foreign_load_heif_init( VipsForeignLoadHeif *heif )
 {
-	heif->ctx = heif_context_alloc();
 	heif->n = 1;
+
+	heif->reader = VIPS_ARRAY( NULL, 1, struct heif_reader );
+
+	/* The first version to support heif_reader.
+	 */
+	heif->reader->reader_api_version = 1;
+	heif->reader->get_position = vips_foreign_load_heif_get_position;
+	heif->reader->read = vips_foreign_load_heif_read;
+	heif->reader->seek = vips_foreign_load_heif_seek;
+	heif->reader->wait_for_file_size = 
+		vips_foreign_load_heif_wait_for_file_size;
 }
 
 typedef struct _VipsForeignLoadHeifFile {
@@ -799,6 +1051,24 @@ typedef VipsForeignLoadHeifClass VipsForeignLoadHeifFileClass;
 G_DEFINE_TYPE( VipsForeignLoadHeifFile, vips_foreign_load_heif_file, 
 	vips_foreign_load_heif_get_type() );
 
+static int
+vips_foreign_load_heif_file_build( VipsObject *object )
+{
+	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) object;
+	VipsForeignLoadHeifFile *file = (VipsForeignLoadHeifFile *) object;
+
+	if( file->filename ) 
+		if( !(heif->source = 
+			vips_source_new_from_file( file->filename )) )
+			return( -1 );
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_load_heif_file_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	return( 0 );
+}
+
 static int
 vips_foreign_load_heif_file_is_a( const char *filename )
 {
@@ -810,34 +1080,6 @@ vips_foreign_load_heif_file_is_a( const char *filename )
 	return( vips_foreign_load_heif_is_a( buf, 12 ) );
 }
 
-static int
-vips_foreign_load_heif_file_header( VipsForeignLoad *load )
-{
-	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) load;
-	VipsForeignLoadHeifFile *file = (VipsForeignLoadHeifFile *) load;
-
-	struct heif_error error;
-
-	error = heif_context_read_from_file( heif->ctx, file->filename, NULL );
-	if( error.code ) {
-		vips__heif_error( &error );
-		return( -1 );
-	}
-
-	if( VIPS_FOREIGN_LOAD_CLASS( 
-		vips_foreign_load_heif_file_parent_class )->header( load ) )
-		return( -1 );
-
-	VIPS_SETSTR( load->out->filename, file->filename );
-
-	return( 0 );
-}
-
-const char *vips__heif_suffs[] = { 
-	".heic",
-	NULL 
-};
-
 static void
 vips_foreign_load_heif_file_class_init( VipsForeignLoadHeifFileClass *class )
 {
@@ -850,11 +1092,11 @@ vips_foreign_load_heif_file_class_init( VipsForeignLoadHeifFileClass *class )
 	gobject_class->get_property = vips_object_get_property;
 
 	object_class->nickname = "heifload";
+	object_class->build = vips_foreign_load_heif_file_build;
 
 	foreign_class->suffs = vips__heif_suffs;
 
 	load_class->is_a = vips_foreign_load_heif_file_is_a;
-	load_class->header = vips_foreign_load_heif_file_header;
 
 	VIPS_ARG_STRING( class, "filename", 1, 
 		_( "Filename" ),
@@ -884,34 +1126,32 @@ typedef VipsForeignLoadHeifClass VipsForeignLoadHeifBufferClass;
 G_DEFINE_TYPE( VipsForeignLoadHeifBuffer, vips_foreign_load_heif_buffer, 
 	vips_foreign_load_heif_get_type() );
 
+static int
+vips_foreign_load_heif_buffer_build( VipsObject *object )
+{
+	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) object;
+	VipsForeignLoadHeifBuffer *buffer = 
+		(VipsForeignLoadHeifBuffer *) object;
+
+	if( buffer->buf )
+		if( !(heif->source = vips_source_new_from_memory( 
+			VIPS_AREA( buffer->buf )->data, 
+			VIPS_AREA( buffer->buf )->length )) )
+			return( -1 );
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_load_heif_file_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	return( 0 );
+}
+
 static gboolean
 vips_foreign_load_heif_buffer_is_a( const void *buf, size_t len )
 {
 	return( vips_foreign_load_heif_is_a( buf, len ) );
 }
 
-static int
-vips_foreign_load_heif_buffer_header( VipsForeignLoad *load )
-{
-	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) load;
-	VipsForeignLoadHeifBuffer *buffer = (VipsForeignLoadHeifBuffer *) load;
-
-	struct heif_error error;
-
-	error = heif_context_read_from_memory( heif->ctx, 
-		buffer->buf->data, buffer->buf->length, NULL );
-	if( error.code ) {
-		vips__heif_error( &error );
-		return( -1 );
-	}
-
-	if( VIPS_FOREIGN_LOAD_CLASS( 
-		vips_foreign_load_heif_buffer_parent_class )->header( load ) )
-		return( -1 );
-
-	return( 0 );
-}
-
 static void
 vips_foreign_load_heif_buffer_class_init( 
 	VipsForeignLoadHeifBufferClass *class )
@@ -924,9 +1164,9 @@ vips_foreign_load_heif_buffer_class_init(
 	gobject_class->get_property = vips_object_get_property;
 
 	object_class->nickname = "heifload_buffer";
+	object_class->build = vips_foreign_load_heif_buffer_build;
 
 	load_class->is_a_buffer = vips_foreign_load_heif_buffer_is_a;
-	load_class->header = vips_foreign_load_heif_buffer_header;
 
 	VIPS_ARG_BOXED( class, "buffer", 1, 
 		_( "Buffer" ),
@@ -942,6 +1182,78 @@ vips_foreign_load_heif_buffer_init( VipsForeignLoadHeifBuffer *buffer )
 {
 }
 
+typedef struct _VipsForeignLoadHeifSource {
+	VipsForeignLoadHeif parent_object;
+
+	/* Load from a source.
+	 */
+	VipsSource *source;
+
+} VipsForeignLoadHeifSource;
+
+typedef VipsForeignLoadHeifClass VipsForeignLoadHeifSourceClass;
+
+G_DEFINE_TYPE( VipsForeignLoadHeifSource, vips_foreign_load_heif_source, 
+	vips_foreign_load_heif_get_type() );
+
+static int
+vips_foreign_load_heif_source_build( VipsObject *object )
+{
+	VipsForeignLoadHeif *heif = (VipsForeignLoadHeif *) object;
+	VipsForeignLoadHeifSource *source = 
+		(VipsForeignLoadHeifSource *) object;
+
+	if( source->source ) {
+		heif->source = source->source;
+		g_object_ref( heif->source );
+	}
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_load_heif_source_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	return( 0 );
+}
+
+static gboolean
+vips_foreign_load_heif_source_is_a_source( VipsSource *source )
+{
+	const char *p;
+
+	return( (p = (const char *) vips_source_sniff( source, 12 )) &&
+		vips_foreign_load_heif_is_a( p, 12 ) );
+}
+
+static void
+vips_foreign_load_heif_source_class_init( 
+	VipsForeignLoadHeifSourceClass *class )
+{
+	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
+
+	gobject_class->set_property = vips_object_set_property;
+	gobject_class->get_property = vips_object_get_property;
+
+	object_class->nickname = "heifload_source";
+	object_class->build = vips_foreign_load_heif_source_build;
+
+	load_class->is_a_source = vips_foreign_load_heif_source_is_a_source;
+
+	VIPS_ARG_OBJECT( class, "source", 1,
+		_( "Source" ),
+		_( "Source to load from" ),
+		VIPS_ARGUMENT_REQUIRED_INPUT, 
+		G_STRUCT_OFFSET( VipsForeignLoadHeifSource, source ),
+		VIPS_TYPE_SOURCE );
+
+}
+
+static void
+vips_foreign_load_heif_source_init( VipsForeignLoadHeifSource *source )
+{
+}
+
 #endif /*HAVE_HEIF_DECODER*/
 
 /**
@@ -955,7 +1267,6 @@ vips_foreign_load_heif_buffer_init( VipsForeignLoadHeifBuffer *buffer )
  * * @page: %gint, page (top-level image number) to read
  * * @n: %gint, load this many pages
  * * @thumbnail: %gboolean, fetch thumbnail instead of image
- * * @autorotate: %gboolean, rotate image upright during load 
  *
  * Read a HEIF image file into a VIPS image. 
  *
@@ -972,17 +1283,6 @@ vips_foreign_load_heif_buffer_init( VipsForeignLoadHeifBuffer *buffer )
  * If @thumbnail is %TRUE, then fetch a stored thumbnail rather than the
  * image.
  *
- * Setting @autorotate to %TRUE will make the loader interpret the 
- * orientation tag and automatically rotate the image appropriately during
- * load. 
- *
- * If @autorotate is %FALSE, the metadata field #VIPS_META_ORIENTATION is set 
- * to the value of the orientation tag. Applications may read and interpret 
- * this field
- * as they wish later in processing. See vips_autorot(). Save
- * operations will use #VIPS_META_ORIENTATION, if present, to set the
- * orientation of output images. 
- *
  * See also: vips_image_new_from_file().
  *
  * Returns: 0 on success, -1 on error.
@@ -1012,7 +1312,6 @@ vips_heifload( const char *filename, VipsImage **out, ... )
  * * @page: %gint, page (top-level image number) to read
  * * @n: %gint, load this many pages
  * * @thumbnail: %gboolean, fetch thumbnail instead of image
- * * @autorotate: %gboolean, rotate image upright during load 
  *
  * Read a HEIF image file into a VIPS image. 
  * Exactly as vips_heifload(), but read from a memory buffer. 
@@ -1043,3 +1342,34 @@ vips_heifload_buffer( void *buf, size_t len, VipsImage **out, ... )
 
 	return( result );
 }
+
+/**
+ * vips_heifload_source:
+ * @source: source to load from
+ * @out: (out): image to write
+ * @...: %NULL-terminated list of optional named arguments
+ *
+ * Optional arguments:
+ *
+ * * @page: %gint, page (top-level image number) to read
+ * * @n: %gint, load this many pages
+ * * @thumbnail: %gboolean, fetch thumbnail instead of image
+ *
+ * Exactly as vips_heifload(), but read from a source. 
+ *
+ * See also: vips_heifload().
+ *
+ * Returns: 0 on success, -1 on error.
+ */
+int
+vips_heifload_source( VipsSource *source, VipsImage **out, ... )
+{
+	va_list ap;
+	int result;
+
+	va_start( ap, out );
+	result = vips_call_split( "heifload_source", ap, source, out );
+	va_end( ap );
+
+	return( result );
+}
