@@ -100,6 +100,16 @@
  * 	- strict round down on shrink-on-load
  * 16/8/18
  * 	- shut down the input file as soon as we can [kleisauke]
+ * 20/7/19
+ * 	- close input on minimise rather than Y read position
+ * 3/10/19
+ * 	- restart after minimise
+ * 14/10/19
+ * 	- revise for source IO
+ * 5/5/20 angelmixu
+ * 	- better handling of JFIF res unit 0
+ * 13/9/20
+ * 	- set resolution unit from JFIF 
  */
 
 /*
@@ -165,10 +175,6 @@ typedef struct _ReadJpeg {
 	 */
 	gboolean fail;
 
-	/* Used for file input only.
-	 */
-	char *filename;
-
 	struct jpeg_decompress_struct cinfo;
         ErrorManager eman;
 	gboolean invert_pels;
@@ -188,25 +194,116 @@ typedef struct _ReadJpeg {
 	 */
 	int output_width;
 	int output_height;
+
+	/* The source we read from.
+	 */
+	VipsSource *source;
+
 } ReadJpeg;
 
-/* This can be called many times. It's called directly at the end of image
- * read.
+#define SOURCE_BUFFER_SIZE (4096)
+
+/* Private struct for source input.
  */
+typedef struct {
+	/* Public jpeg fields.
+	 */
+	struct jpeg_source_mgr pub;
+
+	/* Private stuff during read.
+	 */
+	VipsSource *source;
+	unsigned char buf[SOURCE_BUFFER_SIZE];
+
+} Source;
+
 static void
-readjpeg_close_input( ReadJpeg *jpeg )
+source_init_source( j_decompress_ptr cinfo )
 {
-	VIPS_FREEF( fclose, jpeg->eman.fp );
+	Source *src = (Source *) cinfo->src;
 
-	/* Don't call jpeg_finish_decompress(). It just checks the tail of the
-	 * file and who cares about that. All mem is freed in
-	 * jpeg_destroy_decompress().
+	/* Start off empty ... libjpeg will call fill_input_buffer to get the
+	 * first bytes.
 	 */
+	src->pub.next_input_byte = src->buf;
+	src->pub.bytes_in_buffer = 0;
+}
 
-	/* I don't think this can fail. It's harmless to call many times. 
-	 */
-	jpeg_destroy_decompress( &jpeg->cinfo );
+/* Fill the input buffer --- called whenever buffer is emptied.
+ */
+static boolean
+source_fill_input_buffer( j_decompress_ptr cinfo )
+{
+	static const JOCTET eoi_buffer[4] = {
+		(JOCTET) 0xFF, (JOCTET) JPEG_EOI, 0, 0
+	};
 
+	Source *src = (Source *) cinfo->src;
+
+	size_t read;
+	
+	if( (read = vips_source_read( src->source, 
+		src->buf, SOURCE_BUFFER_SIZE )) > 0 ) {
+		src->pub.next_input_byte = src->buf;
+		src->pub.bytes_in_buffer = read;
+	}
+	else {
+		WARNMS( cinfo, JWRN_JPEG_EOF );
+		src->pub.next_input_byte = eoi_buffer;
+		src->pub.bytes_in_buffer = 2;
+	}
+
+	return( TRUE );
+}
+
+static void
+skip_input_data( j_decompress_ptr cinfo, long num_bytes )
+{
+	Source *src = (Source *) cinfo->src;
+
+	if( num_bytes > 0 ) {
+		while (num_bytes > (long) src->pub.bytes_in_buffer) {
+			num_bytes -= (long) src->pub.bytes_in_buffer;
+			(void) (*src->pub.fill_input_buffer) (cinfo);
+
+			/* note we assume that fill_input_buffer will never 
+			 * return FALSE, so suspension need not be handled.
+			 */
+		}
+
+		src->pub.next_input_byte += (size_t) num_bytes;
+		src->pub.bytes_in_buffer -= (size_t) num_bytes;
+	}
+}
+
+static int
+readjpeg_open_input( ReadJpeg *jpeg )
+{
+	j_decompress_ptr cinfo = &jpeg->cinfo;
+
+	if( jpeg->source &&
+		!cinfo->src ) {
+		Source *src;
+
+		if( vips_source_rewind( jpeg->source ) )
+			return( -1 );
+
+		cinfo->src = (struct jpeg_source_mgr *)
+			(*cinfo->mem->alloc_small)( 
+				(j_common_ptr) cinfo, JPOOL_PERMANENT,
+				sizeof( Source ) );
+
+		src = (Source *) cinfo->src;
+		src->source = jpeg->source;
+		src->pub.init_source = source_init_source;
+		src->pub.fill_input_buffer = source_fill_input_buffer;
+		src->pub.resync_to_restart = jpeg_resync_to_restart; 
+		src->pub.skip_input_data = skip_input_data; 
+		src->pub.bytes_in_buffer = 0;
+		src->pub.next_input_byte = src->buf;
+	}
+
+	return( 0 );
 }
 
 /* This can be called many times.
@@ -224,30 +321,45 @@ readjpeg_free( ReadJpeg *jpeg )
 		jpeg->eman.pub.num_warnings = 0;
 	}
 
-	readjpeg_close_input( jpeg );
+	/* Don't call jpeg_finish_decompress(). It just checks the tail of the
+	 * file and who cares about that. All mem is freed in
+	 * jpeg_destroy_decompress().
+	 */
 
-	VIPS_FREE( jpeg->filename );
+	/* I don't think this can fail. It's harmless to call many times. 
+	 */
+	jpeg_destroy_decompress( &jpeg->cinfo );
+
+	VIPS_UNREF( jpeg->source );
 
 	return( 0 );
 }
 
 static void
-readjpeg_close_cb( VipsObject *object, ReadJpeg *jpeg )
+readjpeg_close_cb( VipsImage *image, ReadJpeg *jpeg )
 {
 	(void) readjpeg_free( jpeg );
 }
 
+static void
+readjpeg_minimise_cb( VipsImage *image, ReadJpeg *jpeg )
+{
+	vips_source_minimise( jpeg->source );
+}
+
 static ReadJpeg *
-readjpeg_new( VipsImage *out, int shrink, gboolean fail, gboolean autorotate )
+readjpeg_new( VipsSource *source, VipsImage *out, 
+	int shrink, gboolean fail, gboolean autorotate )
 {
 	ReadJpeg *jpeg;
 
 	if( !(jpeg = VIPS_NEW( out, ReadJpeg )) )
 		return( NULL );
 
+	jpeg->source = source;
+	g_object_ref( source );
 	jpeg->shrink = shrink;
 	jpeg->fail = fail;
-	jpeg->filename = NULL;
         jpeg->cinfo.err = jpeg_std_error( &jpeg->eman.pub );
 	jpeg->eman.pub.error_exit = vips__new_error_exit;
 	jpeg->eman.pub.output_message = vips__new_output_message;
@@ -270,40 +382,26 @@ readjpeg_new( VipsImage *out, int shrink, gboolean fail, gboolean autorotate )
 
 	g_signal_connect( out, "close", 
 		G_CALLBACK( readjpeg_close_cb ), jpeg ); 
+	g_signal_connect( out, "minimise", 
+		G_CALLBACK( readjpeg_minimise_cb ), jpeg ); 
 
 	return( jpeg );
 }
 
-/* Set input to a file.
- */
-static int
-readjpeg_file( ReadJpeg *jpeg, const char *filename )
-{
-	jpeg->filename = g_strdup( filename );
-        if( !(jpeg->eman.fp = vips__file_open_read( filename, NULL, FALSE )) ) 
-                return( -1 );
-        jpeg_stdio_src( &jpeg->cinfo, jpeg->eman.fp );
-
-	return( 0 );
-}
-
 static const char *
 find_chroma_subsample( struct jpeg_decompress_struct *cinfo )
 {
-	gboolean has_subsample;
-
 	/* libjpeg only uses 4:4:4 and 4:2:0, confusingly. 
 	 *
 	 * http://poynton.ca/PDFs/Chroma_subsampling_notation.pdf
 	 */
-	has_subsample = cinfo->max_h_samp_factor > 1 ||
+	gboolean has_subsample = cinfo->max_h_samp_factor > 1 ||
 		cinfo->max_v_samp_factor > 1;
-	if( cinfo->num_components > 3 )
-		/* A cmyk image.
-		 */
-		return( has_subsample ? "4:2:0:4" : "4:4:4:4" );
-	else
-		return( has_subsample ? "4:2:0" : "4:4:4" );
+	gboolean is_cmyk = cinfo->num_components > 3;
+
+	return( is_cmyk ? 
+		(has_subsample ? "4:2:0:4" : "4:4:4:4" ) :
+		(has_subsample ? "4:2:0" : "4:4:4") );
 }
 
 static int
@@ -320,7 +418,8 @@ attach_blob( VipsImage *im, const char *field, void *data, size_t data_length )
 	}
 
 #ifdef DEBUG
-	printf( "attach_blob: attaching %d bytes of %s\n", data_length, field );
+	printf( "attach_blob: attaching %zd bytes of %s\n", 
+		data_length, field );
 #endif /*DEBUG*/
 
 	vips_image_set_blob_copy( im, field, data, data_length );
@@ -409,24 +508,29 @@ read_jpeg_header( ReadJpeg *jpeg, VipsImage *out )
 		break;
 	}
 
-	/* Get the jfif resolution. exif may overwrite this later.
+#ifdef DEBUG
+	if( cinfo->saw_JFIF_marker )
+		printf( "read_jpeg_header: jfif _density %d, %d, unit %d\n",
+			cinfo->X_density, cinfo->Y_density,
+			cinfo->density_unit );
+#endif /*DEBUG*/
+
+	/* Get the jfif resolution. exif may overwrite this later. Default to
+	 * 72dpi (as EXIF does).
 	 */
-	xres = 1.0;
-	yres = 1.0;
+	xres = 72.0 / 25.4;
+	yres = 72.0 / 25.4;
 	if( cinfo->saw_JFIF_marker &&
 		cinfo->X_density != 1U && 
 		cinfo->Y_density != 1U ) {
-#ifdef DEBUG
-		printf( "read_jpeg_header: seen jfif _density %d, %d\n",
-			cinfo->X_density, cinfo->Y_density );
-#endif /*DEBUG*/
-
 		switch( cinfo->density_unit ) {
 		case 0:
-			/* None. Just set.
+			/* X_density / Y_density gives the pixel aspect ratio.
+			 * Leave xres, but adjust yres.
 			 */
-			xres = cinfo->X_density;
-			yres = cinfo->Y_density;
+			if( cinfo->Y_density > 0 )
+				yres = xres * cinfo->X_density / 
+					cinfo->Y_density;
 			break;
 
 		case 1:
@@ -434,6 +538,8 @@ read_jpeg_header( ReadJpeg *jpeg, VipsImage *out )
 			 */
 			xres = cinfo->X_density / 25.4;
 			yres = cinfo->Y_density / 25.4;
+			vips_image_set_string( out, 
+				VIPS_META_RESOLUTION_UNIT, "in" );
 			break;
 
 		case 2:
@@ -441,6 +547,8 @@ read_jpeg_header( ReadJpeg *jpeg, VipsImage *out )
 			 */
 			xres = cinfo->X_density / 10.0;
 			yres = cinfo->Y_density / 10.0;
+			vips_image_set_string( out, 
+				VIPS_META_RESOLUTION_UNIT, "cm" );
 			break;
 
 		default:
@@ -463,6 +571,9 @@ read_jpeg_header( ReadJpeg *jpeg, VipsImage *out )
 		interpretation,
 		xres, yres );
 
+	VIPS_SETSTR( out->filename, 
+		vips_connection_filename( VIPS_CONNECTION( jpeg->source ) ) );
+
 	vips_image_pipelinev( out, VIPS_DEMAND_STYLE_FATSTRIP, NULL );
 
 	/* cinfo->output_width and cinfo->output_height round up with
@@ -624,7 +735,7 @@ read_jpeg_header( ReadJpeg *jpeg, VipsImage *out )
 		}
 
 		vips_image_set_blob( out, VIPS_META_ICC_NAME, 
-			(VipsCallbackFn) vips_free, data, data_length );
+			(VipsCallbackFn) vips_area_free_cb, data, data_length );
 	}
 
 	return( 0 );
@@ -681,7 +792,7 @@ read_jpeg_generate( VipsRegion *or,
 		VIPS_GATE_STOP( "read_jpeg_generate: work" );
 
 #ifdef DEBUG
-		printf( "read_jpeg_generate: lonjmp() exit\n" ); 
+		printf( "read_jpeg_generate: longjmp() exit\n" ); 
 #endif /*DEBUG*/
 
 		return( -1 );
@@ -721,46 +832,11 @@ read_jpeg_generate( VipsRegion *or,
 		jpeg->y_pos += 1; 
 	}
 
-	/* Shut down the input file as soon as we can. 
-	 */
-	if( jpeg->y_pos >= or->im->Ysize ) 
-		readjpeg_close_input( jpeg );
-
 	VIPS_GATE_STOP( "read_jpeg_generate: work" );
 
 	return( 0 );
 }
 
-/* Auto-rotate, if rotate_image is set.
- */
-static VipsImage *
-read_jpeg_rotate( VipsObject *process, VipsImage *im )
-{
-	VipsImage **t = (VipsImage **) vips_object_local_array( process, 2 );
-	VipsAngle angle = vips_autorot_get_angle( im );
-
-	if( angle != VIPS_ANGLE_D0 ) {
-		/* Need to copy to memory or disc, we have to stay seq.
-		 */
-		const guint64 image_size = VIPS_IMAGE_SIZEOF_IMAGE( im );
-		const guint64 disc_threshold = vips_get_disc_threshold();
-
-		if( image_size > disc_threshold ) 
-			t[0] = vips_image_new_temp_file( "%s.v" );
-		else
-			t[0] = vips_image_new_memory();
-
-		if( vips_image_write( im, t[0] ) ||
-			vips_rot( t[0], &t[1], angle, NULL ) )
-			return( NULL );
-		im = t[1];
-
-		vips_autorot_remove_angle( im ); 
-	}
-
-	return( im );
-}
-
 /* Read a cinfo to a VIPS image.
  */
 static int
@@ -768,7 +844,7 @@ read_jpeg_image( ReadJpeg *jpeg, VipsImage *out )
 {
 	struct jpeg_decompress_struct *cinfo = &jpeg->cinfo;
 	VipsImage **t = (VipsImage **) 
-		vips_object_local_array( VIPS_OBJECT( out ), 3 );
+		vips_object_local_array( VIPS_OBJECT( out ), 5 );
 
 	VipsImage *im;
 
@@ -799,10 +875,18 @@ read_jpeg_image( ReadJpeg *jpeg, VipsImage *out )
 		vips_extract_area( t[1], &t[2], 
 			0, 0, jpeg->output_width, jpeg->output_height, NULL ) )
 		return( -1 );
-
 	im = t[2];
-	if( jpeg->autorotate )
-		im = read_jpeg_rotate( VIPS_OBJECT( out ), im );
+
+	if( jpeg->autorotate &&
+		vips_image_get_orientation( im ) != 1 ) {
+		/* We have to copy to memory before calling autorot, since it
+		 * needs random access.
+		 */
+		if( !(t[3] = vips_image_copy_memory( im )) ||
+			vips_autorot( t[3], &t[4], NULL ) )
+			return( -1 );
+		im = t[4];
+	}
 
 	if( vips_image_write( im, out ) )
 		return( -1 );
@@ -810,8 +894,6 @@ read_jpeg_image( ReadJpeg *jpeg, VipsImage *out )
 	return( 0 );
 }
 
-/* Read the jpeg from file or buffer.
- */
 static int
 vips__jpeg_read( ReadJpeg *jpeg, VipsImage *out, gboolean header_only )
 {
@@ -847,15 +929,9 @@ vips__jpeg_read( ReadJpeg *jpeg, VipsImage *out, gboolean header_only )
 
 		/* Swap width and height if we're going to rotate this image.
 		 */
-		if( jpeg->autorotate ) { 
-			VipsAngle angle = vips_autorot_get_angle( out ); 
-
-			if( angle == VIPS_ANGLE_D90 || 
-				angle == VIPS_ANGLE_D270 )
-				VIPS_SWAP( int, out->Xsize, out->Ysize );
-
-			/* We won't be returning an orientation tag.
-			 */
+		if( jpeg->autorotate &&
+			vips_image_get_orientation_swap( out ) ) {
+			VIPS_SWAP( int, out->Xsize, out->Ysize );
 			vips_autorot_remove_angle( out ); 
 		}
 	}
@@ -867,223 +943,40 @@ vips__jpeg_read( ReadJpeg *jpeg, VipsImage *out, gboolean header_only )
 	return( 0 );
 }
 
-/* Read a JPEG file into a VIPS image.
- */
 int
-vips__jpeg_read_file( const char *filename, VipsImage *out, 
-	gboolean header_only, int shrink, gboolean fail, 
-	gboolean autorotate )
-{
-	ReadJpeg *jpeg;
-
-	if( !(jpeg = readjpeg_new( out, shrink, fail, autorotate )) )
-		return( -1 );
-
-	/* Here for longjmp() from vips__new_error_exit() during startup.
-	 */
-	if( setjmp( jpeg->eman.jmp ) ) 
-		return( -1 );
-
-	/* Set input to file.
-	 */
-	if( readjpeg_file( jpeg, filename ) ) 
-		return( -1 );
-
-	if( vips__jpeg_read( jpeg, out, header_only ) ) 
-		return( -1 );
-
-	VIPS_SETSTR( out->filename, filename );
-
-	/* We can kill off the decompress early if this is just a header read.
-	 * This saves an fd during read.
-	 */
-	if( header_only )
-		readjpeg_free( jpeg );
-
-	return( 0 );
-}
-
-/* Just like the above, but we read from a memory buffer.
- */
-typedef struct {
-	/* Public jpeg fields.
-	 */
-	struct jpeg_source_mgr pub;
-
-	/* Private stuff during read.
-	 */
-	const JOCTET *buf;
-	size_t len;
-} InputBuffer;
-
-static void
-init_source (j_decompress_ptr cinfo)
-{
-  /* no work necessary here */
-}
-
-/*
- * Fill the input buffer --- called whenever buffer is emptied.
- *
- * We fill the buffer on readjpeg_buffer(), so this will only be called if
- * libjpeg tries to read beyond the buffer.
- */
-
-static boolean
-fill_input_buffer (j_decompress_ptr cinfo)
-{
-  static const JOCTET eoi_buffer[4] = {
-    (JOCTET) 0xFF, (JOCTET) JPEG_EOI, 0, 0
-  };
-
-  InputBuffer *src = (InputBuffer *) cinfo->src;
-
-  WARNMS(cinfo, JWRN_JPEG_EOF);
-  src->pub.next_input_byte = eoi_buffer;
-  src->pub.bytes_in_buffer = 2;
-
-  return TRUE;
-}
-
-/*
- * Skip data --- used to skip over a potentially large amount of
- * uninteresting data (such as an APPn marker).
- *
- * Writers of suspendable-input applications must note that skip_input_data
- * is not granted the right to give a suspension return.  If the skip extends
- * beyond the data currently in the buffer, the buffer can be marked empty so
- * that the next read will cause a fill_input_buffer call that can suspend.
- * Arranging for additional bytes to be discarded before reloading the input
- * buffer is the application writer's problem.
- */
-
-static void
-skip_input_data (j_decompress_ptr cinfo, long num_bytes)
-{
-  InputBuffer *src = (InputBuffer *) cinfo->src;
-
-  if (num_bytes > 0) {
-    while (num_bytes > (long) src->pub.bytes_in_buffer) {
-      num_bytes -= (long) src->pub.bytes_in_buffer;
-      (void) (*src->pub.fill_input_buffer) (cinfo);
-
-      /* note we assume that fill_input_buffer will never return FALSE,
-       * so suspension need not be handled.
-       */
-    }
-
-    src->pub.next_input_byte += (size_t) num_bytes;
-    src->pub.bytes_in_buffer -= (size_t) num_bytes;
-  }
-}
-
-/*
- * An additional method that can be provided by data source modules is the
- * resync_to_restart method for error recovery in the presence of RST markers.
- * For the moment, this source module just uses the default resync method
- * provided by the JPEG library.  That method assumes that no backtracking
- * is possible.
- */
-
-/*
- * Terminate source --- called by jpeg_finish_decompress
- * after all data has been read.  Often a no-op.
- *
- * NB: *not* called by jpeg_abort or jpeg_destroy; surrounding
- * application must deal with any cleanup that should happen even
- * for error exit.
- */
-
-static void
-term_source (j_decompress_ptr cinfo)
-{
-  /* no work necessary here */
-}
-
-/*
- * Prepare for input from a memory buffer. The caller needs to free the
- * buffer after decompress is done, we don't take ownership.
- */
-
-static void
-readjpeg_buffer (ReadJpeg *jpeg, const void *buf, size_t len)
-{
-  j_decompress_ptr cinfo = &jpeg->cinfo;
-  InputBuffer *src;
-
-  /* Empty buffer is a fatal error.
-   */
-  if (len == 0  || buf == NULL)
-    ERREXIT(cinfo, JERR_INPUT_EMPTY);
-
-  /* The source object and input buffer are made permanent so that a series
-   * of JPEG images can be read from the same file by calling jpeg_stdio_src
-   * only before the first one.  (If we discarded the buffer at the end of
-   * one image, we'd likely lose the start of the next one.)
-   * This makes it unsafe to use this manager and a different source
-   * manager serially with the same JPEG object.  Caveat programmer.
-   */
-  if (cinfo->src == NULL) {	/* first time for this JPEG object? */
-    cinfo->src = (struct jpeg_source_mgr *)
-      (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_PERMANENT,
-				  sizeof(InputBuffer));
-    src = (InputBuffer *) cinfo->src;
-    src->buf = buf;
-    src->len = len;
-  }
-
-  src = (InputBuffer *) cinfo->src;
-  src->pub.init_source = init_source;
-  src->pub.fill_input_buffer = fill_input_buffer;
-  src->pub.skip_input_data = skip_input_data;
-  src->pub.resync_to_restart = jpeg_resync_to_restart; /* use default method */
-  src->pub.term_source = term_source;
-  src->pub.bytes_in_buffer = len;
-  src->pub.next_input_byte = buf;
-}
-
-int
-vips__jpeg_read_buffer( const void *buf, size_t len, VipsImage *out, 
+vips__jpeg_read_source( VipsSource *source, VipsImage *out,
 	gboolean header_only, int shrink, int fail, gboolean autorotate )
 {
 	ReadJpeg *jpeg;
 
-	if( !(jpeg = readjpeg_new( out, shrink, fail, autorotate )) )
+	if( !(jpeg = readjpeg_new( source, out, shrink, fail, autorotate )) )
 		return( -1 );
 
 	if( setjmp( jpeg->eman.jmp ) ) 
 		return( -1 );
 
-	/* Set input to buffer.
-	 */
-	readjpeg_buffer( jpeg, buf, len );
-
-	if( vips__jpeg_read( jpeg, out, header_only ) ) 
+	if( readjpeg_open_input( jpeg ) ||
+		vips__jpeg_read( jpeg, out, header_only ) )
 		return( -1 );
 
-	return( 0 );
-}
-
-int
-vips__isjpeg_buffer( const void *buf, size_t len )
-{
-	const guchar *str = (const guchar *) buf;
-
-	if( len >= 2 &&
-		str[0] == 0xff && 
-		str[1] == 0xd8 )
-		return( 1 );
+	if( header_only )
+		vips_source_minimise( source );
+	else {
+		if( vips_source_decode( source ) )
+			return( -1 );
+	}
 
 	return( 0 );
 }
 
 int
-vips__isjpeg( const char *filename )
+vips__isjpeg_source( VipsSource *source )
 {
-	unsigned char buf[2];
+	const unsigned char *p;
 
-	if( vips__get_bytes( filename, buf, 2 ) == 2 &&
-		vips__isjpeg_buffer( buf, 2 ) )
+	if( (p = vips_source_sniff( source, 2 )) &&
+		p[0] == 0xff && 
+		p[1] == 0xd8 )
 		return( 1 );
 
 	return( 0 );
