@@ -2,6 +2,8 @@
  *
  * 24/11/11
  * 	- wrap a class around the jpeg writer
+ * 18/2/20 Elad-Laufer
+ * 	- add subsample_mode, deprecate no_subsample
  */
 
 /*
@@ -54,20 +56,6 @@
 
 #ifdef HAVE_JPEG
 
-#ifdef HAVE_EXIF
-#ifdef UNTAGGED_EXIF
-#include <exif-data.h>
-#include <exif-loader.h>
-#include <exif-ifd.h>
-#include <exif-utils.h>
-#else /*!UNTAGGED_EXIF*/
-#include <libexif/exif-data.h>
-#include <libexif/exif-loader.h>
-#include <libexif/exif-ifd.h>
-#include <libexif/exif-utils.h>
-#endif /*UNTAGGED_EXIF*/
-#endif /*HAVE_EXIF*/
-
 typedef struct _VipsForeignSaveJpeg {
 	VipsForeignSave parent_object;
 
@@ -87,10 +75,17 @@ typedef struct _VipsForeignSaveJpeg {
 	 */
 	gboolean interlace;
 
-	/* Disable chroma subsampling. 
+	/* Deprecated: Disable chroma subsampling. Use subsample_mode instead.
 	 */
 	gboolean no_subsample;
 
+	/* Select chroma subsampling mode:
+	 * auto will disable subsampling for Q >= 90
+	 * on will always enable subsampling
+	 * off will always disable subsampling
+	 */
+	VipsForeignJpegSubsample subsample_mode;
+
 	/* Apply trellis quantisation to each 8x8 block.
 	 */
 	gboolean trellis_quant;
@@ -120,10 +115,30 @@ G_DEFINE_ABSTRACT_TYPE( VipsForeignSaveJpeg, vips_foreign_save_jpeg,
 /* Type promotion for save ... just always go to uchar.
  */
 static int bandfmt_jpeg[10] = {
-/* UC  C   US  S   UI  I   F   X   D   DX */
-   UC, UC, UC, UC, UC, UC, UC, UC, UC, UC
+     /* UC  C   US  S   UI  I   F   X   D   DX */
+	UC, UC, UC, UC, UC, UC, UC, UC, UC, UC
 };
 
+static int
+vips_foreign_save_jpeg_build( VipsObject *object )
+{
+	VipsForeignSaveJpeg *jpeg = (VipsForeignSaveJpeg *) object;
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_save_jpeg_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	/* no_subsample is deprecated, but we retain backwards compatibility
+	 * new code should use subsample_mode
+	 */
+	if( vips_object_argument_isset( object, "no_subsample" ) )
+		jpeg->subsample_mode = jpeg->no_subsample ? 
+			VIPS_FOREIGN_JPEG_SUBSAMPLE_OFF : 
+			VIPS_FOREIGN_JPEG_SUBSAMPLE_AUTO;
+
+	return( 0 );
+}
+
 static void
 vips_foreign_save_jpeg_class_init( VipsForeignSaveJpegClass *class )
 {
@@ -137,6 +152,7 @@ vips_foreign_save_jpeg_class_init( VipsForeignSaveJpegClass *class )
 
 	object_class->nickname = "jpegsave_base";
 	object_class->description = _( "save jpeg" );
+	object_class->build = vips_foreign_save_jpeg_build;
 
 	foreign_class->suffs = vips__jpeg_suffs;
 
@@ -176,7 +192,7 @@ vips_foreign_save_jpeg_class_init( VipsForeignSaveJpegClass *class )
 	VIPS_ARG_BOOL( class, "no_subsample", 14,
 		_( "No subsample" ),
 		_( "Disable chroma subsample" ),
-		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
 		G_STRUCT_OFFSET( VipsForeignSaveJpeg, no_subsample ),
 		FALSE );
 
@@ -196,7 +212,7 @@ vips_foreign_save_jpeg_class_init( VipsForeignSaveJpegClass *class )
 
 	VIPS_ARG_BOOL( class, "optimize_scans", 17,
 		_( "Optimize scans" ),
-		_( "Split the spectrum of DCT coefficients into separate scans" ),
+		_( "Split spectrum of DCT coefficients into separate scans" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignSaveJpeg, optimize_scans ),
 		FALSE );
@@ -208,12 +224,81 @@ vips_foreign_save_jpeg_class_init( VipsForeignSaveJpegClass *class )
 		G_STRUCT_OFFSET( VipsForeignSaveJpeg, quant_table ),
 		0, 8, 0 );
 
+	VIPS_ARG_ENUM( class, "subsample_mode", 19,
+		_( "Subsample mode" ),
+		_( "Select chroma subsample operation mode" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSaveJpeg, subsample_mode ),
+		VIPS_TYPE_FOREIGN_JPEG_SUBSAMPLE,
+		VIPS_FOREIGN_JPEG_SUBSAMPLE_AUTO );
 }
 
 static void
 vips_foreign_save_jpeg_init( VipsForeignSaveJpeg *jpeg )
 {
 	jpeg->Q = 75;
+	jpeg->subsample_mode = VIPS_FOREIGN_JPEG_SUBSAMPLE_AUTO;
+}
+
+typedef struct _VipsForeignSaveJpegTarget {
+	VipsForeignSaveJpeg parent_object;
+
+	VipsTarget *target;
+
+} VipsForeignSaveJpegTarget;
+
+typedef VipsForeignSaveJpegClass VipsForeignSaveJpegTargetClass;
+
+G_DEFINE_TYPE( VipsForeignSaveJpegTarget, vips_foreign_save_jpeg_target, 
+	vips_foreign_save_jpeg_get_type() );
+
+static int
+vips_foreign_save_jpeg_target_build( VipsObject *object )
+{
+	VipsForeignSave *save = (VipsForeignSave *) object;
+	VipsForeignSaveJpeg *jpeg = (VipsForeignSaveJpeg *) object;
+	VipsForeignSaveJpegTarget *target = 
+		(VipsForeignSaveJpegTarget *) object;
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_save_jpeg_target_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	if( vips__jpeg_write_target( save->ready, target->target,
+		jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
+		jpeg->interlace, save->strip, jpeg->trellis_quant,
+		jpeg->overshoot_deringing, jpeg->optimize_scans,
+		jpeg->quant_table, jpeg->subsample_mode ) )
+		return( -1 );
+
+	return( 0 );
+}
+
+static void
+vips_foreign_save_jpeg_target_class_init( 
+	VipsForeignSaveJpegTargetClass *class )
+{
+	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+
+	gobject_class->set_property = vips_object_set_property;
+	gobject_class->get_property = vips_object_get_property;
+
+	object_class->nickname = "jpegsave_target";
+	object_class->description = _( "save image to jpeg target" );
+	object_class->build = vips_foreign_save_jpeg_target_build;
+
+	VIPS_ARG_OBJECT( class, "target", 1,
+		_( "Target" ),
+		_( "Target to save to" ),
+		VIPS_ARGUMENT_REQUIRED_INPUT, 
+		G_STRUCT_OFFSET( VipsForeignSaveJpegTarget, target ),
+		VIPS_TYPE_TARGET );
+}
+
+static void
+vips_foreign_save_jpeg_target_init( VipsForeignSaveJpegTarget *target )
+{
 }
 
 typedef struct _VipsForeignSaveJpegFile {
@@ -237,16 +322,23 @@ vips_foreign_save_jpeg_file_build( VipsObject *object )
 	VipsForeignSaveJpeg *jpeg = (VipsForeignSaveJpeg *) object;
 	VipsForeignSaveJpegFile *file = (VipsForeignSaveJpegFile *) object;
 
+	VipsTarget *target;
+
 	if( VIPS_OBJECT_CLASS( vips_foreign_save_jpeg_file_parent_class )->
 		build( object ) )
 		return( -1 );
 
-	if( vips__jpeg_write_file( save->ready, file->filename,
-		jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
-		jpeg->interlace, save->strip, jpeg->no_subsample,
-		jpeg->trellis_quant, jpeg->overshoot_deringing,
-		jpeg->optimize_scans, jpeg->quant_table ) )
+	if( !(target = vips_target_new_to_file( file->filename )) )
 		return( -1 );
+	if( vips__jpeg_write_target( save->ready, target,
+		jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
+		jpeg->interlace, save->strip, jpeg->trellis_quant,
+		jpeg->overshoot_deringing, jpeg->optimize_scans,
+		jpeg->quant_table, jpeg->subsample_mode ) ) {
+		VIPS_UNREF( target );
+		return( -1 );
+	}
+	VIPS_UNREF( target );
 
 	return( 0 );
 }
@@ -298,27 +390,31 @@ vips_foreign_save_jpeg_buffer_build( VipsObject *object )
 	VipsForeignSaveJpeg *jpeg = (VipsForeignSaveJpeg *) object;
 	VipsForeignSaveJpegBuffer *file = (VipsForeignSaveJpegBuffer *) object;
 
-	void *obuf;
-	size_t olen;
+	VipsTarget *target;
 	VipsBlob *blob;
 
 	if( VIPS_OBJECT_CLASS( vips_foreign_save_jpeg_buffer_parent_class )->
 		build( object ) )
 		return( -1 );
 
-	if( vips__jpeg_write_buffer( save->ready, 
-		&obuf, &olen, jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
-		jpeg->interlace, save->strip, jpeg->no_subsample,
-		jpeg->trellis_quant, jpeg->overshoot_deringing,
-		jpeg->optimize_scans, jpeg->quant_table ) )
+	if( !(target = vips_target_new_to_memory()) )
 		return( -1 );
 
-	/* obuf is a g_free() buffer, not vips_free().
-	 */
-	blob = vips_blob_new( (VipsCallbackFn) g_free, obuf, olen );
+	if( vips__jpeg_write_target( save->ready, target,
+		jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
+		jpeg->interlace, save->strip, jpeg->trellis_quant,
+		jpeg->overshoot_deringing, jpeg->optimize_scans,
+		jpeg->quant_table, jpeg->subsample_mode ) ) {
+		VIPS_UNREF( target );
+		return( -1 );
+	}
+
+	g_object_get( target, "blob", &blob, NULL );
 	g_object_set( file, "buffer", blob, NULL );
 	vips_area_unref( VIPS_AREA( blob ) );
 
+	VIPS_UNREF( target );
+
 	return( 0 );
 }
 
@@ -365,30 +461,39 @@ vips_foreign_save_jpeg_mime_build( VipsObject *object )
 	VipsForeignSave *save = (VipsForeignSave *) object;
 	VipsForeignSaveJpeg *jpeg = (VipsForeignSaveJpeg *) object;
 
-	void *obuf;
+	VipsTarget *target;
+	VipsBlob *blob;
+	const unsigned char *obuf;
 	size_t olen;
 
 	if( VIPS_OBJECT_CLASS( vips_foreign_save_jpeg_mime_parent_class )->
 		build( object ) )
 		return( -1 );
 
-	if( vips__jpeg_write_buffer( save->ready, 
-		&obuf, &olen, jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
-		jpeg->interlace, save->strip, jpeg->no_subsample,
-		jpeg->trellis_quant, jpeg->overshoot_deringing,
-		jpeg->optimize_scans, jpeg->quant_table ) )
+	if( !(target = vips_target_new_to_memory()) )
 		return( -1 );
 
+	if( vips__jpeg_write_target( save->ready, target,
+		jpeg->Q, jpeg->profile, jpeg->optimize_coding, 
+		jpeg->interlace, save->strip, jpeg->trellis_quant,
+		jpeg->overshoot_deringing, jpeg->optimize_scans,
+		jpeg->quant_table, jpeg->subsample_mode ) ) {
+		VIPS_UNREF( target );
+		return( -1 );
+	}
+
+	g_object_get( target, "blob", &blob, NULL );
+
+	obuf = vips_blob_get( blob, &olen );
 	printf( "Content-length: %zu\r\n", olen );
 	printf( "Content-type: image/jpeg\r\n" );
 	printf( "\r\n" );
-	if( fwrite( obuf, sizeof( char ), olen, stdout ) != olen ) {
-		vips_error( "VipsJpeg", "%s", _( "error writing output" ) );
-		return( -1 );
-	}
+	(void) fwrite( obuf, sizeof( char ), olen, stdout );
 	fflush( stdout );
 
-	g_free( obuf );
+	vips_area_unref( VIPS_AREA( blob ) );
+
+	VIPS_UNREF( target );
 
 	return( 0 );
 }
@@ -420,11 +525,11 @@ vips_foreign_save_jpeg_mime_init( VipsForeignSaveJpegMime *mime )
  * Optional arguments:
  *
  * * @Q: %gint, quality factor
- * * @profile: filename of ICC profile to attach
+ * * @profile: %gchararray, filename of ICC profile to attach
  * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
  * * @interlace: %gboolean, write an interlaced (progressive) jpeg
  * * @strip: %gboolean, remove all metadata from image
- * * @no_subsample: %gboolean, disable chroma subsampling
+ * * @subsample_mode: #VipsForeignJpegSubsample, chroma subsampling mode
  * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
  * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
  * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
@@ -442,7 +547,7 @@ vips_foreign_save_jpeg_mime_init( VipsForeignSaveJpegMime *mime )
  * contains an ICC profile named #VIPS_META_ICC_NAME, the
  * profile from the VIPS header will be attached.
  *
- * If @optimize_coding is set, the Huffman tables are optimised. This is
+ * If @optimize_coding is set, the Huffman tables are optimized. This is
  * sllightly slower and produces slightly smaller files. 
  *
  * If @interlace is set, the jpeg files will be interlaced (progressive jpeg,
@@ -450,10 +555,10 @@ vips_foreign_save_jpeg_mime_init( VipsForeignSaveJpegMime *mime )
  * conection, but need much more memory to encode and decode. 
  *
  * If @strip is set, no EXIF data, IPTC data, ICC profile or XMP metadata is 
- * written into the output file. 
+ * written into the output file.
  *
- * If @no_subsample is set, chrominance subsampling is disabled. This will 
- * improve quality at the cost of larger file size. Useful for high Q factors. 
+ * Chroma subsampling is normally automatically disabled for Q > 90. You can
+ * force the subsampling mode with @@subsample_mode.
  *
  * If @trellis_quant is set and the version of libjpeg supports it
  * (e.g. mozjpeg >= 3.0), apply trellis quantisation to each 8x8 block.
@@ -496,6 +601,9 @@ vips_foreign_save_jpeg_mime_init( VipsForeignSaveJpegMime *mime )
  * Tables 5-7 are based on older research papers, but generally achieve worse
  * compression ratios and/or quality than 2 or 4.
  *
+ * For maximum compression with mozjpeg, a useful set of options is `strip, 
+ * optimize-coding, interlace, optimize-scans, trellis-quant, quant_table=3`.
+ *
  * The image is automatically converted to RGB, Monochrome or CMYK before 
  * saving. 
  *
@@ -524,6 +632,44 @@ vips_jpegsave( VipsImage *in, const char *filename, ... )
 	return( result );
 }
 
+/**
+ * vips_jpegsave_target: (method)
+ * @in: image to save 
+ * @target: save image to this target
+ * @...: %NULL-terminated list of optional named arguments
+ *
+ * Optional arguments:
+ *
+ * * @Q: %gint, quality factor
+ * * @profile: %gchararray, filename of ICC profile to attach
+ * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
+ * * @interlace: %gboolean, write an interlaced (progressive) jpeg
+ * * @strip: %gboolean, remove all metadata from image
+ * * @subsample_mode: #VipsForeignJpegSubsample, chroma subsampling mode
+ * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
+ * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
+ * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
+ * * @quant_table: %gint, quantization table index
+ *
+ * As vips_jpegsave(), but save to a target.
+ *
+ * See also: vips_jpegsave(), vips_image_write_to_target().
+ *
+ * Returns: 0 on success, -1 on error.
+ */
+int
+vips_jpegsave_target( VipsImage *in, VipsTarget *target, ... )
+{
+	va_list ap;
+	int result;
+
+	va_start( ap, target );
+	result = vips_call_split( "jpegsave_target", ap, in, target );
+	va_end( ap );
+
+	return( result );
+}
+
 /**
  * vips_jpegsave_buffer: (method)
  * @in: image to save 
@@ -534,11 +680,11 @@ vips_jpegsave( VipsImage *in, const char *filename, ... )
  * Optional arguments:
  *
  * * @Q: %gint, quality factor
- * * @profile: filename of ICC profile to attach
+ * * @profile: %gchararray, filename of ICC profile to attach
  * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
  * * @interlace: %gboolean, write an interlaced (progressive) jpeg
  * * @strip: %gboolean, remove all metadata from image
- * * @no_subsample: %gboolean, disable chroma subsampling
+ * * @subsample_mode: #VipsForeignJpegSubsample, chroma subsampling mode
  * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
  * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
  * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
@@ -590,11 +736,11 @@ vips_jpegsave_buffer( VipsImage *in, void **buf, size_t *len, ... )
  * Optional arguments:
  *
  * * @Q: %gint, quality factor
- * * @profile: filename of ICC profile to attach
+ * * @profile: %gchararray, filename of ICC profile to attach
  * * @optimize_coding: %gboolean, compute optimal Huffman coding tables
  * * @interlace: %gboolean, write an interlaced (progressive) jpeg
  * * @strip: %gboolean, remove all metadata from image
- * * @no_subsample: %gboolean, disable chroma subsampling
+ * * @subsample_mode: #VipsForeignJpegSubsample, chroma subsampling mode
  * * @trellis_quant: %gboolean, apply trellis quantisation to each 8x8 block
  * * @overshoot_deringing: %gboolean, overshoot samples with extreme values
  * * @optimize_scans: %gboolean, split DCT coefficients into separate scans
