@@ -4,6 +4,8 @@
  *
  * 24/7/18
  * 	- add the sniffer
+ * 16/10/20 [bfriesen]
+ * 	- set matte and depth appropriately for GM in magick_import_pixels()
  */
 
 /*
@@ -209,6 +211,33 @@ magick_set_number_scenes( ImageInfo *image_info, int scene, int number_scenes )
 	image_info->scenes = strdup( page );
 }
 
+int
+magick_optimize_image_layers( Image **images, ExceptionInfo *exception )
+{
+	Image *tmp;
+
+	tmp = OptimizePlusImageLayers( *images, exception );
+
+	if( exception->severity != UndefinedException ) {
+		VIPS_FREEF( DestroyImageList, tmp );
+		return MagickFalse;
+	}
+
+	VIPS_FREEF( DestroyImageList, *images );
+	*images = tmp;
+
+	return MagickTrue;
+}
+
+int
+magick_optimize_image_transparency( const Image *images,
+	ExceptionInfo *exception )
+{
+	OptimizeImageTransparency( images, exception );
+
+	return( exception->severity == UndefinedException );
+}
+
 /* Does a few bytes look like a file IM can handle?
  */
 gboolean
@@ -260,10 +289,17 @@ int
 magick_set_image_size( Image *image, const size_t width, const size_t height,
 	ExceptionInfo *exception )
 {
-	(void) exception;
 #ifdef HAVE_SETIMAGEEXTENT
-	return( SetImageExtent( image, width, height ) );
+	int result = SetImageExtent( image, width, height );
+
+	/* IM6 sets the exception on the image.
+	 */
+	if( !result )
+		magick_inherit_exception( exception, image );
+
+	return( result ); 
 #else /*!HAVE_SETIMAGEEXTENT*/
+	(void) exception;
 	image->columns = width;
 	image->rows = height;
 
@@ -285,6 +321,7 @@ magick_import_pixels( Image *image, const ssize_t x, const ssize_t y,
 		type, pixels ) );
 #else /*!HAVE_IMPORTIMAGEPIXELS*/
 	Image *constitute_image;
+	unsigned int storage_type_depth;
 
 	g_assert( image );
 	g_assert( image->signature == MagickSignature );
@@ -294,7 +331,43 @@ magick_import_pixels( Image *image, const ssize_t x, const ssize_t y,
 	if( !constitute_image ) 
 		return( MagickFalse );
 
+	/* image needs to inherit these fields from constitute_image.
+	 */
+	switch( type ) {
+	case CharPixel: 
+		storage_type_depth = sizeof( unsigned char ) * 8; 
+		break;
+
+	case ShortPixel: 
+		storage_type_depth = sizeof( unsigned short ) * 8; 
+		break;
+
+	case IntegerPixel: 
+		storage_type_depth = sizeof( unsigned short ) * 8; 
+		break;
+
+	case LongPixel: 
+		storage_type_depth = sizeof( unsigned long ) * 8; 
+		break;
+
+	case FloatPixel: 
+		storage_type_depth = sizeof( float ) * 8; 
+		break;
+
+	case DoublePixel: 
+		storage_type_depth = sizeof( double ) * 8; 
+		break;
+
+	default:
+		storage_type_depth = QuantumDepth;
+		break;
+
+	}
+	image->depth = VIPS_MIN( storage_type_depth, QuantumDepth );
+	image->matte = constitute_image->matte;
+
 	(void) CompositeImage( image, CopyCompositeOp, constitute_image, x, y );
+
 	DestroyImage( constitute_image );
 
 	return( image->exception.severity == UndefinedException );
@@ -449,6 +522,43 @@ magick_set_number_scenes( ImageInfo *image_info, int scene, int number_scenes )
 #endif
 }
 
+int
+magick_optimize_image_layers( Image **images, ExceptionInfo *exception )
+{
+#ifdef HAVE_OPTIMIZEPLUSIMAGELAYERS
+	Image *tmp;
+
+	tmp = OptimizePlusImageLayers(*images, exception );
+
+	if ( exception->severity != UndefinedException )
+		return MagickFalse;
+
+	VIPS_FREEF( DestroyImageList, *images );
+
+	*images = tmp;
+
+	return MagickTrue;
+#else /*!HAVE_OPTIMIZEPLUSIMAGELAYERS*/
+	g_warning( "%s", _( "layer optimization is not supported by "
+		"your version of libMagick" ) );
+	return MagickTrue;
+#endif /*HAVE_OPTIMIZEPLUSIMAGELAYERS*/
+}
+
+int
+magick_optimize_image_transparency( const Image *images,
+	ExceptionInfo *exception )
+{
+#ifdef HAVE_OPTIMIZEIMAGETRANSPARENCY
+	OptimizeImageTransparency(images, exception);
+	return ( exception->severity == UndefinedException );
+#else /*!HAVE_OPTIMIZEIMAGETRANSPARENCY*/
+	g_warning( "%s", _( "transparency optimization is not supported by "
+		"your version of libMagick" ) );
+	return MagickTrue;
+#endif /*HAVE_OPTIMIZEIMAGETRANSPARENCY*/
+}
+
 /* Does a few bytes look like a file IM can handle?
  */
 gboolean
@@ -572,6 +682,8 @@ magick_vips_error( const char *domain, ExceptionInfo *exception )
 static void *
 magick_genesis_cb( void *client )
 {
+	ExceptionInfo *exception;
+
 #ifdef DEBUG
 	printf( "magick_genesis_cb:\n" ); 
 #endif /*DEBUG*/
@@ -579,9 +691,18 @@ magick_genesis_cb( void *client )
 #if defined(HAVE_MAGICKCOREGENESIS) || defined(HAVE_MAGICK7) 
 	MagickCoreGenesis( vips_get_argv0(), MagickFalse );
 #else /*!HAVE_MAGICKCOREGENESIS*/
-	InitializeMagick( "" );
+	InitializeMagick( vips_get_argv0() );
 #endif /*HAVE_MAGICKCOREGENESIS*/
 
+	/* This forces *magick to init all loaders. We have to do this so we
+	 * can sniff files with GetImageMagick(). 
+	 *
+	 * We don't care about errors from magickinit.
+	 */
+	exception = magick_acquire_exception();
+	(void) GetMagickInfo( "*", exception );
+	magick_destroy_exception(exception);
+
 	return( NULL );
 }
 
@@ -604,13 +725,13 @@ magick_set_vips_profile_cb( Image *image,
 	char name_text[256];
 	VipsBuf vips_name = VIPS_BUF_STATIC( name_text );
 
-	if( strcmp( name, "XMP" ) == 0 )
+	if( strcasecmp( name, "XMP" ) == 0 )
 		vips_buf_appendf( &vips_name, VIPS_META_XMP_NAME );
-	else if( strcmp( name, "IPTC" ) == 0 )
+	else if( strcasecmp( name, "IPTC" ) == 0 )
 		vips_buf_appendf( &vips_name, VIPS_META_IPTC_NAME );
-	else if( strcmp( name, "ICM" ) == 0 )
+	else if( strcasecmp( name, "ICC" ) == 0 )
 		vips_buf_appendf( &vips_name, VIPS_META_ICC_NAME );
-	else if( strcmp( name, "EXIF" ) == 0 )
+	else if( strcasecmp( name, "EXIF" ) == 0 )
 		vips_buf_appendf( &vips_name, VIPS_META_EXIF_NAME );
 	else
 		vips_buf_appendf( &vips_name, "magickprofile-%s", name );
@@ -654,7 +775,7 @@ magick_set_magick_profile_cb( VipsImage *im,
 	else if( strcmp( name, VIPS_META_IPTC_NAME ) == 0 )
 		vips_buf_appendf( &buf, "IPTC" );
 	else if( strcmp( name, VIPS_META_ICC_NAME ) == 0 )
-		vips_buf_appendf( &buf, "ICM" );
+		vips_buf_appendf( &buf, "ICC" );
 	else if( strcmp( name, VIPS_META_EXIF_NAME ) == 0 )
 		vips_buf_appendf( &buf, "EXIF" );
 	else if( vips_isprefix( "magickprofile-", name ) ) 
@@ -692,4 +813,4 @@ magick_set_magick_profile( Image *image,
 	return( 0 );
 }
 
-#endif /*HAVE_MAGICK*/
+#endif /*defined(HAVE_MAGICK6) || defined(HAVE_MAGICK7)*/
