@@ -6,6 +6,12 @@
  * 17/2/19
  * 	- support ICC, XMP, EXIF, IPTC metadata
  * 	- write with a single call to vips_sink_disc()
+ * 29/6/19
+ * 	- support "strip" option
+ * 6/7/19 [deftomat]
+ * 	- support array of delays 
+ * 5/8/19 DarthSim
+ * 	- support GIF optimization
  */
 
 /*
@@ -59,6 +65,8 @@ typedef struct _VipsForeignSaveMagick {
 	char *filename;		/* NULL during buffer output */
 	char *format;
 	int quality;
+	gboolean optimize_gif_frames;
+	gboolean optimize_gif_transparency;
 
 	ImageInfo *image_info;
 	ExceptionInfo *exception;
@@ -68,6 +76,9 @@ typedef struct _VipsForeignSaveMagick {
 	Image *current_image;
 
 	int page_height;
+	GValue delay_gvalue;
+	int *delays;
+	int delays_length;
 
 	/* The position of current_image in the output.
 	 */
@@ -93,6 +104,7 @@ vips_foreign_save_magick_dispose( GObject *gobject )
 	VIPS_FREEF( DestroyImageList, magick->images );
 	VIPS_FREEF( DestroyImageInfo, magick->image_info );
 	VIPS_FREEF( magick_destroy_exception, magick->exception );
+	g_value_unset( &magick->delay_gvalue );
 
 	G_OBJECT_CLASS( vips_foreign_save_magick_parent_class )->
 		dispose( gobject );
@@ -110,6 +122,7 @@ vips_foreign_save_magick_next_image( VipsForeignSaveMagick *magick )
 	Image *image;
 	int number;
 	const char *str;
+	int page_index;
 
 	g_assert( !magick->current_image );
 
@@ -136,25 +149,41 @@ vips_foreign_save_magick_next_image( VipsForeignSaveMagick *magick )
 	}
 
 	if( !magick_set_image_size( image, 
-		im->Xsize, magick->page_height, magick->exception ) )
+		im->Xsize, magick->page_height, magick->exception ) ) {
+		magick_vips_error( class->nickname, magick->exception ); 
 		return( -1 );
+	}
 
-	if( vips_image_get_typeof( im, "gif-delay" ) &&
-		!vips_image_get_int( im, "gif-delay", &number ) )
-		image->delay = (size_t) number;
+	/* Delay must be converted from milliseconds into centiseconds
+	 * as GIF image requires centiseconds.
+	 */
+	if( magick->delays ) {
+		page_index = magick->position.top / magick->page_height;
+		if( page_index < magick->delays_length ) 
+			image->delay = 
+				VIPS_RINT( magick->delays[page_index] / 10.0 );
+	}
 
 	/* ImageMagick uses iterations like this (at least in gif save):
 	 * 	0 - set 0 loops (infinite)
 	 * 	1 - don't write the netscape extension block
 	 * 	2 - loop once
 	 * 	3 - loop twice etc.
-	 *
-	 * We have the simple gif meaning, so we must add one unless it's
-	 * zero.
 	 */
-	if( vips_image_get_typeof( im, "gif-loop" ) &&
-		!vips_image_get_int( im, "gif-loop", &number ) )
-		image->iterations = (size_t) (number ? number + 1 : 0);
+	if( vips_image_get_typeof( im, "loop" ) &&
+		!vips_image_get_int( im, "loop", &number ) ) {
+		image->iterations = (size_t) number;
+	}
+	else {
+		/* DEPRECATED "gif-loop"
+		 *
+		 * We have the simple gif meaning, so we must add one unless 
+		 * it's zero.
+		 */
+		if( vips_image_get_typeof( im, "gif-loop" ) &&
+			!vips_image_get_int( im, "gif-loop", &number ) )
+			image->iterations = (size_t) (number ? number + 1 : 0);
+	}
 
 	if( vips_image_get_typeof( im, "gif-comment" ) &&
 		!vips_image_get_string( im, "gif-comment", &str ) )
@@ -165,7 +194,8 @@ vips_foreign_save_magick_next_image( VipsForeignSaveMagick *magick )
 	 */
 	image->dispose = BackgroundDispose;
 
-	if( magick_set_magick_profile( image, im, magick->exception ) ) {
+	if( !save->strip &&
+		magick_set_magick_profile( image, im, magick->exception ) ) {
 		magick_vips_error( class->nickname, magick->exception ); 
 		return( -1 );
 	}
@@ -335,10 +365,43 @@ vips_foreign_save_magick_build( VipsObject *object )
 
 	magick->page_height = vips_image_get_page_height( im );
 
+	/* Get as a gvalue so we can keep a ref to the delay array while we
+	 * need it.
+	 */
+	if( vips_image_get_typeof( im, "delay" ) ) {
+		g_value_unset( &magick->delay_gvalue );
+		if( vips_image_get( im, "delay", &magick->delay_gvalue ) ) 
+			return( -1 );
+		magick->delays = vips_value_get_array_int( 
+			&magick->delay_gvalue, &magick->delays_length );
+	}
+
 	if( vips_sink_disc( im, 
 		vips_foreign_save_magick_write_block, magick ) ) 
 		return( -1 );
 
+	if( magick->optimize_gif_frames ) {
+		if( !magick_optimize_image_layers( &magick->images, 
+			magick->exception ) ) {
+			magick_inherit_exception( magick->exception, 
+				magick->images );
+			magick_vips_error( class->nickname, magick->exception );
+
+			return( -1 );
+		}
+	}
+
+	if( magick->optimize_gif_transparency ) {
+		if( !magick_optimize_image_transparency( magick->images, 
+			magick->exception ) ) {
+			magick_inherit_exception( magick->exception, 
+				magick->images );
+			magick_vips_error( class->nickname, magick->exception );
+
+			return( -1 );
+		}
+	}
+
 	return( 0 );
 }
 
@@ -402,11 +465,30 @@ vips_foreign_save_magick_class_init( VipsForeignSaveMagickClass *class )
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignSaveMagick, quality ),
 		0, 100, 0 );
+
+	VIPS_ARG_BOOL( class, "optimize_gif_frames", 4,
+		_( "Optimize_gif_frames" ),
+		_( "Apply GIF frames optimization" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSaveMagick, optimize_gif_frames ),
+		FALSE );
+
+	VIPS_ARG_BOOL( class, "optimize_gif_transparency", 5,
+		_( "Optimize_gif_transparency" ),
+		_( "Apply GIF transparency optimization" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSaveMagick, 
+			optimize_gif_transparency ),
+		FALSE );
 }
 
 static void
 vips_foreign_save_magick_init( VipsForeignSaveMagick *magick )
 {
+	/* Init to an int just to have something there. It is swapped for an
+	 * int array later.
+	 */
+	g_value_init( &magick->delay_gvalue, G_TYPE_INT );
 }
 
 typedef struct _VipsForeignSaveMagickFile {
@@ -511,9 +593,7 @@ vips_foreign_save_magick_buffer_build( VipsObject *object )
 		return( -1 );
 	}
 
-	/* obuf is a g_free() buffer, not vips_free().
-	 */
-	blob = vips_blob_new( (VipsCallbackFn) g_free, obuf, olen );
+	blob = vips_blob_new( (VipsCallbackFn) vips_area_free_cb, obuf, olen );
 	g_object_set( buffer, "buffer", blob, NULL );
 	vips_area_unref( VIPS_AREA( blob ) );
 
@@ -560,6 +640,8 @@ vips_foreign_save_magick_buffer_init( VipsForeignSaveMagickBuffer *buffer )
  *
  * * @quality: %gint, quality factor
  * * @format: %gchararray, format to save as
+ * * @optimize_gif_frames: %gboolean, apply GIF frames optimization
+ * * @optimize_gif_transparency: %gboolean, apply GIF transparency optimization
  *
  * Write an image using libMagick.
  *
@@ -568,6 +650,15 @@ vips_foreign_save_magick_buffer_init( VipsForeignSaveMagickBuffer *buffer )
  * Use @format to explicitly set the save format, for example, "BMP". Otherwise
  * the format is guessed from the filename suffix.
  *
+ * If @optimize_gif_frames is set, GIF frames are cropped to the smallest size
+ * while preserving the results of the GIF animation. This takes some time for
+ * computation but saves some time on encoding and produces smaller files in
+ * some cases.
+ *
+ * If @optimize_gif_transparency is set, pixels that don't change the image
+ * through animation are made transparent. This takes some time for computation
+ * but saves some time on encoding and produces smaller files in some cases.
+ *
  * See also: vips_magicksave_buffer(), vips_magickload().
  *
  * Returns: 0 on success, -1 on error.
@@ -596,6 +687,8 @@ vips_magicksave( VipsImage *in, const char *filename, ... )
  *
  * * @quality: %gint, quality factor
  * * @format: %gchararray, format to save as
+ * * @optimize_gif_frames: %gboolean, apply GIF frames optimization
+ * * @optimize_gif_transparency: %gboolean, apply GIF transparency optimization
  *
  * As vips_magicksave(), but save to a memory buffer. 
  *
