@@ -51,6 +51,8 @@
  * 	- reorganise to support invalidate on read error
  * 27/1/18
  * 	- option to attach associated images as metadata
+ * 22/6/20 adamu
+ * 	- set libvips xres/yres from openslide mpp-x/mpp-y
  */
 
 /*
@@ -332,7 +334,7 @@ readslide_attach_associated( ReadSlide *rslide, VipsImage *image )
 		openslide_get_associated_image_dimensions( rslide->osr,
 			*associated_name, &w, &h );
 		vips_image_init_fields( associated, w, h, 4, VIPS_FORMAT_UCHAR,
-			VIPS_CODING_NONE, VIPS_INTERPRETATION_RGB, 1.0, 1.0 );
+			VIPS_CODING_NONE, VIPS_INTERPRETATION_sRGB, 1.0, 1.0 );
 		vips_image_pipelinev( associated, 
 			VIPS_DEMAND_STYLE_THINSTRIP, NULL );
 
@@ -361,6 +363,17 @@ readslide_attach_associated( ReadSlide *rslide, VipsImage *image )
 	return( 0 );
 }
 
+/* Read out a resolution field, converting to pixels per mm.
+ */
+static double
+readslice_parse_res( ReadSlide *rslide, const char *name )
+{
+	const char *value = openslide_get_property_value( rslide->osr, name );
+	double mpp = g_ascii_strtod( value, NULL );
+
+	return( mpp == 0 ? 1.0 : 1000.0 / mpp );
+}
+
 static int
 readslide_parse( ReadSlide *rslide, VipsImage *image )
 {
@@ -369,6 +382,8 @@ readslide_parse( ReadSlide *rslide, VipsImage *image )
 	const char *background;
 	const char * const *properties;
 	char *associated_names;
+	double xres;
+	double yres;
 
 	rslide->osr = openslide_open( rslide->filename );
 	if( rslide->osr == NULL ) {
@@ -493,14 +508,28 @@ readslide_parse( ReadSlide *rslide, VipsImage *image )
 		rslide->bounds.height = h;
 	}
 
-	vips_image_init_fields( image, w, h, 4, VIPS_FORMAT_UCHAR,
-		VIPS_CODING_NONE, VIPS_INTERPRETATION_RGB, 1.0, 1.0 );
+	/* Try to get resolution from openslide properties.
+	 */
+	xres = 1.0;
+	yres = 1.0;
 
 	for( properties = openslide_get_property_names( rslide->osr );
-		*properties != NULL; properties++ )
-		vips_image_set_string( image, *properties,
-			openslide_get_property_value( rslide->osr,
-			*properties ) );
+		*properties != NULL; properties++ ) {
+		const char *name = *properties;
+		const char *value = 
+			openslide_get_property_value( rslide->osr, name );
+
+		/* Can be NULL for some openslides with some images.
+		 */
+		if( value ) { 
+			vips_image_set_string( image, name, value ); 
+
+			if( strcmp( *properties, "openslide.mpp-x" ) == 0 ) 
+				xres = readslice_parse_res( rslide, name );
+			if( strcmp( *properties, "openslide.mpp-y" ) == 0 ) 
+				yres = readslice_parse_res( rslide, name );
+		}
+	}
 
 	associated_names = g_strjoinv( ", ", (char **)
 		openslide_get_associated_image_names( rslide->osr ) );
@@ -508,6 +537,9 @@ readslide_parse( ReadSlide *rslide, VipsImage *image )
 		"slide-associated-images", associated_names );
 	VIPS_FREE( associated_names );
 
+	vips_image_init_fields( image, w, h, 4, VIPS_FORMAT_UCHAR,
+		VIPS_CODING_NONE, VIPS_INTERPRETATION_sRGB, xres, yres );
+
 	return( 0 );
 }
 
