@@ -6,6 +6,8 @@
  * 	- compression should be 0-9, not 1-10
  * 20/6/18 [felixbuenemann]
  * 	- support png8 palette write with palette, colours, Q, dither
+ * 24/6/20
+ * 	- add @bitdepth, deprecate @colours
  */
 
 /*
@@ -63,9 +65,17 @@ typedef struct _VipsForeignSavePng {
 	char *profile;
 	VipsForeignPngFilter filter;
 	gboolean palette;
-	int colours;
 	int Q;
 	double dither;
+	int bitdepth;
+
+	/* Set by subclasses.
+	 */
+	VipsTarget *target;
+
+	/* Deprecated.
+	 */
+	int colours;
 } VipsForeignSavePng;
 
 typedef VipsForeignSaveClass VipsForeignSavePngClass;
@@ -73,6 +83,55 @@ typedef VipsForeignSaveClass VipsForeignSavePngClass;
 G_DEFINE_ABSTRACT_TYPE( VipsForeignSavePng, vips_foreign_save_png, 
 	VIPS_TYPE_FOREIGN_SAVE );
 
+static void
+vips_foreign_save_png_dispose( GObject *gobject )
+{
+	VipsForeignSavePng *png = (VipsForeignSavePng *) gobject;
+
+	if( png->target ) 
+		vips_target_finish( png->target );
+	VIPS_UNREF( png->target );
+
+	G_OBJECT_CLASS( vips_foreign_save_png_parent_class )->
+		dispose( gobject );
+}
+
+static int
+vips_foreign_save_png_build( VipsObject *object )
+{
+	VipsForeignSave *save = (VipsForeignSave *) object;
+	VipsForeignSavePng *png = (VipsForeignSavePng *) object;
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_save_png_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	/* Deprecated "colours" arg just sets bitdepth large enough to hold
+	 * that many colours.
+	 */
+        if( vips_object_argument_isset( object, "colours" ) ) 
+		png->bitdepth = ceil( log2( png->colours ) );
+
+        if( !vips_object_argument_isset( object, "bitdepth" ) ) 
+		png->bitdepth = 
+			save->ready->BandFmt == VIPS_FORMAT_UCHAR ? 8 : 16;
+
+	/* If this is a RGB or RGBA image and a low bit depth has been
+	 * requested, enable palettization.
+	 */
+        if( save->ready->Bands > 2 &&
+		png->bitdepth < 8 )
+		png->palette = TRUE;
+
+	if( vips__png_write_target( save->ready, png->target,
+		png->compression, png->interlace, png->profile, png->filter,
+		save->strip, png->palette, png->Q, png->dither,
+		png->bitdepth ) )
+		return( -1 );
+
+	return( 0 );
+}
+
 /* Save a bit of typing.
  */
 #define UC VIPS_FORMAT_UCHAR
@@ -99,11 +158,13 @@ vips_foreign_save_png_class_init( VipsForeignSavePngClass *class )
 	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
 	VipsForeignSaveClass *save_class = (VipsForeignSaveClass *) class;
 
+	gobject_class->dispose = vips_foreign_save_png_dispose;
 	gobject_class->set_property = vips_object_set_property;
 	gobject_class->get_property = vips_object_get_property;
 
 	object_class->nickname = "pngsave_base";
 	object_class->description = _( "save png" );
+	object_class->build = vips_foreign_save_png_build;
 
 	foreign_class->suffs = vips__png_suffs;
 
@@ -146,13 +207,6 @@ vips_foreign_save_png_class_init( VipsForeignSavePngClass *class )
 		G_STRUCT_OFFSET( VipsForeignSavePng, palette ),
 		FALSE );
 
-	VIPS_ARG_INT( class, "colours", 14,
-		_( "Colours" ),
-		_( "Max number of palette colours" ),
-		VIPS_ARGUMENT_OPTIONAL_INPUT,
-		G_STRUCT_OFFSET( VipsForeignSavePng, colours ),
-		2, 256, 256 );
-
 	VIPS_ARG_INT( class, "Q", 15,
 		_( "Quality" ),
 		_( "Quantisation quality" ),
@@ -167,6 +221,20 @@ vips_foreign_save_png_class_init( VipsForeignSavePngClass *class )
 		G_STRUCT_OFFSET( VipsForeignSavePng, dither ),
 		0.0, 1.0, 1.0 );
 
+	VIPS_ARG_INT( class, "bitdepth", 17,
+		_( "Bit depth" ),
+		_( "Write as a 1, 2, 4 or 8 bit image" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSavePng, bitdepth ),
+		0, 8, 0 );
+
+	VIPS_ARG_INT( class, "colours", 14,
+		_( "Colours" ),
+		_( "Max number of palette colours" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
+		G_STRUCT_OFFSET( VipsForeignSavePng, colours ),
+		2, 256, 256 );
+
 }
 
 static void
@@ -174,11 +242,64 @@ vips_foreign_save_png_init( VipsForeignSavePng *png )
 {
 	png->compression = 6;
 	png->filter = VIPS_FOREIGN_PNG_FILTER_ALL;
-	png->colours = 256;
 	png->Q = 100;
 	png->dither = 1.0;
 }
 
+typedef struct _VipsForeignSavePngTarget {
+	VipsForeignSavePng parent_object;
+
+	VipsTarget *target;
+} VipsForeignSavePngTarget;
+
+typedef VipsForeignSavePngClass VipsForeignSavePngTargetClass;
+
+G_DEFINE_TYPE( VipsForeignSavePngTarget, vips_foreign_save_png_target, 
+	vips_foreign_save_png_get_type() );
+
+static int
+vips_foreign_save_png_target_build( VipsObject *object )
+{
+	VipsForeignSavePng *png = (VipsForeignSavePng *) object;
+	VipsForeignSavePngTarget *target = (VipsForeignSavePngTarget *) object;
+
+	png->target = target->target;
+	g_object_ref( png->target );
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_save_png_target_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	return( 0 );
+}
+
+static void
+vips_foreign_save_png_target_class_init( VipsForeignSavePngTargetClass *class )
+{
+	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+
+	gobject_class->set_property = vips_object_set_property;
+	gobject_class->get_property = vips_object_get_property;
+
+	object_class->nickname = "pngsave_target";
+	object_class->description = _( "save image to target as PNG" );
+	object_class->build = vips_foreign_save_png_target_build;
+
+	VIPS_ARG_OBJECT( class, "target", 1,
+		_( "Target" ),
+		_( "Target to save to" ),
+		VIPS_ARGUMENT_REQUIRED_INPUT, 
+		G_STRUCT_OFFSET( VipsForeignSavePngTarget, target ),
+		VIPS_TYPE_TARGET );
+
+}
+
+static void
+vips_foreign_save_png_target_init( VipsForeignSavePngTarget *target )
+{
+}
+
 typedef struct _VipsForeignSavePngFile {
 	VipsForeignSavePng parent_object;
 
@@ -193,20 +314,16 @@ G_DEFINE_TYPE( VipsForeignSavePngFile, vips_foreign_save_png_file,
 static int
 vips_foreign_save_png_file_build( VipsObject *object )
 {
-	VipsForeignSave *save = (VipsForeignSave *) object;
 	VipsForeignSavePng *png = (VipsForeignSavePng *) object;
-	VipsForeignSavePngFile *png_file = (VipsForeignSavePngFile *) object;
+	VipsForeignSavePngFile *file = (VipsForeignSavePngFile *) object;
+
+	if( !(png->target = vips_target_new_to_file( file->filename )) )
+		return( -1 );
 
 	if( VIPS_OBJECT_CLASS( vips_foreign_save_png_file_parent_class )->
 		build( object ) )
 		return( -1 );
 
-	if( vips__png_write( save->ready, 
-		png_file->filename, png->compression, png->interlace, 
-		png->profile, png->filter, save->strip, png->palette,
-		png->colours, png->Q, png->dither ) )
-		return( -1 );
-
 	return( 0 );
 }
 
@@ -250,27 +367,20 @@ G_DEFINE_TYPE( VipsForeignSavePngBuffer, vips_foreign_save_png_buffer,
 static int
 vips_foreign_save_png_buffer_build( VipsObject *object )
 {
-	VipsForeignSave *save = (VipsForeignSave *) object;
 	VipsForeignSavePng *png = (VipsForeignSavePng *) object;
+	VipsForeignSavePngBuffer *buffer = (VipsForeignSavePngBuffer *) object;
 
-	void *obuf;
-	size_t olen;
 	VipsBlob *blob;
 
+	if( !(png->target = vips_target_new_to_memory()) )
+		return( -1 );
+
 	if( VIPS_OBJECT_CLASS( vips_foreign_save_png_buffer_parent_class )->
 		build( object ) )
 		return( -1 );
 
-	if( vips__png_write_buf( save->ready, &obuf, &olen,
-		png->compression, png->interlace, png->profile, png->filter,
-		save->strip, png->palette, png->colours, png->Q, png->dither ) )
-		return( -1 );
-
-	/* vips__png_write_buf() makes a buffer that needs g_free(), not
-	 * vips_free().
-	 */
-	blob = vips_blob_new( (VipsCallbackFn) g_free, obuf, olen );
-	g_object_set( object, "buffer", blob, NULL );
+	g_object_get( png->target, "blob", &blob, NULL );
+	g_object_set( buffer, "buffer", blob, NULL );
 	vips_area_unref( VIPS_AREA( blob ) );
 
 	return( 0 );
@@ -312,14 +422,14 @@ vips_foreign_save_png_buffer_init( VipsForeignSavePngBuffer *buffer )
  *
  * Optional arguments:
  *
- * * @compression: compression level
- * * @interlace: interlace image
- * * @profile: ICC profile to embed
+ * * @compression: %gint, compression level
+ * * @interlace: %gboolean, interlace image
+ * * @profile: %gchararray, ICC profile to embed
  * * @filter: #VipsForeignPngFilter row filter flag(s)
- * * @palette: enable quantisation to 8bpp palette
- * * @colours: max number of palette colours for quantisation
- * * @Q: quality for 8bpp quantisation (does not exceed @colours)
- * * @dither: amount of dithering for 8bpp quantization
+ * * @palette: %gboolean, enable quantisation to 8bpp palette
+ * * @Q: %gint, quality for 8bpp quantisation 
+ * * @dither: %gdouble, amount of dithering for 8bpp quantization
+ * * @bitdepth: %int, set write bit depth to 1, 2, 4 or 8
  *
  * Write a VIPS image to a file as PNG.
  *
@@ -345,13 +455,15 @@ vips_foreign_save_png_buffer_init( VipsForeignSavePngBuffer *buffer )
  * alpha before saving. Images with more than one byte per band element are
  * saved as 16-bit PNG, others are saved as 8-bit PNG.
  *
- * Set @palette to %TRUE to enable quantisation to an 8-bit per pixel palette
- * image with alpha transparency support. If @colours is given, it limits the
- * maximum number of palette entries. Similar to JPEG the quality can also be
- * be changed with the @Q parameter which further reduces the palette size and
- * @dither controls the amount of Floyd-Steinberg dithering.
+ * Set @palette to %TRUE to enable palette mode for RGB or RGBA images. A
+ * palette will be computed with enough space for @bitdepth (1, 2, 4 or 8) 
+ * bits. Use @Q to set the optimisation effort, and @dither to set the degree of
+ * Floyd-Steinberg dithering.
  * This feature requires libvips to be compiled with libimagequant.
  *
+ * You can also set @bitdepth for mono and mono + alpha images, and the image
+ * will be quantized.
+ *
  * XMP metadata is written to the XMP chunk. PNG comments are written to
  * separate text chunks.
  *
@@ -381,14 +493,14 @@ vips_pngsave( VipsImage *in, const char *filename, ... )
  *
  * Optional arguments:
  *
- * * @compression: compression level
- * * @interlace: interlace image
- * * @profile: ICC profile to embed
- * * @filter: libpng row filter flag(s)
- * * @palette: enable quantisation to 8bpp palette
- * * @colours: max number of palette colours for quantisation
- * * @Q: quality for 8bpp quantisation (does not exceed @colours)
- * * @dither: amount of dithering for 8bpp quantization
+ * * @compression: %gint, compression level
+ * * @interlace: %gboolean, interlace image
+ * * @profile: %gchararray, ICC profile to embed
+ * * @filter: #VipsForeignPngFilter row filter flag(s)
+ * * @palette: %gboolean, enable quantisation to 8bpp palette
+ * * @Q: %gint, quality for 8bpp quantisation 
+ * * @dither: %gdouble, amount of dithering for 8bpp quantization
+ * * @bitdepth: %int, set write bit depth to 1, 2, 4 or 8
  *
  * As vips_pngsave(), but save to a memory buffer. 
  *
@@ -427,3 +539,39 @@ vips_pngsave_buffer( VipsImage *in, void **buf, size_t *len, ... )
 
 	return( result );
 }
+
+/**
+ * vips_pngsave_target: (method)
+ * @in: image to save 
+ * @target: save image to this target
+ * @...: %NULL-terminated list of optional named arguments
+ *
+ * Optional arguments:
+ *
+ * * @compression: compression level
+ * * @interlace: interlace image
+ * * @profile: ICC profile to embed
+ * * @filter: libpng row filter flag(s)
+ * * @palette: enable quantisation to 8bpp palette
+ * * @Q: quality for 8bpp quantisation 
+ * * @dither: amount of dithering for 8bpp quantization
+ * * @bitdepth: %int, set write bit depth to 1, 2, 4 or 8
+ *
+ * As vips_pngsave(), but save to a target.
+ *
+ * See also: vips_pngsave(), vips_image_write_to_target().
+ *
+ * Returns: 0 on success, -1 on error.
+ */
+int
+vips_pngsave_target( VipsImage *in, VipsTarget *target, ... )
+{
+	va_list ap;
+	int result;
+
+	va_start( ap, target );
+	result = vips_call_split( "pngsave_target", ap, in, target );
+	va_end( ap );
+
+	return( result );
+}
