@@ -14,6 +14,11 @@
  * 	- handle scaling of svg files missing width and height attributes
  * 22/3/18 lovell
  * 	- svgload was missing is_a
+ * 28/6/19
+ * 	- add "unlimited"
+ * 	- requires us to use the gio API to librsvg
+ * 11/9/19
+ * 	- rework as a sequential loader to reduce overcomputation
  */
 
 /*
@@ -59,10 +64,10 @@
 #include <ctype.h>
 
 #include <vips/vips.h>
-#include <vips/buf.h>
 #include <vips/internal.h>
+#include <vips/debug.h>
 
-#ifdef HAVE_RSVG
+#if defined(HAVE_RSVG)
 
 #include <cairo.h>
 #include <librsvg/rsvg.h>
@@ -107,6 +112,10 @@ typedef struct _VipsForeignLoadSvg {
 	 */
 	double cairo_scale;
 
+	/* Allow SVGs of any size.
+	 */
+	gboolean unlimited;
+
 	RsvgHandle *page;
 
 } VipsForeignLoadSvg;
@@ -141,7 +150,7 @@ vips_foreign_load_svg_is_a( const void *buf, size_t len )
 	/* If the buffer looks like a zip, deflate to here and then search
 	 * that for <svg.
 	 */
-	char obuf[224];
+	char obuf[SVG_HEADER_SIZE];
 #endif /*HANDLE_SVGZ*/
 
 	int i;
@@ -177,8 +186,10 @@ vips_foreign_load_svg_is_a( const void *buf, size_t len )
 		do {
 			zs.avail_out = sizeof( obuf ) - opos;
 			zs.next_out = (unsigned char *) obuf + opos;
-			if( inflate( &zs, Z_NO_FLUSH ) < Z_OK ) 
+			if( inflate( &zs, Z_NO_FLUSH ) < Z_OK ) {
+				inflateEnd( &zs );
 				return( FALSE );
+			}
 			opos = sizeof( obuf ) - zs.avail_out;
 		} while( opos < sizeof( obuf ) && 
 			zs.avail_in > 0 );
@@ -200,7 +211,7 @@ vips_foreign_load_svg_is_a( const void *buf, size_t len )
 	 * before the <svg line.
 	 *
 	 * Simple rules:
-	 * - first 24 chars are plain ascii
+	 * - first 24 chars are plain ascii (x09-x7F)
 	 * - first SVG_HEADER_SIZE chars contain "<svg", upper or lower case.
 	 *
 	 * We could rsvg_handle_new_from_data() on the buffer, but that can be
@@ -209,7 +220,7 @@ vips_foreign_load_svg_is_a( const void *buf, size_t len )
 	if( len < 24 )
 		return( 0 );
 	for( i = 0; i < 24; i++ )
-		if( !isascii( str[i] ) )
+		if( !isascii( str[i] ) || str[i] < 9 )
 			return( FALSE );
 	for( i = 0; i < SVG_HEADER_SIZE && i < len - 5; i++ )
 		if( g_ascii_strncasecmp( str + i, "<svg", 4 ) == 0 )
@@ -232,15 +243,13 @@ vips_foreign_load_svg_dispose( GObject *gobject )
 static VipsForeignFlags
 vips_foreign_load_svg_get_flags_filename( const char *filename )
 {
-	/* We can render any part of the page on demand.
-	 */
-	return( VIPS_FOREIGN_PARTIAL );
+	return( VIPS_FOREIGN_SEQUENTIAL );
 }
 
 static VipsForeignFlags
 vips_foreign_load_svg_get_flags( VipsForeignLoad *load )
 {
-	return( VIPS_FOREIGN_PARTIAL );
+	return( VIPS_FOREIGN_SEQUENTIAL );
 }
 
 static void
@@ -273,9 +282,10 @@ vips_foreign_load_svg_parse( VipsForeignLoadSvg *svg, VipsImage *out )
 			 * cairo instead.
 			 */
 			svg->cairo_scale = scale;
-			width = width * scale;
-			height = height * scale;
-		} else {
+			width = VIPS_ROUND_UINT( width * scale );
+			height = VIPS_ROUND_UINT( height * scale );
+		} 
+		else {
 			/* SVG with width and height reports correctly scaled 
 			 * dimensions.
 			 */
@@ -321,6 +331,12 @@ vips_foreign_load_svg_generate( VipsRegion *or,
 	cairo_t *cr;
 	int y;
 
+#ifdef DEBUG
+	printf( "vips_foreign_load_svg_generate:\n     "
+		"left = %d, top = %d, width = %d, height = %d\n", 
+		r->left, r->top, r->width, r->height ); 
+#endif /*DEBUG*/
+
 	/* rsvg won't always paint the background.
 	 */
 	vips_region_black( or ); 
@@ -364,44 +380,30 @@ vips_foreign_load_svg_load( VipsForeignLoad *load )
 {
 	VipsForeignLoadSvg *svg = (VipsForeignLoadSvg *) load;
 	VipsImage **t = (VipsImage **) 
-		vips_object_local_array( (VipsObject *) load, 2 );
+		vips_object_local_array( (VipsObject *) load, 3 );
 
-	int tile_width;
-	int tile_height;
-	int max_tiles;
-
-	/* Read to this image, then cache to out, see below.
+	/* librsvg starts to fail if any axis in a single render call is over
+	 * RSVG_MAX_WIDTH pixels. vips_sequential() will use a tilecache where 
+	 * each tile is a strip the width of the image and (in this case) 
+	 * 2000 pixels high. To be able to render images wider than 
+	 * RSVG_MAX_WIDTH pixels, we need to chop these strips up.
+	 * We use a small tilecache between the seq and our gen to do this.
+	 *
+	 * Make tiles 2000 pixels high to limit overcomputation. 
 	 */
 	t[0] = vips_image_new(); 
-
 	vips_foreign_load_svg_parse( svg, t[0] ); 
 	if( vips_image_generate( t[0], 
-		NULL, vips_foreign_load_svg_generate, NULL, svg, NULL ) )
-		return( -1 );
-
-	/* librsvg starts to fail if any axis in a single render call is over 
-	 * 32767. Use a tilecache so we can render very wide images, though we
-	 * set it up like a linecache. 
-	 *
-	 * Don't thread the cache: we rely on this to keep calls to rsvg 
-	 * single-threaded.
-	 *
-	 * Make tiles 2000 pixels high to limit overcomputation. Make sure we
-	 * have two rows of tiles so we don't recompute requests that cross
-	 * tile boundaries. 
-	 */
-	tile_width = VIPS_MIN( t[0]->Xsize, RSVG_MAX_WIDTH );
-	max_tiles = 2 * VIPS_ROUND_UP( t[0]->Xsize, RSVG_MAX_WIDTH ) / 
-		RSVG_MAX_WIDTH;
-	tile_height = 2000;
-	if( vips_tilecache( t[0], &t[1],
-		"tile_width", tile_width,
-		"tile_height", tile_height,
-		"max_tiles", max_tiles,
-		"access", VIPS_ACCESS_SEQUENTIAL,
-		NULL ) ) 
-		return( -1 );
-	if( vips_image_write( t[1], load->real ) ) 
+		NULL, vips_foreign_load_svg_generate, NULL, svg, NULL ) ||
+		vips_tilecache( t[0], &t[1],
+			"tile_width", VIPS_MIN( t[0]->Xsize, RSVG_MAX_WIDTH ),
+			"tile_height", 2000,
+			"max_tiles", 1,
+			NULL ) ||
+		vips_sequential( t[1], &t[2], 
+			"tile_height", 2000, 
+			NULL ) ||
+		vips_image_write( t[2], load->real ) ) 
 		return( -1 );
 
 	return( 0 );
@@ -412,6 +414,7 @@ vips_foreign_load_svg_class_init( VipsForeignLoadSvgClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
 	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsForeignClass *foreign_class = (VipsForeignClass *) class;
 	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
 
 	gobject_class->dispose = vips_foreign_load_svg_dispose;
@@ -421,6 +424,10 @@ vips_foreign_load_svg_class_init( VipsForeignLoadSvgClass *class )
 	object_class->nickname = "svgload";
 	object_class->description = _( "load SVG with rsvg" );
 
+	/* is_a() is not that quick ... lower the priority.
+	 */
+	foreign_class->priority = -5;
+
 	load_class->get_flags_filename = 
 		vips_foreign_load_svg_get_flags_filename;
 	load_class->get_flags = vips_foreign_load_svg_get_flags;
@@ -440,6 +447,13 @@ vips_foreign_load_svg_class_init( VipsForeignLoadSvgClass *class )
 		G_STRUCT_OFFSET( VipsForeignLoadSvg, scale ),
 		0.001, 100000.0, 1.0 );
 
+	VIPS_ARG_BOOL( class, "unlimited", 23,
+		_( "Unlimited" ),
+		_( "Allow SVG of any size" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignLoadSvg, unlimited ),
+		FALSE );
+
 }
 
 static void
@@ -450,6 +464,104 @@ vips_foreign_load_svg_init( VipsForeignLoadSvg *svg )
 	svg->cairo_scale = 1.0;
 }
 
+typedef struct _VipsForeignLoadSvgSource {
+	VipsForeignLoadSvg parent_object;
+
+	/* Load from a source.
+	 */
+	VipsSource *source;
+
+} VipsForeignLoadSvgSource;
+
+typedef VipsForeignLoadClass VipsForeignLoadSvgSourceClass;
+
+G_DEFINE_TYPE( VipsForeignLoadSvgSource, vips_foreign_load_svg_source, 
+	vips_foreign_load_svg_get_type() );
+
+gboolean
+vips_foreign_load_svg_source_is_a_source( VipsSource *source )
+{
+	unsigned char *data;
+	size_t bytes_read;
+
+	if( (bytes_read = vips_source_sniff_at_most( source, 
+		&data, SVG_HEADER_SIZE )) <= 0 )
+		return( FALSE );
+
+	return( vips_foreign_load_svg_is_a( data, bytes_read ) );
+}
+
+static int
+vips_foreign_load_svg_source_header( VipsForeignLoad *load )
+{
+	VipsForeignLoadSvg *svg = (VipsForeignLoadSvg *) load;
+	VipsForeignLoadSvgSource *source = 
+		(VipsForeignLoadSvgSource *) load;
+	RsvgHandleFlags flags = svg->unlimited ? RSVG_HANDLE_FLAG_UNLIMITED : 0;
+
+	GError *error = NULL;
+
+	GInputStream *gstream;
+
+	if( vips_source_rewind( source->source ) )
+		return( -1 );
+
+	gstream = vips_g_input_stream_new_from_source( source->source );
+	if( !(svg->page = rsvg_handle_new_from_stream_sync( 
+		gstream, NULL, flags, NULL, &error )) ) {
+		g_object_unref( gstream );
+		vips_g_error( &error );
+		return( -1 ); 
+	}
+	g_object_unref( gstream );
+
+	return( vips_foreign_load_svg_header( load ) );
+}
+
+static int
+vips_foreign_load_svg_source_load( VipsForeignLoad *load )
+{
+	VipsForeignLoadSvgSource *source = (VipsForeignLoadSvgSource *) load;
+
+	if( vips_source_rewind( source->source ) ||
+		vips_foreign_load_svg_load( load ) ||
+		vips_source_decode( source->source ) )
+		return( -1 );
+
+	return( 0 );
+}
+
+static void
+vips_foreign_load_svg_source_class_init( VipsForeignLoadSvgSourceClass *class )
+{
+	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
+
+	gobject_class->set_property = vips_object_set_property;
+	gobject_class->get_property = vips_object_get_property;
+
+	object_class->nickname = "svgload_source";
+	object_class->description = _( "load svg from source" );
+
+	load_class->is_a_source = vips_foreign_load_svg_source_is_a_source;
+	load_class->header = vips_foreign_load_svg_source_header;
+	load_class->load = vips_foreign_load_svg_source_load;
+
+	VIPS_ARG_OBJECT( class, "source", 1,
+		_( "Source" ),
+		_( "Source to load from" ),
+		VIPS_ARGUMENT_REQUIRED_INPUT, 
+		G_STRUCT_OFFSET( VipsForeignLoadSvgSource, source ),
+		VIPS_TYPE_SOURCE );
+
+}
+
+static void
+vips_foreign_load_svg_source_init( VipsForeignLoadSvgSource *source )
+{
+}
+
 typedef struct _VipsForeignLoadSvgFile {
 	VipsForeignLoadSvg parent_object;
 
@@ -480,14 +592,20 @@ vips_foreign_load_svg_file_header( VipsForeignLoad *load )
 {
 	VipsForeignLoadSvg *svg = (VipsForeignLoadSvg *) load;
 	VipsForeignLoadSvgFile *file = (VipsForeignLoadSvgFile *) load;
+	RsvgHandleFlags flags = svg->unlimited ? RSVG_HANDLE_FLAG_UNLIMITED : 0;
 
 	GError *error = NULL;
 
-	if( !(svg->page = rsvg_handle_new_from_file( 
-		file->filename, &error )) ) { 
+	GFile *gfile;
+
+	gfile = g_file_new_for_path( file->filename );
+	if( !(svg->page = rsvg_handle_new_from_gfile_sync( 
+		gfile, flags, NULL, &error )) ) { 
+		g_object_unref( gfile );
 		vips_g_error( &error );
 		return( -1 ); 
 	}
+	g_object_unref( gfile );
 
 	VIPS_SETSTR( load->out->filename, file->filename );
 
@@ -558,14 +676,21 @@ vips_foreign_load_svg_buffer_header( VipsForeignLoad *load )
 	VipsForeignLoadSvg *svg = (VipsForeignLoadSvg *) load;
 	VipsForeignLoadSvgBuffer *buffer = 
 		(VipsForeignLoadSvgBuffer *) load;
+	RsvgHandleFlags flags = svg->unlimited ? RSVG_HANDLE_FLAG_UNLIMITED : 0;
 
 	GError *error = NULL;
 
-	if( !(svg->page = rsvg_handle_new_from_data( 
-		buffer->buf->data, buffer->buf->length, &error )) ) { 
+	GInputStream *gstream;
+
+	gstream = g_memory_input_stream_new_from_data( 
+		buffer->buf->data, buffer->buf->length, NULL );
+	if( !(svg->page = rsvg_handle_new_from_stream_sync( 
+		gstream, NULL, flags, NULL, &error )) ) { 
+		g_object_unref( gstream );
 		vips_g_error( &error );
 		return( -1 ); 
 	}
+	g_object_unref( gstream );
 
 	return( vips_foreign_load_svg_header( load ) );
 }
@@ -612,6 +737,7 @@ vips_foreign_load_svg_buffer_init( VipsForeignLoadSvgBuffer *buffer )
  *
  * * @dpi: %gdouble, render at this DPI
  * * @scale: %gdouble, scale render by this factor
+ * * @unlimited: %gboolean, allow SVGs of any size
  *
  * Render a SVG file into a VIPS image.  Rendering uses the librsvg library
  * and should be fast.
@@ -622,6 +748,9 @@ vips_foreign_load_svg_buffer_init( VipsForeignLoadSvgBuffer *buffer )
  * This function only reads the image header and does not render any pixel
  * data. Rendering occurs when pixels are accessed.
  *
+ * SVGs larger than 10MB are normally blocked for security. Set @unlimited to
+ * allow SVGs of any size.
+ *
  * See also: vips_image_new_from_file().
  *
  * Returns: 0 on success, -1 on error.
@@ -650,6 +779,7 @@ vips_svgload( const char *filename, VipsImage **out, ... )
  *
  * * @dpi: %gdouble, render at this DPI
  * * @scale: %gdouble, scale render by this factor
+ * * @unlimited: %gboolean, allow SVGs of any size
  *
  * Read a SVG-formatted memory block into a VIPS image. Exactly as
  * vips_svgload(), but read from a memory buffer. 
@@ -681,3 +811,28 @@ vips_svgload_buffer( void *buf, size_t len, VipsImage **out, ... )
 	return( result );
 }
 
+/**
+ * vips_svgload_source:
+ * @source: source to load from
+ * @out: (out): image to write
+ * @...: %NULL-terminated list of optional named arguments
+ *
+ * Exactly as vips_svgload(), but read from a source. 
+ *
+ * See also: vips_svgload().
+ *
+ * Returns: 0 on success, -1 on error.
+ */
+int
+vips_svgload_source( VipsSource *source, VipsImage **out, ... )
+{
+	va_list ap;
+	int result;
+
+	va_start( ap, out );
+	result = vips_call_split( "svgload_source", ap, source, out );
+	va_end( ap );
+
+	return( result );
+}
+
