@@ -12,6 +12,14 @@
  * 	- target libwebp 0.5+ and remove some ifdefs
  * 	- add animated webp write
  * 	- use libwebpmux instead of our own thing, phew
+ * 6/7/19 [deftomat]
+ * 	- support array of delays 
+ * 8/7/19
+ * 	- set loop even if we strip
+ * 14/10/19
+ * 	- revise for target IO
+ * 18/7/20
+ * 	- add @profile param to match tiff, jpg, etc.
  */
 
 /*
@@ -82,6 +90,7 @@ typedef struct {
 	int kmin;
 	int kmax;
 	gboolean strip;
+	const char *profile;
 
 	WebPConfig config;
 
@@ -140,7 +149,7 @@ vips_webp_write_init( VipsWebPWrite *write, VipsImage *image,
 	gboolean smart_subsample, gboolean near_lossless,
 	int alpha_q, int reduction_effort,
 	gboolean min_size, int kmin, int kmax,
-	gboolean strip )
+	gboolean strip, const char *profile )
 {
 	write->image = NULL;
 	write->Q = Q;
@@ -154,12 +163,13 @@ vips_webp_write_init( VipsWebPWrite *write, VipsImage *image,
 	write->kmin = kmin;
 	write->kmax = kmax;
 	write->strip = strip;
+	write->profile = profile;
 	WebPMemoryWriterInit( &write->memory_writer );
 	write->enc = NULL;
 	write->mux = NULL;
 
-	/* We must copy the input image since we will be updating the
-	 * metadata when we write the exif.
+	/* We need a copy of the input image in case we change the metadata
+	 * eg. in vips__exif_update().
 	 */
 	if( vips_copy( image, &write->image, NULL ) ) {
 		vips_webp_write_unset( write );
@@ -193,7 +203,7 @@ vips_webp_write_init( VipsWebPWrite *write, VipsImage *image,
 	if( near_lossless )
 		write->config.near_lossless = Q;
 	if( smart_subsample )
-		write->config.preprocessing |= 4;
+		write->config.use_sharp_yuv = 1;
 
 	if( !WebPValidateConfig( &write->config ) ) {
 		vips_webp_write_unset( write );
@@ -284,32 +294,16 @@ write_webp_single( VipsWebPWrite *write, VipsImage *image )
 	return( 0 );
 }
 
-/* Get a defaulted int value.
- */
-static int
-get_int( VipsImage *image, const char *field, int default_value )
-{
-	int value;
-
-	if( vips_image_get_typeof( image, field ) &&
-		!vips_image_get_int( image, field, &value ) )
-		return( value );
-
-	return( default_value );
-}
-
 /* Write a set of animated frames into write->memory_writer.
  */
 static int
 write_webp_anim( VipsWebPWrite *write, VipsImage *image, int page_height )
 {
-	/* 100ms is the webp default. gif-delay is in centiseconds (the GIF
-	 * standard).
-	 */
-	const int delay = 10 * get_int( image, "gif-delay", 10 );
-
 	WebPAnimEncoderOptions anim_config;
 	WebPData webp_data;
+	int gif_delay;
+	int *delay;
+	int delay_length;
 	int top;
 	int timestamp_ms;
 
@@ -320,6 +314,7 @@ write_webp_anim( VipsWebPWrite *write, VipsImage *image, int page_height )
 	}
 
 	anim_config.minimize_size = write->min_size;
+	anim_config.allow_mixed = write->min_size;
 	anim_config.kmin = write->kmin;
 	anim_config.kmax = write->kmax;
 
@@ -331,10 +326,26 @@ write_webp_anim( VipsWebPWrite *write, VipsImage *image, int page_height )
 		return( -1 );
 	}
 
+	/* There might just be the old gif-delay field. This is centiseconds.
+	 */
+	gif_delay = 4;
+	if( vips_image_get_typeof( image, "gif-delay" ) &&
+		vips_image_get_int( image, "gif-delay", &gif_delay ) )
+		return( -1 );
+
+	/* New images have an array of ints instead.
+	 */
+	delay = NULL;
+	if( vips_image_get_typeof( image, "delay" ) &&
+		vips_image_get_array_int( image, "delay", 
+			&delay, &delay_length ) )
+		return( -1 );
+
 	timestamp_ms = 0;
 	for( top = 0; top < image->Ysize; top += page_height ) {
 		VipsImage *x;
 		WebPPicture pic;
+		int page_index;
 
 		if( vips_crop( image, &x, 
 			0, top, image->Xsize, page_height, NULL ) )
@@ -357,7 +368,21 @@ write_webp_anim( VipsWebPWrite *write, VipsImage *image, int page_height )
 
 		WebPPictureFree( &pic );
 
-		timestamp_ms += delay;
+		page_index = top / page_height;
+		if( delay &&
+			page_index < delay_length )
+			timestamp_ms += delay[page_index];
+		else 
+			timestamp_ms += gif_delay * 10;
+	}
+
+	/* Closes encoder and adds last frame delay.
+	 */
+	if( !WebPAnimEncoderAdd( write->enc, 
+		NULL, timestamp_ms, NULL ) ) {
+		vips_error( "vips2webp",
+			"%s", _( "anim close error" ) );
+		return( -1 );
 	}
 
 	if( !WebPAnimEncoderAssemble( write->enc, &webp_data ) ) {
@@ -431,10 +456,6 @@ vips_webp_add_chunks( VipsWebPWrite *write )
 {
 	int i;
 
-	if( vips_image_get_typeof( write->image, "gif-loop" ) )
-		vips_webp_set_count( write, 
-			get_int( write->image, "gif-loop", 0 ) );
-
 	for( i = 0; i < vips__n_webp_names; i++ ) { 
 		const char *vips_name = vips__webp_names[i].vips;
 		const char *webp_name = vips__webp_names[i].webp;
@@ -459,11 +480,6 @@ vips_webp_add_metadata( VipsWebPWrite *write )
 {
 	WebPData data;
 
-	/* Rebuild the EXIF block, if any, ready for writing. 
-	 */
-	if( vips__exif_update( write->image ) )
-		return( -1 ); 
-
 	data.bytes = write->memory_writer.mem;
 	data.size = write->memory_writer.size;
 
@@ -474,10 +490,43 @@ vips_webp_add_metadata( VipsWebPWrite *write )
 		return( -1 );
 	}
 
+	if( vips_image_get_typeof( write->image, "loop" ) ) {
+		int loop;
+
+		if( vips_image_get_int( write->image, "loop", &loop ) )
+			return( -1 );
+
+		vips_webp_set_count( write, loop );
+	}
+	/* DEPRECATED "gif-loop"
+	 */
+	else if ( vips_image_get_typeof( write->image, "gif-loop" ) ) {
+		int gif_loop;
+
+		if( vips_image_get_int( write->image, "gif-loop", &gif_loop ) )
+			return( -1 );
+
+		vips_webp_set_count( write, gif_loop == 0 ? 0 : gif_loop + 1 );
+	}
+
 	/* Add extra metadata.
 	 */
-	if( vips_webp_add_chunks( write ) ) 
-		return( -1 );
+	if( !write->strip ) {
+		/* We need to rebuild exif from the other image tags before
+		 * writing the metadata.
+		 */
+		if( vips__exif_update( write->image ) )
+			return( -1 );
+
+		/* Override profile.
+		 */
+		if( write->profile &&
+			vips__profile_set( write->image, write->profile ) )
+			return( -1 );
+
+		if( vips_webp_add_chunks( write ) ) 
+			return( -1 );
+	}
 
 	if( WebPMuxAssemble( write->mux, &data ) != WEBP_MUX_OK ) {
 		vips_error( "vips2webp", "%s", _( "mux error" ) );
@@ -494,19 +543,19 @@ vips_webp_add_metadata( VipsWebPWrite *write )
 }
 
 int
-vips__webp_write_file( VipsImage *image, const char *filename, 
+vips__webp_write_target( VipsImage *image, VipsTarget *target,
 	int Q, gboolean lossless, VipsForeignWebpPreset preset,
 	gboolean smart_subsample, gboolean near_lossless,
 	int alpha_q, int reduction_effort,
 	gboolean min_size, int kmin, int kmax,
-	gboolean strip )
+	gboolean strip, const char *profile )
 {
 	VipsWebPWrite write;
-	FILE *fp;
 
 	if( vips_webp_write_init( &write, image,
 		Q, lossless, preset, smart_subsample, near_lossless,
-		alpha_q, reduction_effort, min_size, kmin, kmax, strip ) )
+		alpha_q, reduction_effort, min_size, kmin, kmax, strip,
+		profile ) )
 		return( -1 );
 
 	if( write_webp( &write ) ) {
@@ -514,62 +563,18 @@ vips__webp_write_file( VipsImage *image, const char *filename,
 		return( -1 );
 	}
 
-	if( !strip &&
-		vips_webp_add_metadata( &write ) ) {
+	if( vips_webp_add_metadata( &write ) ) {
 		vips_webp_write_unset( &write );
 		return( -1 );
 	}
 
-	if( !(fp = vips__file_open_write( filename, FALSE )) ) {
+	if( vips_target_write( target, 
+		write.memory_writer.mem, write.memory_writer.size ) ) {
 		vips_webp_write_unset( &write );
 		return( -1 );
 	}
 
-	if( vips__file_write( 
-		write.memory_writer.mem, write.memory_writer.size, 1, fp ) ) {
-		fclose( fp );
-		vips_webp_write_unset( &write );
-		return( -1 );
-	}
-
-	fclose( fp );
-
-	vips_webp_write_unset( &write );
-
-	return( 0 );
-}
-
-int
-vips__webp_write_buffer( VipsImage *image, void **obuf, size_t *olen, 
-	int Q, gboolean lossless, VipsForeignWebpPreset preset,
-	gboolean smart_subsample, gboolean near_lossless,
-	int alpha_q, int reduction_effort,
-	gboolean min_size, int kmin, int kmax,
-	gboolean strip )
-{
-	VipsWebPWrite write;
-
-	if( vips_webp_write_init( &write, image,
-		Q, lossless, preset, smart_subsample, near_lossless,
-		alpha_q, reduction_effort, min_size, kmin, kmax, strip ) )
-		return( -1 );
-
-	if( write_webp( &write ) ) {
-		vips_webp_write_unset( &write );
-		return( -1 );
-	}
-
-	if( !strip &&
-		vips_webp_add_metadata( &write ) ) {
-		vips_webp_write_unset( &write );
-		return( -1 );
-	}
-
-	*obuf = write.memory_writer.mem;
-	*olen = write.memory_writer.size;
-	write.memory_writer.mem = NULL;
-	write.memory_writer.size = 0;
-	write.memory_writer.max_size = 0;
+	vips_target_finish( target );
 
 	vips_webp_write_unset( &write );
 
