@@ -20,6 +20,12 @@
  * 30/4/19
  * 	- deprecate shrink, use scale instead, and make it a double ... this
  * 	  lets us do faster and more accurate thumbnailing
+ * 27/6/19
+ * 	- disable alpha output if all frame fill the canvas and are solid
+ * 6/7/19 [deftomat]
+ * 	- support array of delays 
+ * 14/10/19
+ * 	- revise for source IO
  */
 
 /*
@@ -75,15 +81,12 @@
 /* What we track during a read.
  */
 typedef struct {
-	/* File source.
-	 */
-	char *filename;
+	VipsImage *out;
+	VipsSource *source;
 
-	/* Memory source. We use gint64 rather than size_t since we use
-	 * vips_file_length() and vips__mmap() for file sources.
+	/* The data we load, as a webp object.
 	 */
-	const void *data;
-	gint64 length;
+	WebPData data;
 
 	/* Load this page (frame number).
 	 */
@@ -97,17 +100,22 @@ typedef struct {
 	 */
 	double scale;
 
+	/* Size of each frame in input image coordinates.
+	 */
+	int canvas_width;
+	int canvas_height;
+
+	/* Size of each frame, in scaled output image coordinates,
+	 */
+	int frame_width;
+	int frame_height;
+
 	/* Size of final output image. 
 	 */
 	int width;
 	int height;
 
-	/* Size of each frame.
-	 */
-	int frame_width;
-	int frame_height;
-
-	/* TRUE for RGBA.
+	/* TRUE if we will save the final image as RGBA.
 	 */
 	int alpha;
 
@@ -115,13 +123,9 @@ typedef struct {
 	 */
 	int frame_count;
 
-	/* Delay between frames. We don't let this change between frames.
+	/* Delays between frames (in miliseconds).
 	 */
-	int delay;
-
-	/* If we are opening a file object, the fd.
-	 */
-	int fd;
+	int *delays;
 
 	/* Parse with this.
 	 */
@@ -228,7 +232,7 @@ vips_image_paint_area( VipsImage *image, const VipsRect *r, const VipsPel *ink )
 /* Blend two guint8.
  */
 #define BLEND( X, aX, Y, aY, scale ) \
-	((X * aX + Y * aY) * scale >> 24)
+	(((X * aX + Y * aY) * scale + (1 << 12)) >> 24)
 
 /* Extract R, G, B, A, assuming little-endian.
  */
@@ -239,7 +243,8 @@ vips_image_paint_area( VipsImage *image, const VipsRect *r, const VipsPel *ink )
 
 /* Rebuild RGBA, assuming little-endian.
  */
-#define setRGBA( R, G, B, A ) (R | (G << 8) | (B << 16) | (A << 24))
+#define setRGBA( R, G, B, A ) \
+	(R | (G << 8) | (B << 16) | ((guint32) A << 24))
 
 /* OVER blend of two unpremultiplied RGBA guint32
  *
@@ -255,9 +260,9 @@ blend_pixel( guint32 A, guint32 B )
 
 	guint8 aB = getA( B );
 
-	guint8 fac = (aB * (256 - aA)) >> 8;
+	guint8 fac = (aB * (255 - aA) + 127) >> 8;
 	guint8 aR =  aA + fac;
-	int scale = (1 << 24) / aR;
+	int scale = aR == 0 ? 0 : (1 << 24) / aR;
 
 	guint8 rR = BLEND( getR( A ), aA, getR( B ), fac, scale );
 	guint8 gR = BLEND( getG( A ), aA, getG( B ), fac, scale );
@@ -280,11 +285,6 @@ vips_image_paint_image( VipsImage *frame,
 
 	g_assert( VIPS_IMAGE_SIZEOF_PEL( sub ) == ps );
 
-	/* Disable blend if we are not RGBA.
-	 */
-	if( frame->Bands != 4 )
-		blend = FALSE;
-
 	vips_rect_intersectrect( &frame_rect, &sub_rect, &ovl );
 	if( !vips_rect_isempty( &ovl ) ) {
 		VipsPel *p, *q;
@@ -312,27 +312,15 @@ vips_image_paint_image( VipsImage *frame,
 }
 
 int
-vips__iswebp_buffer( const void *buf, size_t len )
+vips__iswebp_source( VipsSource *source )
 {
+	const unsigned char *p;
+
 	/* WebP is "RIFF xxxx WEBP" at the start, so we need 12 bytes.
 	 */
-	if( len >= 12 &&
-		vips_isprefix( "RIFF", (char *) buf ) &&
-		vips_isprefix( "WEBP", (char *) buf + 8 ) )
-		return( 1 );
-
-	return( 0 );
-}
-
-int
-vips__iswebp( const char *filename )
-{
-	/* Magic number, see above.
-	 */
-	unsigned char header[12];
-
-	if( vips__get_bytes( filename, header, 12 ) == 12 &&
-		vips__iswebp_buffer( header, 12 ) )
+	if( (p = vips_source_sniff( source, 12 )) &&
+		vips_isprefix( "RIFF", (char *) p ) &&
+		vips_isprefix( "WEBP", (char *) p + 8 ) )
 		return( 1 );
 
 	return( 0 );
@@ -346,62 +334,53 @@ read_free( Read *read )
 	VIPS_FREEF( WebPDemuxDelete, read->demux );
 	WebPFreeDecBuffer( &read->config.output );
 
-	if( read->fd > 0 &&
-		read->data &&
-		read->length > 0 ) { 
-		vips__munmap( read->data, read->length ); 
-		read->data = NULL;
-		read->length = 0;
-	}
-
-	VIPS_FREEF( vips_tracked_close, read->fd ); 
-	VIPS_FREE( read->filename );
+	VIPS_UNREF( read->source );
+	VIPS_FREE( read->delays );
 	VIPS_FREE( read );
 
 	return( 0 );
 }
 
+static void
+read_close_cb( VipsImage *image, Read *read )
+{
+	read_free( read );
+}
+
 static Read *
-read_new( const char *filename, const void *data, size_t length, 
-	int page, int n, double scale )
+read_new( VipsImage *out, VipsSource *source, int page, int n, double scale )
 {
 	Read *read;
 
 	if( !(read = VIPS_NEW( NULL, Read )) )
 		return( NULL );
 
-	read->filename = g_strdup( filename );
-	read->data = data;
-	read->length = length;
+	read->out = out;
+	read->source = source;
+	g_object_ref( source );
 	read->page = page;
 	read->n = n;
 	read->scale = scale;
-	read->delay = 100;
-	read->fd = 0;
+	read->delays = NULL;
 	read->demux = NULL;
 	read->frame = NULL;
 	read->dispose_method = WEBP_MUX_DISPOSE_NONE;
 	read->frame_no = 0;
 
-	if( read->filename ) { 
-		/* libwebp makes streaming from a file source very hard. We 
-		 * have to read to a full memory buffer, then copy to out.
-		 *
-		 * mmap the input file, it's slightly quicker.
-		 */
-		if( (read->fd = vips__open_image_read( read->filename )) < 0 ||
-			(read->length = vips_file_length( read->fd )) < 0 ||
-			!(read->data = vips__mmap( read->fd, 
-				FALSE, read->length, 0 )) ) {
-			read_free( read );
-			return( NULL );
-		}
-	}
+	/* Everything has to stay open until read has finished, unfortunately,
+	 * since webp relies on us mapping the whole file.
+	 */
+	g_signal_connect( out, "close", 
+		G_CALLBACK( read_close_cb ), read ); 
 
 	WebPInitDecoderConfig( &read->config );
 	read->config.options.use_threads = 1;
 	read->config.output.is_external_memory = 1;
 
+	if( !(read->data.bytes = 
+		vips_source_map( source, &read->data.size )) ) 
+		return( NULL );
+
 	return( read );
 }
 
@@ -417,29 +396,27 @@ const int vips__n_webp_names = VIPS_NUMBER( vips__webp_names );
 static int
 read_header( Read *read, VipsImage *out )
 {
-	WebPData data;
-	int canvas_width;
-	int canvas_height;
 	int flags;
 	int i;
 
-	data.bytes = read->data;
-	data.size = read->length;
-	if( !(read->demux = WebPDemux( &data )) ) {
+	if( !(read->demux = WebPDemux( &read->data )) ) {
 		vips_error( "webp", "%s", _( "unable to parse image" ) ); 
 		return( -1 ); 
 	}
 
-	canvas_width = WebPDemuxGetI( read->demux, WEBP_FF_CANVAS_WIDTH );
-	canvas_height = WebPDemuxGetI( read->demux, WEBP_FF_CANVAS_HEIGHT );
+	read->canvas_width = 
+		WebPDemuxGetI( read->demux, WEBP_FF_CANVAS_WIDTH );
+	read->canvas_height = 
+		WebPDemuxGetI( read->demux, WEBP_FF_CANVAS_HEIGHT );
+
 	/* We round-to-nearest cf. pdfload etc.
 	 */
-	read->frame_width = VIPS_RINT( canvas_width * read->scale );
-	read->frame_height = VIPS_RINT( canvas_height * read->scale );
+	read->frame_width = VIPS_RINT( read->canvas_width * read->scale );
+	read->frame_height = VIPS_RINT( read->canvas_height * read->scale );
 
 #ifdef DEBUG
-	printf( "webp2vips: canvas_width = %d\n", canvas_width );
-	printf( "webp2vips: canvas_height = %d\n", canvas_height );
+	printf( "webp2vips: canvas_width = %d\n", read->canvas_width );
+	printf( "webp2vips: canvas_height = %d\n", read->canvas_height );
 	printf( "webp2vips: frame_width = %d\n", read->frame_width );
 	printf( "webp2vips: frame_height = %d\n", read->frame_height );
 #endif /*DEBUG*/
@@ -447,10 +424,11 @@ read_header( Read *read, VipsImage *out )
 	flags = WebPDemuxGetI( read->demux, WEBP_FF_FORMAT_FLAGS );
 
 	read->alpha = flags & ALPHA_FLAG;
-	if( read->alpha )  
-		read->config.output.colorspace = MODE_RGBA;
-	else
-		read->config.output.colorspace = MODE_RGB;
+
+	/* We do everything as RGBA and then, if we can, drop the alpha on
+	 * save.
+	 */
+	read->config.output.colorspace = MODE_RGBA;
 
 	if( flags & ANIMATION_FLAG ) { 
 		int loop_count;
@@ -466,25 +444,53 @@ read_header( Read *read, VipsImage *out )
 		printf( "webp2vips: frame_count = %d\n", read->frame_count );
 #endif /*DEBUG*/
 
-		vips_image_set_int( out, "gif-loop", loop_count );
+		vips_image_set_int( out, "loop", loop_count );
+
+		/* DEPRECATED "gif-loop"
+		 *
+		 * Not the correct behavior as loop=1 became gif-loop=0
+		 * but we want to keep the old behavior untouched!
+		 */
+		vips_image_set_int( out, "gif-loop", 
+			loop_count == 0 ? 0 : loop_count - 1 );
+		
 		vips_image_set_int( out, 
 			VIPS_META_PAGE_HEIGHT, read->frame_height );
 
-		/* We must get the first frame to get the delay. Frames number
-		 * from 1.
-		 */
 		if( WebPDemuxGetFrame( read->demux, 1, &iter ) ) {
-			read->delay = iter.duration;
+			int i;
 
-#ifdef DEBUG
-			printf( "webp2vips: duration = %d\n", read->delay );
-#endif /*DEBUG*/
+			read->delays = (int *) 
+				g_malloc0( read->frame_count * sizeof( int ) );
+			for( i = 0; i < read->frame_count; i++ ) 
+				read->delays[i] = 40;
+
+			do {
+				g_assert( iter.frame_num >= 1 &&
+					iter.frame_num <= read->frame_count );
+
+				read->delays[iter.frame_num - 1] = 
+					iter.duration;
+
+				/* We need the alpha in an animation if:
+				 *   - any frame has transparent pixels 
+				 *   - any frame doesn't fill the whole canvas.
+				 */
+				if( iter.has_alpha ||
+					iter.width != read->canvas_width ||
+					iter.height != read->canvas_height ) 
+					read->alpha = TRUE;
+			} while( WebPDemuxNextFrame( &iter ) );
+
+			vips_image_set_array_int( out, 
+				"delay", read->delays, read->frame_count );
 
 			/* webp uses ms for delays, gif uses centiseconds.
 			 */
 			vips_image_set_int( out, "gif-delay", 
-				VIPS_RINT( read->delay / 10.0 ) );
+				VIPS_RINT( read->delays[0] / 10.0 ) );
 		}
+
 		WebPDemuxReleaseIterator( &iter );
 
 		if( read->n == -1 )
@@ -512,8 +518,16 @@ read_header( Read *read, VipsImage *out )
 		read->frame_count = 1;
 	}
 
+	/* height can be huge if this is an animated webp image.
+	 */
 	if( read->width <= 0 ||
-		read->height <= 0 ) {
+		read->height <= 0 ||
+		read->width > 0x3FFF ||
+		read->height > VIPS_MAX_COORD ||
+		read->frame_width <= 0 ||
+		read->frame_height <= 0 ||
+		read->frame_width > 0x3FFF ||
+		read->frame_height > 0x3FFF ) { 
 		vips_error( "webp", "%s", _( "bad image dimensions" ) ); 
 		return( -1 ); 
 	}
@@ -532,10 +546,12 @@ read_header( Read *read, VipsImage *out )
 		}
 	}
 
+	/* The canvas is always RGBA, we drop alpha to RGB on output if we
+	 * can.
+	 */
 	read->frame = vips_image_new_memory();
 	vips_image_init_fields( read->frame,
-		read->frame_width, read->frame_height,
-		read->alpha ? 4 : 3,
+		read->frame_width, read->frame_height, 4, 
 		VIPS_FORMAT_UCHAR, VIPS_CODING_NONE,
 		VIPS_INTERPRETATION_sRGB,
 		1.0, 1.0 );
@@ -551,6 +567,8 @@ read_header( Read *read, VipsImage *out )
 		VIPS_INTERPRETATION_sRGB,
 		1.0, 1.0 );
 	vips_image_pipelinev( out, VIPS_DEMAND_STYLE_THINSTRIP, NULL );
+	VIPS_SETSTR( out->filename, 
+		vips_connection_filename( VIPS_CONNECTION( read->source ) ) );
 
 	if( !WebPDemuxGetFrame( read->demux, 1, &read->iter ) ) {
 		vips_error( "webp", 
@@ -561,28 +579,6 @@ read_header( Read *read, VipsImage *out )
 	return( 0 );
 }
 
-int
-vips__webp_read_file_header( const char *filename, VipsImage *out, 
-	int page, int n, double scale )
-{
-	Read *read;
-
-	if( !(read = read_new( filename, NULL, 0, page, n, scale )) ) {
-		vips_error( "webp2vips",
-			_( "unable to open \"%s\"" ), filename ); 
-		return( -1 );
-	}
-
-	if( read_header( read, out ) ) {
-		read_free( read );
-		return( -1 );
-	}
-
-	read_free( read );
-
-	return( 0 );
-}
-
 /* Read a single frame -- a width * height block of pixels. This will get
  * blended into the accumulator at some offset.
  */
@@ -598,8 +594,7 @@ read_frame( Read *read,
 
 	frame = vips_image_new_memory();
 	vips_image_init_fields( frame,
-		width, height, 
-		read->alpha ? 4 : 3,
+		width, height, 4,
 		VIPS_FORMAT_UCHAR, VIPS_CODING_NONE,
 		VIPS_INTERPRETATION_sRGB,
 		1.0, 1.0 );
@@ -688,11 +683,6 @@ read_next_frame( Read *read )
 		printf( "don't blend\n" ); 
 #endif /*DEBUG*/
 
-	if( read->frame_count > 1 &&
-		read->iter.duration != read->delay ) 
-		g_warning( "webp2vips: "
-			"not all frames have equal duration" );
-
 	if( !(frame = read_frame( read, 
 		area.width, area.height,
 		read->iter.fragment.bytes, read->iter.fragment.size )) ) 
@@ -702,7 +692,8 @@ read_next_frame( Read *read )
 	 */
 	vips_image_paint_image( read->frame, frame, 
 		area.left, area.top, 
-		read->iter.blend_method == WEBP_MUX_BLEND );
+		read->iter.frame_num > 1 &&
+			read->iter.blend_method == WEBP_MUX_BLEND );
 
 	g_object_unref( frame );
 
@@ -744,9 +735,29 @@ read_webp_generate( VipsRegion *or,
 		read->frame_no += 1;
 	}
 
-	memcpy( VIPS_REGION_ADDR( or, 0, r->top ),
-		VIPS_IMAGE_ADDR( read->frame, 0, line ),
-		VIPS_IMAGE_SIZEOF_LINE( read->frame ) );
+	if( or->im->Bands == 4 ) 
+		memcpy( VIPS_REGION_ADDR( or, 0, r->top ),
+			VIPS_IMAGE_ADDR( read->frame, 0, line ),
+			VIPS_IMAGE_SIZEOF_LINE( read->frame ) );
+	else {
+		int x;
+		VipsPel *p;
+		VipsPel *q;
+
+		/* We know that alpha is solid, so we can just drop the 4th
+		 * band.
+		 */
+		p = VIPS_IMAGE_ADDR( read->frame, 0, line );
+		q = VIPS_REGION_ADDR( or, 0, r->top );
+		for( x = 0; x < r->width; x++ ) {
+			q[0] = p[0];
+			q[1] = p[1];
+			q[2] = p[2];
+
+			q += 3;
+			p += 4;
+		}
+	}
 
 	return( 0 );
 }
@@ -771,67 +782,27 @@ read_image( Read *read, VipsImage *out )
 }
 
 int
-vips__webp_read_file( const char *filename, VipsImage *out, 
+vips__webp_read_header_source( VipsSource *source, VipsImage *out,
 	int page, int n, double scale )
 {
 	Read *read;
 
-	if( !(read = read_new( filename, NULL, 0, page, n, scale )) ) {
-		vips_error( "webp2vips",
-			_( "unable to open \"%s\"" ), filename ); 
+	if( !(read = read_new( out, source, page, n, scale )) || 
+		read_header( read, out ) )
 		return( -1 );
-	}
-
-	if( read_image( read, out ) ) {
-		read_free( read );
-		return( -1 );
-	}
-
-	read_free( read );
 
 	return( 0 );
 }
 
 int
-vips__webp_read_buffer_header( const void *buf, size_t len, VipsImage *out,
+vips__webp_read_source( VipsSource *source, VipsImage *out, 
 	int page, int n, double scale )
 {
 	Read *read;
 
-	if( !(read = read_new( NULL, buf, len, page, n, scale )) ) {
-		vips_error( "webp2vips",
-			"%s", _( "unable to open buffer" ) ); 
+	if( !(read = read_new( out, source, page, n, scale )) || 
+		read_image( read, out ) )
 		return( -1 );
-	}
-
-	if( read_header( read, out ) ) {
-		read_free( read );
-		return( -1 );
-	}
-
-	read_free( read );
-
-	return( 0 );
-}
-
-int
-vips__webp_read_buffer( const void *buf, size_t len, VipsImage *out, 
-	int page, int n, double scale )
-{
-	Read *read;
-
-	if( !(read = read_new( NULL, buf, len, page, n, scale )) ) {
-		vips_error( "webp2vips",
-			"%s", _( "unable to open buffer" ) ); 
-		return( -1 );
-	}
-
-	if( read_image( read, out ) ) {
-		read_free( read );
-		return( -1 );
-	}
-
-	read_free( read );
 
 	return( 0 );
 }
