@@ -6,6 +6,8 @@
  * 	- add animated webp support
  * 15/1/19 lovell
  * 	- add @reduction_effort 
+ * 18/7/20
+ * 	- add @profile param to match tiff, jpg, etc.
  */
 
 /*
@@ -99,6 +101,10 @@ typedef struct _VipsForeignSaveWebp {
 	 */
 	int kmax;
 
+	/* Profile to embed.
+	 */
+	char *profile;
+
 } VipsForeignSaveWebp;
 
 typedef VipsForeignSaveClass VipsForeignSaveWebpClass;
@@ -204,6 +210,13 @@ vips_foreign_save_webp_class_init( VipsForeignSaveWebpClass *class )
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignSaveWebp, reduction_effort ),
 		0, 6, 4 );
+
+	VIPS_ARG_STRING( class, "profile", 20, 
+		_( "Profile" ), 
+		_( "ICC profile to embed" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSaveWebp, profile ),
+		NULL );
 }
 
 static void
@@ -219,6 +232,69 @@ vips_foreign_save_webp_init( VipsForeignSaveWebp *webp )
 	webp->kmax = INT_MAX;
 }
 
+typedef struct _VipsForeignSaveWebpTarget {
+	VipsForeignSaveWebp parent_object;
+
+	VipsTarget *target;
+
+} VipsForeignSaveWebpTarget;
+
+typedef VipsForeignSaveWebpClass VipsForeignSaveWebpTargetClass;
+
+G_DEFINE_TYPE( VipsForeignSaveWebpTarget, vips_foreign_save_webp_target, 
+	vips_foreign_save_webp_get_type() );
+
+static int
+vips_foreign_save_webp_target_build( VipsObject *object )
+{
+	VipsForeignSave *save = (VipsForeignSave *) object;
+	VipsForeignSaveWebp *webp = (VipsForeignSaveWebp *) object;
+	VipsForeignSaveWebpTarget *target = 
+		(VipsForeignSaveWebpTarget *) object;
+
+	if( VIPS_OBJECT_CLASS( vips_foreign_save_webp_target_parent_class )->
+		build( object ) )
+		return( -1 );
+
+	if( vips__webp_write_target( save->ready, target->target, 
+		webp->Q, webp->lossless, webp->preset,
+		webp->smart_subsample, webp->near_lossless,
+		webp->alpha_q, webp->reduction_effort,
+		webp->min_size, webp->kmin, webp->kmax,
+		save->strip, webp->profile ) )
+		return( -1 );
+
+	return( 0 );
+}
+
+static void
+vips_foreign_save_webp_target_class_init( 
+	VipsForeignSaveWebpTargetClass *class )
+{
+	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+
+	gobject_class->set_property = vips_object_set_property;
+	gobject_class->get_property = vips_object_get_property;
+
+	object_class->nickname = "webpsave_target";
+	object_class->description = _( "save image to webp target" );
+	object_class->build = vips_foreign_save_webp_target_build;
+
+	VIPS_ARG_OBJECT( class, "target", 1,
+		_( "Target" ),
+		_( "Target to save to" ),
+		VIPS_ARGUMENT_REQUIRED_INPUT, 
+		G_STRUCT_OFFSET( VipsForeignSaveWebpTarget, target ),
+		VIPS_TYPE_TARGET );
+}
+
+static void
+vips_foreign_save_webp_target_init( VipsForeignSaveWebpTarget *target )
+{
+}
+
+
 typedef struct _VipsForeignSaveWebpFile {
 	VipsForeignSaveWebp parent_object;
 
@@ -240,17 +316,24 @@ vips_foreign_save_webp_file_build( VipsObject *object )
 	VipsForeignSaveWebp *webp = (VipsForeignSaveWebp *) object;
 	VipsForeignSaveWebpFile *file = (VipsForeignSaveWebpFile *) object;
 
+	VipsTarget *target;
+
 	if( VIPS_OBJECT_CLASS( vips_foreign_save_webp_file_parent_class )->
 		build( object ) )
 		return( -1 );
 
-	if( vips__webp_write_file( save->ready, file->filename, 
+	if( !(target = vips_target_new_to_file( file->filename )) )
+		return( -1 );
+	if( vips__webp_write_target( save->ready, target,
 		webp->Q, webp->lossless, webp->preset,
 		webp->smart_subsample, webp->near_lossless,
 		webp->alpha_q, webp->reduction_effort,
 		webp->min_size, webp->kmin, webp->kmax,
-		save->strip ) )
+		save->strip, webp->profile ) ) {
+		VIPS_UNREF( target );
 		return( -1 );
+	}
+	VIPS_UNREF( target );
 
 	return( 0 );
 }
@@ -300,30 +383,35 @@ vips_foreign_save_webp_buffer_build( VipsObject *object )
 {
 	VipsForeignSave *save = (VipsForeignSave *) object;
 	VipsForeignSaveWebp *webp = (VipsForeignSaveWebp *) object;
-	VipsForeignSaveWebpBuffer *file = (VipsForeignSaveWebpBuffer *) object;
+	VipsForeignSaveWebpBuffer *buffer = 
+		(VipsForeignSaveWebpBuffer *) object;
 
-	void *obuf;
-	size_t olen;
+	VipsTarget *target;
 	VipsBlob *blob;
 
 	if( VIPS_OBJECT_CLASS( vips_foreign_save_webp_buffer_parent_class )->
 		build( object ) )
 		return( -1 );
 
-	if( vips__webp_write_buffer( save->ready, &obuf, &olen, 
+	if( !(target = vips_target_new_to_memory()) )
+		return( -1 );
+
+	if( vips__webp_write_target( save->ready, target,
 		webp->Q, webp->lossless, webp->preset,
 		webp->smart_subsample, webp->near_lossless,
 		webp->alpha_q, webp->reduction_effort,
 		webp->min_size, webp->kmin, webp->kmax,
-		save->strip ) )
+		save->strip, webp->profile ) ) {
+		VIPS_UNREF( target );
 		return( -1 );
+	}
 
-	/* obuf is a g_free() buffer, not vips_free().
-	 */
-	blob = vips_blob_new( (VipsCallbackFn) g_free, obuf, olen );
-	g_object_set( file, "buffer", blob, NULL );
+	g_object_get( target, "blob", &blob, NULL );
+	g_object_set( buffer, "buffer", blob, NULL );
 	vips_area_unref( VIPS_AREA( blob ) );
 
+	VIPS_UNREF( target );
+
 	return( 0 );
 }
 
@@ -370,31 +458,40 @@ vips_foreign_save_webp_mime_build( VipsObject *object )
 	VipsForeignSave *save = (VipsForeignSave *) object;
 	VipsForeignSaveWebp *webp = (VipsForeignSaveWebp *) object;
 
-	void *obuf;
-	size_t olen;
+	VipsTarget *target;
+	VipsBlob *blob;
+	void *data;
+	size_t len;
 
 	if( VIPS_OBJECT_CLASS( vips_foreign_save_webp_mime_parent_class )->
 		build( object ) )
 		return( -1 );
 
-	if( vips__webp_write_buffer( save->ready, &obuf, &olen, 
+	if( !(target = vips_target_new_to_memory()) )
+		return( -1 );
+
+	if( vips__webp_write_target( save->ready, target,
 		webp->Q, webp->lossless, webp->preset,
 		webp->smart_subsample, webp->near_lossless,
 		webp->alpha_q, webp->reduction_effort,
 		webp->min_size, webp->kmin, webp->kmax,
-		save->strip ) )
-		return( -1 );
-
-	printf( "Content-length: %zu\r\n", olen );
-	printf( "Content-type: image/webp\r\n" );
-	printf( "\r\n" );
-	if( fwrite( obuf, sizeof( char ), olen, stdout ) != olen ) {
-		vips_error( "VipsWebp", "%s", _( "error writing output" ) );
+		save->strip, webp->profile ) ) {
+		VIPS_UNREF( target );
 		return( -1 );
 	}
+
+	g_object_get( target, "blob", &blob, NULL );
+	data = VIPS_AREA( blob )->data;
+	len = VIPS_AREA( blob )->length;
+	vips_area_unref( VIPS_AREA( blob ) );
+
+	printf( "Content-length: %zu\r\n", len );
+	printf( "Content-type: image/webp\r\n" );
+	printf( "\r\n" );
+	(void) fwrite( data, sizeof( char ), len, stdout );
 	fflush( stdout );
 
-	g_free( obuf );
+	VIPS_UNREF( target );
 
 	return( 0 );
 }
@@ -436,6 +533,7 @@ vips_foreign_save_webp_mime_init( VipsForeignSaveWebpMime *mime )
  * * @kmin: %gint, minimum number of frames between keyframes
  * * @kmax: %gint, maximum number of frames between keyframes
  * * @strip: %gboolean, remove all metadata from image
+ * * @profile: %gchararray, filename of ICC profile to attach
  *
  * Write an image to a file in WebP format. 
  *
@@ -459,15 +557,19 @@ vips_foreign_save_webp_mime_init( VipsForeignSaveWebpMime *mime )
  * with @Q 80, 60, 40 or 20 to apply increasing amounts of preprocessing
  * which improves the near-lossless compression ratio by up to 50%.
  *
- * For animated webp output, @min_size will try to optimise for minimum size.
+ * For animated webp output, @min_size will try to optimize for minimum size.
  *
  * For animated webp output, @kmax sets the maximum number of frames between
  * keyframes. Setting 0 means only keyframes. @kmin sets the minimum number of
  * frames between frames. Setting 0 means no keyframes. By default, keyframes
  * are disabled.
  *
- * Use the metadata items `gif-loop` and `gif-delay` to set the number of
- * loops for the animation and the frame delay.
+ * Use @profile to give the name of a profile to be embedded in the file.
+ * This does not affect the pixels which are written, just the way 
+ * they are tagged. See vips_profile_load() for details on profile naming. 
+ *
+ * Use the metadata items `loop` and `delay` to set the number of
+ * loops for the animation and the frame delays.
  *
  * The writer will attach ICC, EXIF and XMP metadata, unless @strip is set to
  * %TRUE.
@@ -509,6 +611,7 @@ vips_webpsave( VipsImage *in, const char *filename, ... )
  * * @kmin: %gint, minimum number of frames between keyframes
  * * @kmax: %gint, maximum number of frames between keyframes
  * * @strip: %gboolean, remove all metadata from image
+ * * @profile: %gchararray, filename of ICC profile to attach
  *
  * As vips_webpsave(), but save to a memory buffer.
  *
@@ -566,6 +669,7 @@ vips_webpsave_buffer( VipsImage *in, void **buf, size_t *len, ... )
  * * @kmin: %gint, minimum number of frames between keyframes
  * * @kmax: %gint, maximum number of frames between keyframes
  * * @strip: %gboolean, remove all metadata from image
+ * * @profile: %gchararray, filename of ICC profile to attach
  *
  * As vips_webpsave(), but save as a mime webp on stdout.
  *
@@ -585,3 +689,43 @@ vips_webpsave_mime( VipsImage *in, ... )
 
 	return( result );
 }
+
+/**
+ * vips_webpsave_target: (method)
+ * @in: image to save 
+ * @target: save image to this target
+ * @...: %NULL-terminated list of optional named arguments
+ *
+ * Optional arguments:
+ *
+ * * @Q: %gint, quality factor
+ * * @lossless: %gboolean, enables lossless compression
+ * * @preset: #VipsForeignWebpPreset, choose lossy compression preset
+ * * @smart_subsample: %gboolean, enables high quality chroma subsampling
+ * * @near_lossless: %gboolean, preprocess in lossless mode (controlled by Q)
+ * * @alpha_q: %gint, set alpha quality in lossless mode
+ * * @reduction_effort: %gint, level of CPU effort to reduce file size
+ * * @min_size: %gboolean, minimise size
+ * * @kmin: %gint, minimum number of frames between keyframes
+ * * @kmax: %gint, maximum number of frames between keyframes
+ * * @strip: %gboolean, remove all metadata from image
+ * * @profile: %gchararray, filename of ICC profile to attach
+ *
+ * As vips_webpsave(), but save to a target.
+ *
+ * See also: vips_webpsave().
+ *
+ * Returns: 0 on success, -1 on error.
+ */
+int
+vips_webpsave_target( VipsImage *in, VipsTarget *target, ... )
+{
+	va_list ap;
+	int result;
+
+	va_start( ap, target );
+	result = vips_call_split( "webpsave_target", ap, in, target );
+	va_end( ap );
+
+	return( result );
+}
