@@ -292,6 +292,12 @@ void vips_info( const char *domain, const char *fmt, ... )
 	__attribute__((format(printf, 2, 3)));
 void vips_vinfo( const char *domain, const char *fmt, va_list ap );
 
+VipsAngle vips_autorot_get_angle( VipsImage *image );
+
+/* iofuncs
+ */
+int vips_free( void *buf );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
