@@ -39,6 +39,14 @@
 extern "C" {
 #endif /*__cplusplus*/
 
+/* << on an int is undefined in C if the int is negative. Imagine a machine
+ * that uses 1s complement, for example.
+ *
+ * Fuzzers find and warn about this, so we must use this macro instead. Cast
+ * to uint, shift, and cast back.
+ */
+#define VIPS_LSHIFT_INT( I, N ) ((int) ((unsigned int) (I) << (N)))
+
 /* What we store in the Meta hash table. We can't just use GHashTable's 
  * key/value pairs, since we need to iterate over meta in Meta_traverse order.
  *
@@ -125,9 +133,7 @@ void vips__link_break_all( VipsImage *im );
 void *vips__link_map( VipsImage *image, gboolean upstream, 
 	VipsSListMap2Fn fn, void *a, void *b );
 
-char *vips__b64_encode( const unsigned char *data, size_t data_length );
-unsigned char *vips__b64_decode( const char *buffer, size_t *data_length );
-
+gboolean vips__mmap_supported( int fd );
 void *vips__mmap( int fd, int writeable, size_t length, gint64 offset );
 int vips__munmap( const void *start, size_t length );
 int vips_mapfile( VipsImage * );
@@ -146,6 +152,7 @@ int vips__write_extension_block( VipsImage *im, void *buf, int size );
 int vips__writehist( VipsImage *image );
 int vips__read_header_bytes( VipsImage *im, unsigned char *from );
 int vips__write_header_bytes( VipsImage *im, unsigned char *to );
+int vips__image_meta_copy( VipsImage *dst, const VipsImage *src );
 
 extern GMutex *vips__global_lock;
 
@@ -269,6 +276,25 @@ VipsWindow *vips_window_take( VipsWindow *window,
 
 int vips__profile_set( VipsImage *image, const char *name );
 
+int vips__lrmosaic( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int bandno,
+	int xref, int yref, int xsec, int ysec,
+	int hwindowsize, int hsearchsize,
+	int balancetype,
+	int mwidth );
+
+int vips__tbmosaic( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int bandno,
+	int xref, int yref, int xsec, int ysec,
+	int hwindowsize, int hsearchsize,
+	int balancetype,
+	int mwidth );
+
+int vips__correl( VipsImage *ref, VipsImage *sec, 
+	int xref, int yref, int xsec, int ysec,
+	int hwindowsize, int hsearchsize,
+	double *correlation, int *x, int *y );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
