@@ -19,7 +19,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -36,8 +37,7 @@
 extern "C" {
 #endif /*__cplusplus*/
 
-#define VIPS_FREEF( F, S ) \
-G_STMT_START { \
+#define VIPS_FREEF( F, S ) G_STMT_START { \
         if( S ) { \
                 (void) F( (S) ); \
                 (S) = 0; \
@@ -66,7 +66,6 @@ G_STMT_START { \
 
 void *vips_malloc( VipsObject *object, size_t size );
 char *vips_strdup( VipsObject *object, const char *str );
-int vips_free( void *buf );
 
 void vips_tracked_free( void *s );
 void *vips_tracked_malloc( size_t size );
@@ -74,7 +73,7 @@ size_t vips_tracked_get_mem( void );
 size_t vips_tracked_get_mem_highwater( void );
 int vips_tracked_get_allocs( void );
 
-int vips_tracked_open( const char *pathname, int flags, ... );
+int vips_tracked_open( const char *pathname, int flags, int mode );
 int vips_tracked_close( int fd );
 int vips_tracked_get_files( void );
 
