@@ -1051,24 +1051,6 @@ int im_filename_suffix_match( const char *path, const char *suffixes[] );
 char *im_getnextoption( char **in );
 char *im_getsuboption( const char *buf );
 
-int im_lrmerge( VipsImage *ref, VipsImage *sec, VipsImage *out,
-	int dx, int dy, int mwidth );
-int im_tbmerge( VipsImage *ref, VipsImage *sec, VipsImage *out,
-	int dx, int dy, int mwidth );
-
-int im_lrmosaic( VipsImage *ref, VipsImage *sec, VipsImage *out,
-	int bandno,
-	int xref, int yref, int xsec, int ysec,
-	int hwindowsize, int hsearchsize,
-	int balancetype,
-	int mwidth );
-int im_tbmosaic( VipsImage *ref, VipsImage *sec, VipsImage *out, 
-	int bandno,
-	int xref, int yref, int xsec, int ysec, 
-	int hwindowsize, int hsearchsize,
-	int balancetype,
-	int mwidth );
-
 int im_match_linear( VipsImage *ref, VipsImage *sec, VipsImage *out,
 	int xr1, int yr1, int xs1, int ys1,
 	int xr2, int yr2, int xs2, int ys2 );
@@ -1083,15 +1065,25 @@ int im_global_balancef( VipsImage *in, VipsImage *out, double gamma );
 int im_remosaic( VipsImage *in, VipsImage *out,
 	const char *old_str, const char *new_str );
 
+int im_lrmerge( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int dx, int dy, int mwidth );
 int im_lrmerge1( VipsImage *ref, VipsImage *sec, VipsImage *out,
 	int xr1, int yr1, int xs1, int ys1,
 	int xr2, int yr2, int xs2, int ys2,
 	int mwidth );
+int im_tbmerge( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int dx, int dy, int mwidth );
 int im_tbmerge1( VipsImage *ref, VipsImage *sec, VipsImage *out,
 	int xr1, int yr1, int xs1, int ys1,
 	int xr2, int yr2, int xs2, int ys2,
 	int mwidth );
 
+int im_lrmosaic( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int bandno,
+	int xref, int yref, int xsec, int ysec,
+	int hwindowsize, int hsearchsize,
+	int balancetype,
+	int mwidth );
 int im_lrmosaic1( VipsImage *ref, VipsImage *sec, VipsImage *out, 
 	int bandno,
 	int xr1, int yr1, int xs1, int ys1, 
@@ -1099,6 +1091,12 @@ int im_lrmosaic1( VipsImage *ref, VipsImage *sec, VipsImage *out,
 	int hwindowsize, int hsearchsize,
 	int balancetype,
 	int mwidth );
+int im_tbmosaic( VipsImage *ref, VipsImage *sec, VipsImage *out, 
+	int bandno,
+	int xref, int yref, int xsec, int ysec, 
+	int hwindowsize, int hsearchsize,
+	int balancetype,
+	int mwidth );
 int im_tbmosaic1( VipsImage *ref, VipsImage *sec, VipsImage *out,
 	int bandno,
 	int xr1, int yr1, int xs1, int ys1,
@@ -1115,13 +1113,6 @@ int im_correl( VipsImage *ref, VipsImage *sec,
 int im_align_bands( VipsImage *in, VipsImage *out );
 int im_maxpos_subpel( VipsImage *in, double *x, double *y );
 
-/* These were public for a while, keep for compat.
- */
-int vips_foreign_load( const char *filename, VipsImage **out, ... )
-	__attribute__((sentinel));
-int vips_foreign_save( VipsImage *in, const char *filename, ... )
-	__attribute__((sentinel));
-
 VipsImage *vips__deprecated_open_read( const char *filename, gboolean sequential );
 VipsImage *vips__deprecated_open_write( const char *filename );
 
@@ -1156,13 +1147,14 @@ int im__colour_unary( const char *domain,
 VipsImage **im__insert_base( const char *domain, 
 	VipsImage *in1, VipsImage *in2, VipsImage *out );
 
-int im__find_lroverlap( VipsImage *ref_in, VipsImage *sec_in, VipsImage *out,
+/* TODO(kleisauke): These are also defined in pmosaicing.h */
+int vips__find_lroverlap( VipsImage *ref_in, VipsImage *sec_in, VipsImage *out,
         int bandno_in,
         int xref, int yref, int xsec, int ysec,
         int halfcorrelation, int halfarea,
         int *dx0, int *dy0,
         double *scale1, double *angle1, double *dx1, double *dy1 );
-int im__find_tboverlap( VipsImage *ref_in, VipsImage *sec_in, VipsImage *out,
+int vips__find_tboverlap( VipsImage *ref_in, VipsImage *sec_in, VipsImage *out,
         int bandno_in,
         int xref, int yref, int xsec, int ysec,
         int halfcorrelation, int halfarea,
@@ -1187,8 +1179,6 @@ void imb_XYZ2Lab( float *, float *, int, im_colour_temperature * );
 void imb_LabS2Lab( signed short *, float *, int );
 void imb_Lab2LabS( float *, signed short *, int n );
 
-void vips__Lab2LabQ_vec( VipsPel *out, float *in, int width );
-void vips__LabQ2Lab_vec( float *out, VipsPel *in, int width );
 
 void im_copy_dmask_matrix( DOUBLEMASK *mask, double **matrix );
 void im_copy_matrix_dmask( double **matrix, DOUBLEMASK *mask );
@@ -1231,6 +1221,18 @@ GOptionGroup *vips_get_option_group( void );
  */
 VipsWindow *vips_window_ref( VipsImage *im, int top, int height );
 
+FILE *vips_popenf( const char *fmt, const char *mode, ... )
+	__attribute__((format(printf, 1, 3)));
+
+/* This stuff is very, very old and should not be used by anyone now.
+ */
+#ifdef VIPS_ENABLE_ANCIENT
+#include <vips/deprecated.h>
+#endif /*VIPS_ENABLE_ANCIENT*/
+
+#include <vips/dispatch.h>
+#include <vips/almostdeprecated.h>
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
