@@ -4,6 +4,8 @@
  * 	- try to make it compile on centos5
  * 7/7/12
  * 	- add a lock so we can run operations from many threads
+ * 28/11/19 [MaxKellermann]
+ * 	- make invalidate advisory rather than immediate
  */
 
 /*
@@ -76,7 +78,7 @@ gboolean vips__cache_trace = FALSE;
  * It was 10,000, but this was too high for batch-style applications with
  * little reuse. 
  */
-static int vips_cache_max = 1000;
+static int vips_cache_max = 100;
 
 /* How many tracked open files we allow before we start dropping cache.
  */
@@ -131,6 +133,11 @@ typedef struct _VipsOperationCacheEntry {
 	 * we can disconnect when we drop an operation.
 	 */
 	gulong invalidate_id;
+
+	/* Set if someone thinks this cache entry should be dropped.
+	 */
+	gboolean invalid;
+
 } VipsOperationCacheEntry;
 
 /* Pass in the pspec so we can get the generic type. For example, a 
@@ -446,7 +453,7 @@ vips_operation_equal( VipsOperation *a, VipsOperation *b )
 }
 
 void *
-vips__cache_once_init( void )
+vips__cache_once_init( void *data )
 {
 	vips_cache_lock = vips_g_mutex_new();
 
@@ -462,7 +469,7 @@ vips__cache_init( void )
 {
 	static GOnce once = G_ONCE_INIT;
 
-	VIPS_ONCE( &once, (GThreadFunc) vips__cache_once_init, NULL );
+	VIPS_ONCE( &once, vips__cache_once_init, NULL );
 }
 
 static void *
@@ -536,6 +543,11 @@ vips_object_unref_arg( VipsObject *object,
 static void
 vips_cache_unref( VipsOperation *operation )
 {
+#ifdef DEBUG
+	printf( "vips_cache_unref: " );
+	vips_object_print_summary( VIPS_OBJECT( operation ) );
+#endif /*DEBUG*/
+
 	(void) vips_argument_map( VIPS_OBJECT( operation ),
 		vips_object_unref_arg, NULL, NULL );
 	g_object_unref( operation );
@@ -550,7 +562,8 @@ vips_cache_remove( VipsOperation *operation )
 		g_hash_table_lookup( vips_cache_table, operation );
 
 #ifdef DEBUG
-	printf( "vips_cache_remove: trimming %p\n", operation );
+	printf( "vips_cache_remove: " );
+	vips_object_print_summary( VIPS_OBJECT( operation ) );
 #endif /*DEBUG*/
 
 	g_assert( entry ); 
@@ -595,7 +608,12 @@ vips_operation_touch( VipsOperation *operation )
 		g_hash_table_lookup( vips_cache_table, operation );
 
 	vips_cache_time += 1;
-	entry->time = vips_cache_time;
+
+	/* Don't up the time for invalid items -- we want them to fall out of
+	 * cache.
+	 */
+	if( !entry->invalid ) 
+		entry->time = vips_cache_time;
 }
 
 /* Ref an operation for the cache. The operation itself, plus all the output 
@@ -604,32 +622,52 @@ vips_operation_touch( VipsOperation *operation )
 static void
 vips_cache_ref( VipsOperation *operation )
 {
+#ifdef DEBUG
+	printf( "vips_cache_ref: " );
+	vips_object_print_summary( VIPS_OBJECT( operation ) );
+#endif /*DEBUG*/
+
 	g_object_ref( operation );
 	(void) vips_argument_map( VIPS_OBJECT( operation ),
 		vips_object_ref_arg, NULL, NULL );
 	vips_operation_touch( operation );
 }
 
+static void
+vips_cache_invalidate_cb( VipsOperation *operation, 
+	VipsOperationCacheEntry *entry )
+{
+#ifdef DEBUG
+	printf( "vips_cache_invalidate_cb: " );
+	vips_object_print_summary( VIPS_OBJECT( operation ) );
+#endif /*DEBUG*/
+
+	entry->invalid = TRUE;
+}
+
 static void
 vips_cache_insert( VipsOperation *operation )
 {
 	VipsOperationCacheEntry *entry = g_new( VipsOperationCacheEntry, 1 );
 
 #ifdef VIPS_DEBUG
-	printf( "vips_cache_insert: adding %p to cache\n", operation );
+	printf( "vips_cache_insert: adding to cache" );
+	vips_object_print_dump( VIPS_OBJECT( operation ) );
 #endif /*VIPS_DEBUG*/
 
 	entry->operation = operation;
 	entry->time = 0;
 	entry->invalidate_id = 0;
+	entry->invalid = FALSE;
 
 	g_hash_table_insert( vips_cache_table, operation, entry );
 	vips_cache_ref( operation );
 
-	/* If the operation signals "invalidate", we must drop it.
+	/* If the operation signals "invalidate", we must tag this cache entry
+	 * for removal.
 	 */
 	entry->invalidate_id = g_signal_connect( operation, "invalidate", 
-		G_CALLBACK( vips_cache_remove ), NULL ); 
+		G_CALLBACK( vips_cache_invalidate_cb ), entry ); 
 }
 
 static void *
@@ -662,6 +700,10 @@ vips_cache_get_first( void )
 void
 vips_cache_drop_all( void )
 {
+#ifdef VIPS_DEBUG
+	printf( "vips_cache_drop_all:\n" );
+#endif /*VIPS_DEBUG*/
+
 	g_mutex_lock( vips_cache_lock );
 
 	if( vips_cache_table ) {
@@ -726,7 +768,8 @@ vips_cache_trim( void )
 		vips_tracked_get_mem() > vips_cache_max_mem) &&
 		(operation = vips_cache_get_lru()) ) {
 #ifdef DEBUG
-		printf( "vips_cache_trim: trimming %p\n", operation );
+		printf( "vips_cache_trim: trimming " );
+		vips_object_print_summary( VIPS_OBJECT( operation ) );
 #endif /*DEBUG*/
 
 		vips_cache_remove( operation );
@@ -763,13 +806,22 @@ vips_cache_operation_lookup( VipsOperation *operation )
 	result = NULL;
 
 	if( (hit = g_hash_table_lookup( vips_cache_table, operation )) ) {
-		if( vips__cache_trace ) {
-			printf( "vips cache*: " );
-			vips_object_print_summary( VIPS_OBJECT( operation ) );
+		if( hit->invalid ) {
+			/* There but has been tagged for removal.
+			 */
+			vips_cache_remove( hit->operation );
+			hit = NULL;
 		}
+		else {
+			if( vips__cache_trace ) {
+				printf( "vips cache*: " );
+				vips_object_print_summary( 
+					VIPS_OBJECT( operation ) );
+			}
 
-		result = hit->operation;
-		vips_cache_ref( result );
+			result = hit->operation;
+			vips_cache_ref( result );
+		}
 	}
 
 	g_mutex_unlock( vips_cache_lock );
