@@ -14,6 +14,8 @@
  * 	- gtkdoc comments
  * 24/6/10
  * 	- fmt to error_exit() may be NULL
+ * 12/9/19 [dineshkannaa]
+ * 	- add vips_error_buffer_copy()
  */
 
 /*
@@ -69,7 +71,7 @@
 #endif /*OS_WIN32*/
 
 /**
- * SECTION: error
+ * SECTION: errors
  * @short_description: error messages and error handling
  * @stability: Stable
  * @include: vips/vips.h
@@ -184,6 +186,26 @@ vips_error_buffer( void )
 	return( msg );
 }
 
+/**
+ * vips_error_buffer_copy: 
+ *
+ * Return a copy of the vips error buffer, and clear it. 
+ *
+ * Returns: a copy of the libvips error buffer
+ */
+char *
+vips_error_buffer_copy( void )
+{
+	char *msg;
+
+	g_mutex_lock( vips__global_lock );
+	msg = g_strdup( vips_buf_all( &vips_error_buf ) );
+	vips_error_clear();
+	g_mutex_unlock( vips__global_lock );
+
+	return( msg );
+}
+
 /* Some systems do not have va_copy() ... this might work (it does on MSVC),
  * apparently.
  *
