@@ -34,6 +34,10 @@
  * 	- add vips_image_get_page_height()
  * 19/6/19
  * 	- add vips_image_get_n_pages()
+ * 20/6/19
+ * 	- add vips_image_get/set_array_int()
+ * 31/1/19
+ * 	- lock for metadata changes
  */
 
 /*
@@ -128,6 +132,11 @@
  * these types, it can be copied between images efficiently.
  */
 
+/* Use in various small places where we need a mutex and it's not worth 
+ * making a private one.
+ */
+static GMutex *vips__meta_lock = NULL;
+
 /* We have to keep the gtype as a string, since we statically init this.
  */
 typedef struct _HeaderField {
@@ -226,7 +235,7 @@ vips_format_sizeof_unsafe( VipsBandFormat format )
 /* Check that this meta is on the hash table.
  */
 static void *
-meta_sanity_on_hash( VipsMeta *meta, VipsImage *im )
+meta_sanity_on_hash( VipsMeta *meta, VipsImage *im, void *b )
 {
 	VipsMeta *found;
 
@@ -441,12 +450,12 @@ vips_image_guess_format( const VipsImage *image )
 		break;
 
 	case VIPS_INTERPRETATION_sRGB: 
+	case VIPS_INTERPRETATION_RGB: 
 		format = VIPS_FORMAT_UCHAR;
 		break;
 
 	case VIPS_INTERPRETATION_XYZ: 
 	case VIPS_INTERPRETATION_LAB: 
-	case VIPS_INTERPRETATION_RGB: 
 	case VIPS_INTERPRETATION_CMC: 
 	case VIPS_INTERPRETATION_LCH: 
 	case VIPS_INTERPRETATION_HSV: 
@@ -517,8 +526,7 @@ vips_image_get_interpretation( const VipsImage *image )
 	return( image->Type );
 }
 
-/* Try to pick a sane value for interpretation, assuming Type has been set
- * incorrectly.
+/* Try to guess a sane value for interpretation.
  */
 static VipsInterpretation
 vips_image_default_interpretation( const VipsImage *image )
@@ -527,7 +535,7 @@ vips_image_default_interpretation( const VipsImage *image )
 	case VIPS_CODING_LABQ:
 		return( VIPS_INTERPRETATION_LABQ );
 	case VIPS_CODING_RAD:
-		return( VIPS_INTERPRETATION_RGB );
+		return( VIPS_INTERPRETATION_sRGB );
 	default:
 		break;
 	}
@@ -576,7 +584,7 @@ vips_image_guess_interpretation( const VipsImage *image )
 		break;
 
 	case VIPS_CODING_RAD:
-		if( image->Type != VIPS_INTERPRETATION_RGB )
+		if( image->Type != VIPS_INTERPRETATION_sRGB )
 			sane = FALSE;
 		break;
 
@@ -715,7 +723,8 @@ vips_image_get_yoffset( const VipsImage *image )
  * vips_image_get_filename: (method)
  * @image: image to get from
  *
- * Returns: the name of the file the image was loaded from. 
+ * Returns: the name of the file the image was loaded from, or NULL if there
+ * is no filename.
  */
 const char *
 vips_image_get_filename( const VipsImage *image )
@@ -794,22 +803,22 @@ vips_image_get_page_height( VipsImage *image )
 {
 	int page_height;
 
-	if( !vips_image_get_typeof( image, VIPS_META_PAGE_HEIGHT ) ||
-		vips_image_get_int( image, VIPS_META_PAGE_HEIGHT, 
-			&page_height ) ||
-		page_height <= 0 ||
-		page_height > image->Ysize ||
-		image->Ysize % page_height != 0 ) 
-		page_height = image->Ysize;
+	if( vips_image_get_typeof( image, VIPS_META_PAGE_HEIGHT ) &&
+		!vips_image_get_int( image, VIPS_META_PAGE_HEIGHT, 
+			&page_height ) &&
+		page_height > 0 &&
+		page_height < image->Ysize &&
+		image->Ysize % page_height == 0 ) 
+		return( page_height );
 
-	return( page_height );
+	return( image->Ysize );
 }
 
 /**
  * vips_image_get_n_pages: (method)
  * @image: image to get from
  *
- * Fetch and sanity-check VIPS_META_N_PAGES. Default to 1 if not present or
+ * Fetch and sanity-check #VIPS_META_N_PAGES. Default to 1 if not present or
  * crazy.
  *
  * This is the number of pages in the image file, not the number of pages that
@@ -822,13 +831,77 @@ vips_image_get_n_pages( VipsImage *image )
 {
 	int n_pages;
 
-	if( !vips_image_get_typeof( image, VIPS_META_N_PAGES ) ||
-		vips_image_get_int( image, VIPS_META_N_PAGES, &n_pages ) ||
-		n_pages < 2 || 
-		n_pages > 1000 )
-		n_pages = 1;
+	if( vips_image_get_typeof( image, VIPS_META_N_PAGES ) &&
+		!vips_image_get_int( image, VIPS_META_N_PAGES, &n_pages ) &&
+		n_pages > 1 &&
+		n_pages < 1000 )
+		return( n_pages );
 
-	return( n_pages );
+	return(	1 );
+}
+
+/**
+ * vips_image_get_n_subifds: (method)
+ * @image: image to get from
+ *
+ * Fetch and sanity-check #VIPS_META_N_SUBIFDS. Default to 0 if not present or
+ * crazy.
+ *
+ * Returns: the number of subifds in the image file
+ */
+int
+vips_image_get_n_subifds( VipsImage *image )
+{
+	int n_subifds;
+
+	if( vips_image_get_typeof( image, VIPS_META_N_SUBIFDS ) &&
+		!vips_image_get_int( image, VIPS_META_N_SUBIFDS, &n_subifds ) &&
+		n_subifds > 1 &&
+		n_subifds < 1000 )
+		return( n_subifds );
+
+	return(	0 );
+}
+
+/**
+ * vips_image_get_orientation: (method)
+ * @image: image to get from
+ *
+ * Fetch and sanity-check #VIPS_META_ORIENTATION. Default to 1 (no rotate, 
+ * no flip) if not present or crazy.
+ *
+ * Returns: the image orientation.
+ */
+int
+vips_image_get_orientation( VipsImage *image )
+{
+	int orientation;
+
+	if( vips_image_get_typeof( image, VIPS_META_ORIENTATION ) &&
+		!vips_image_get_int( image, VIPS_META_ORIENTATION, 
+			&orientation ) &&
+		orientation > 0 &&
+		orientation < 9 )
+		return( orientation );
+
+	return( 1 );
+}
+
+/**
+ * vips_image_get_orientation_swap: (method)
+ * @image: image to get from
+ *
+ * Return %TRUE if applying the orientation would swap width and height.
+ *
+ * Returns: if width/height will swap
+ */
+gboolean
+vips_image_get_orientation_swap( VipsImage *image )
+{
+	int orientation = vips_image_get_orientation( image );
+
+	return( orientation >= 5 &&
+		orientation <= 8 );
 }
 
 /**
@@ -897,7 +970,7 @@ vips_image_init_fields( VipsImage *image,
 }
 
 static void *
-meta_cp_field( VipsMeta *meta, VipsImage *dst )
+meta_cp_field( VipsMeta *meta, VipsImage *dst, void *b )
 {
 #ifdef DEBUG
 {
@@ -921,15 +994,18 @@ meta_cp_field( VipsMeta *meta, VipsImage *dst )
 
 /* Copy meta on to dst. 
  */
-static int
-meta_cp( VipsImage *dst, const VipsImage *src )
+int
+vips__image_meta_copy( VipsImage *dst, const VipsImage *src )
 {
 	if( src->meta ) {
-		/* Loop, copying fields.
+		/* We lock with vips_image_set() to stop races in highly-
+		 * threaded applications.
 		 */
+		g_mutex_lock( vips__meta_lock );
 		meta_init( dst );
 		vips_slist_map2( src->meta_traverse,
 			(VipsSListMap2Fn) meta_cp_field, dst, NULL );
+		g_mutex_unlock( vips__meta_lock );
 	}
 
 	return( 0 );
@@ -975,7 +1051,7 @@ vips__image_copy_fields_array( VipsImage *out, VipsImage *in[] )
 	 * subclass loaders will sometimes write to an image. 
 	 */
 	for( i = ni - 1; i >= 0; i-- ) 
-		if( meta_cp( out, in[i] ) )
+		if( vips__image_meta_copy( out, in[i] ) )
 			return( -1 );
 
 	/* Merge hists first to last.
@@ -1017,8 +1093,17 @@ vips_image_set( VipsImage *image, const char *name, GValue *value )
 	g_assert( name );
 	g_assert( value );
 
+	/* We lock between modifying metadata and copying metadata between
+	 * images, see vips__image_meta_copy().
+	 *
+	 * This prevents modification of metadata by one thread racing with
+	 * metadata copy on another -- this can lead to crashes in
+	 * highly-threaded applications.
+	 */
+	g_mutex_lock( vips__meta_lock );
 	meta_init( image );
 	(void) meta_new( image, name, value );
+	g_mutex_unlock( vips__meta_lock );
 
 	/* If we're setting an EXIF data block, we need to automatically expand 
 	 * out all the tags. This will set things like xres/yres too.
@@ -1033,6 +1118,7 @@ vips_image_set( VipsImage *image, const char *name, GValue *value )
 #ifdef DEBUG
 	meta_sanity( image );
 #endif /*DEBUG*/
+
 }
 
 /* Unforunately gvalue seems to have no way of doing this. Just handle the vips
@@ -1210,17 +1296,32 @@ vips_image_get_typeof( const VipsImage *image, const char *name )
 gboolean
 vips_image_remove( VipsImage *image, const char *name )
 {
-	if( image->meta && 
-		g_hash_table_remove( image->meta, name ) )
-		return( TRUE );
+	gboolean result;
 
-	return( FALSE );
+	result = FALSE;
+
+	if( image->meta ) {
+		/* We lock between modifying metadata and copying metadata 
+		 * between images, see vips__image_meta_copy().
+		 *
+		 * This prevents modification of metadata by one thread 
+		 * racing with metadata copy on another -- this can lead to 
+		 * crashes in highly-threaded applications.
+		 */
+		g_mutex_lock( vips__meta_lock );
+		result = g_hash_table_remove( image->meta, name );
+		g_mutex_unlock( vips__meta_lock );
+	}
+
+	return( result );
 }
 
 /* Deprecated header fields we hide from _map.
  */
 static const char *vips_image_header_deprecated[] = {
-	"ipct-data"
+	"ipct-data",
+	"gif-delay",
+	"gif-loop"
 };
 
 static void *
@@ -1468,7 +1569,7 @@ vips_image_set_blob_copy( VipsImage *image,
 	((unsigned char *) data_copy)[length] = '\0';
 
 	vips_image_set_blob( image, 
-		name, (VipsCallbackFn) vips_free, data_copy, length );
+		name, (VipsCallbackFn) vips_area_free_cb, data_copy, length );
 }
 
 /** 
@@ -1800,6 +1901,64 @@ vips_image_set_image( VipsImage *image, const char *name, VipsImage *im )
 	g_value_unset( &value );
 }
 
+/** 
+ * vips_image_get_array_int: (method)
+ * @image: image to get the metadata from
+ * @name: metadata name
+ * @out: (transfer none): return pointer to array
+ * @n: (allow-none): return the number of elements here, optionally
+ *
+ * Gets @out from @im under the name @name. 
+ * The field must be of type
+ * #VIPS_TYPE_ARRAY_INT.
+ *
+ * Do not free @out. @out is valid as long as @image is valid.
+ *
+ * Use vips_image_get_typeof() to test for the 
+ * existence of a piece of metadata.
+ *
+ * See also: vips_image_get(), vips_image_set_image()
+ *
+ * Returns: 0 on success, -1 otherwise.
+ */
+int 
+vips_image_get_array_int( VipsImage *image, const char *name, 
+	int **out, int *n )
+{
+	GValue value = { 0 };
+
+	if( meta_get_value( image, name, VIPS_TYPE_ARRAY_INT, &value ) ) 
+		return( -1 ); 
+	*out = vips_value_get_array_int( &value, n );
+	g_value_unset( &value );
+
+	return( 0 );
+}
+
+/**
+ * vips_image_set_array_int: (method)
+ * @image: image to attach the metadata to
+ * @name: metadata name
+ * @array: (array length=n) (allow-none): array of ints
+ * @n: the number of elements 
+ *
+ * Attaches @array as a metadata item on @image as @name. 
+ * A convenience function over vips_image_set().
+ *
+ * See also: vips_image_get_image(), vips_image_set().
+ */
+void
+vips_image_set_array_int( VipsImage *image, const char *name,
+	const int *array, int n )
+{
+	GValue value = { 0 };
+
+	g_value_init( &value, VIPS_TYPE_ARRAY_INT );
+	vips_value_set_array_int( &value, array, n );
+	vips_image_set( image, name, &value );
+	g_value_unset( &value );
+}
+
 /**
  * vips_image_history_printf: (method)
  * @image: add history line to this image
@@ -1920,3 +2079,12 @@ vips_image_get_history( VipsImage *image )
 
 	return( image->Hist ? image->Hist : "" );
 }
+
+/* Called during vips_init().
+ */
+void
+vips__meta_init( void )
+{
+	if( !vips__meta_lock ) 
+		vips__meta_lock = vips_g_mutex_new();
+}
