@@ -71,11 +71,11 @@
  *
  * <emphasis>Functional class creation</emphasis> Vips objects have a very 
  * regular lifecycle: initialise, build, use, destroy. They behave rather like
- * function calls and are free of side-effects. 
+ * function calls and are free of side-effects.
  *
  * <emphasis>Run-time introspection</emphasis> Vips objects can be fully 
- * introspected at run-time. There is not need for separate source-code 
- * analysis. 
+ * introspected at run-time. There is no need for separate source-code 
+ * analysis.
  *
  * <emphasis>Command-line interface</emphasis> Any vips object can be run from
  * the command-line with the `vips` driver program. 
@@ -1460,7 +1460,7 @@ vips_object_real_build( VipsObject *object )
 }
 
 static int
-vips_object_real_postbuild( VipsObject *object )
+vips_object_real_postbuild( VipsObject *object, void *data )
 {
 #ifdef DEBUG
 	printf( "vips_object_real_postbuild: " ); 
@@ -1701,7 +1701,7 @@ traverse_find_required_priority( void *data, void *a, void *b )
 }
 
 static gint
-traverse_sort( gconstpointer a, gconstpointer b )
+traverse_sort( gconstpointer a, gconstpointer b, void *user_data )
 {
 	VipsArgumentClass *class1 = (VipsArgumentClass *) a;
 	VipsArgumentClass *class2 = (VipsArgumentClass *) b;
@@ -1803,7 +1803,7 @@ vips_object_class_install_argument( VipsObjectClass *object_class,
 	argument_table_traverse = g_slist_prepend(
 		argument_table_traverse, argument_class );
 	argument_table_traverse = g_slist_sort(
-		argument_table_traverse, traverse_sort );
+		argument_table_traverse, (GCompareFunc) traverse_sort );
 	VIPS_SWAP( GSList *, 
 		argument_table_traverse, 
 		object_class->argument_table_traverse ); 
@@ -1886,29 +1886,48 @@ vips_object_set_argument_from_string( VipsObject *object,
 		VipsImage *out;
 		VipsOperationFlags flags;
 		VipsAccess access;
-
-		if( !value ) {
-			vips_object_no_value( object, name );
-			return( -1 );
-		}
+		char filename[VIPS_PATH_MAX];
+		char option_string[VIPS_PATH_MAX];
 
 		flags = 0;
 		if( VIPS_IS_OPERATION( object ) )
 			flags = vips_operation_get_flags( 
 				VIPS_OPERATION( object ) );
 
-		/* Read the filename. 
-		 */
-		if( flags & (VIPS_OPERATION_SEQUENTIAL_UNBUFFERED |
-			     VIPS_OPERATION_SEQUENTIAL) ) 
+		if( flags & 
+			(VIPS_OPERATION_SEQUENTIAL_UNBUFFERED | 
+			 VIPS_OPERATION_SEQUENTIAL) ) 
 			access = VIPS_ACCESS_SEQUENTIAL;
 		else
 			access = VIPS_ACCESS_RANDOM; 
 
-		if( !(out = vips_image_new_from_file( value, 
-			"access", access,
-			NULL )) )
+		if( !value ) {
+			vips_object_no_value( object, name );
 			return( -1 );
+		}
+		vips__filename_split8( value, filename, option_string );
+
+		if( strcmp( "stdin", filename ) == 0 ) {
+			VipsSource *source;
+
+			if( !(source = 
+				vips_source_new_from_descriptor( 0 )) )
+				return( -1 );
+			if( !(out = vips_image_new_from_source( source, 
+				option_string, 
+				"access", access,
+				NULL )) ) {
+				VIPS_UNREF( source );
+				return( -1 );
+			}
+			VIPS_UNREF( source );
+		}
+		else {
+			if( !(out = vips_image_new_from_file( value, 
+				"access", access,
+				NULL )) )
+				return( -1 );
+		}
 
 		g_value_init( &gvalue, VIPS_TYPE_IMAGE );
 		g_value_set_object( &gvalue, out );
@@ -1918,6 +1937,25 @@ vips_object_set_argument_from_string( VipsObject *object,
 		 */
 		g_object_unref( out );
 	}
+	else if( g_type_is_a( otype, VIPS_TYPE_SOURCE ) ) { 
+		VipsSource *source;
+
+		if( !value ) {
+			vips_object_no_value( object, name );
+			return( -1 );
+		}
+
+		if( !(source = vips_source_new_from_options( value )) )
+			return( -1 );
+	
+		g_value_init( &gvalue, VIPS_TYPE_SOURCE );
+		g_value_set_object( &gvalue, source );
+
+		/* Setting gvalue will have upped @out's count again,
+		 * go back to 1 so that gvalue has the only ref.
+		 */
+		g_object_unref( source );
+	}
 	else if( g_type_is_a( otype, VIPS_TYPE_ARRAY_IMAGE ) ) { 
 		/* We have to have a special case for this, we can't just rely
 		 * on transform_g_string_array_image(), since we need to be
@@ -2138,6 +2176,17 @@ vips_object_print_arg( VipsObject *object, GParamSpec *pspec, VipsBuf *buf )
 	g_value_unset( &value );
 }
 
+/* Is a filename a target, ie. it is of the form ".jpg". Any trailing options 
+ * have already been stripped. Watch out for cases like "./x.jpg".
+ */
+static gboolean
+vips_filename_istarget( const char *filename )
+{
+	const char *p;
+
+	return( (p = strrchr( filename, '.' )) && p == filename );
+}
+
 /* Write a named arg to the string. If the arg does not need a string (see
  * above), arg will be NULL.
  */
@@ -2166,14 +2215,36 @@ vips_object_get_argument_to_string( VipsObject *object,
 
 	if( g_type_is_a( otype, VIPS_TYPE_IMAGE ) ) { 
 		VipsImage *in;
-/* Pull out the image and write it.
-		 */
-		g_object_get( object, name, &in, NULL );
-		if( vips_image_write_to_file( in, arg, NULL ) ) {
-			g_object_unref( in );
-			return( -1 );
+		char filename[VIPS_PATH_MAX];
+		char option_string[VIPS_PATH_MAX];
+
+		vips__filename_split8( arg, filename, option_string );
+
+		if( vips_filename_istarget( filename ) ) {
+			VipsTarget *target;
+
+			if( !(target = vips_target_new_to_descriptor( 1 )) )
+				return( -1 );
+			g_object_get( object, name, &in, NULL );
+			if( vips_image_write_to_target( in, 
+				arg, target, NULL ) ) {
+				VIPS_UNREF( in );
+				VIPS_UNREF( target );
+				return( -1 );
+			}
+			VIPS_UNREF( in );
+			VIPS_UNREF( target );
+		}
+		else {
+			/* Pull out the image and write it.
+			 */
+			g_object_get( object, name, &in, NULL );
+			if( vips_image_write_to_file( in, arg, NULL ) ) {
+				VIPS_UNREF( in );
+				return( -1 );
+			}
+			VIPS_UNREF( in );
 		}
-		g_object_unref( in );
 	}
 	else if( g_type_is_a( otype, VIPS_TYPE_OBJECT ) &&
 		(oclass = g_type_class_ref( otype )) &&
@@ -2985,7 +3056,7 @@ typedef struct {
 } VipsObjectLocal;
 
 static void
-vips_object_local_array_cb( GObject *parent, VipsObjectLocal *local )
+vips_object_local_array_cb( VipsObject *parent, VipsObjectLocal *local )
 {
 	int i;
 
@@ -3051,7 +3122,7 @@ vips_object_set_static( VipsObject *object, gboolean static_object )
 }
 
 static void *
-vips_object_n_static_cb( VipsObject *object, int *n )
+vips_object_n_static_cb( VipsObject *object, int *n, void *b )
 {
 	if( object->static_object )
 		*n += 1;
@@ -3072,7 +3143,7 @@ vips_object_n_static( void )
 }
 
 static void *
-vips_object_print_all_cb( VipsObject *object, int *n )
+vips_object_print_all_cb( VipsObject *object, int *n, void *b )
 {
 	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( object );
 
@@ -3116,7 +3187,7 @@ vips_object_print_all( void )
 }
 
 static void *
-vips_object_sanity_all_cb( VipsObject *object )
+vips_object_sanity_all_cb( VipsObject *object, void *a, void *b )
 {
 	(void) vips_object_sanity( object );
 
