@@ -611,7 +611,7 @@ vips_operation_get_flags( VipsOperation *operation )
 void
 vips_operation_class_print_usage( VipsOperationClass *operation_class )
 {
-	char str[2048];
+	char str[4096];
 	VipsBuf buf = VIPS_BUF_STATIC( str );
 
 	operation_class->usage( operation_class, &buf );
@@ -1245,7 +1245,8 @@ typedef struct _VipsCall {
 static const char *
 vips_call_get_arg( VipsCall *call, int i )
 {
-	if( i < 0 || i >= call->argc ) {
+	if( i < 0 || 
+		i >= call->argc ) {
 		vips_error( VIPS_OBJECT_GET_CLASS( call->operation )->nickname, 
 			"%s", _( "too few arguments" ) );
 		return( NULL );
