@@ -45,6 +45,9 @@
  * 	- multiply transparent images through alpha in vips_region_shrink()
  * 13/6/18 harukizaemon
  * 	- add VipsRegionShrink parameter to vips_region_shrink()
+ * 9/6/19
+ * 	- saner behaviour for vips_region_fetch() if the request is partly 
+ * 	  outside the image
  */
 
 /*
@@ -364,28 +367,6 @@ vips_region_summary( VipsObject *object, VipsBuf *buf )
 	VIPS_OBJECT_CLASS( vips_region_parent_class )->summary( object, buf );
 }
 
-static void
-vips_region_sanity( VipsObject *object, VipsBuf *buf )
-{
-	VipsRegion *region = VIPS_REGION( object );
-
-	(void) vips_object_sanity( VIPS_OBJECT( region->im ) );
-
-	switch( region->im->dtype ) { 
-	case VIPS_IMAGE_PARTIAL:
-		/* Start and stop can be NULL, but not generate.
-		 */
-		if( !region->im->generate_fn )
-			vips_buf_appends( buf, "generate NULL in partial\n" );
-		break;
-	
-	default:
-		break;
-	}
-
-	VIPS_OBJECT_CLASS( vips_region_parent_class )->sanity( object, buf );
-}
-
 /* If a region is being created in one thread (eg. the main thread) and then
  * used in another (eg. a worker thread), the new thread needs to tell VIPS
  * to stop sanity g_assert() fails. The previous owner needs to
@@ -491,7 +472,6 @@ vips_region_class_init( VipsRegionClass *class )
 
 	vobject_class->summary = vips_region_summary;
 	vobject_class->dump = vips_region_dump;
-	vobject_class->sanity = vips_region_sanity;
 	vobject_class->build = vips_region_build;
 }
 
@@ -527,12 +507,8 @@ vips_region_new( VipsImage *image )
 	 * We can't use the property system, we need to be very threaded.
 	 */
 	g_object_ref( image );
-
 	g_assert( G_OBJECT( image )->ref_count > 1 );
-
-#ifdef DEBUG
 	g_assert( vips_object_sanity( VIPS_OBJECT( image ) ) );
-#endif /*DEBUG*/
 
 	region = VIPS_REGION( g_object_new( VIPS_TYPE_REGION, NULL ) );
 	region->im = image;
@@ -542,9 +518,7 @@ vips_region_new( VipsImage *image )
 		return( NULL );
 	}
 
-#ifdef DEBUG
 	g_assert( vips_object_sanity( VIPS_OBJECT( region ) ) );
-#endif /*DEBUG*/
 
 	return( region ); 
 }
@@ -1261,6 +1235,8 @@ vips_region_shrink_uncoded_mean( VipsRegion *from,
  * IS stable with respect to the initial arrangement of input values
  */
 #define SHRINK_TYPE_MEDIAN( TYPE ) { \
+	int ls = VIPS_REGION_LSKIP( from ); \
+	\
 	for( x = 0; x < target->width; x++ ) { \
 		TYPE *tp = (TYPE *) p; \
 		TYPE *tp1 = (TYPE *) (p + ls); \
@@ -1280,50 +1256,6 @@ vips_region_shrink_uncoded_mean( VipsRegion *from,
 	} \
 }
 
-/* Generate area @target in @to using pixels in @from. Non-complex.
- */
-static void
-vips_region_shrink_uncoded_median( VipsRegion *from,
-	VipsRegion *to, const VipsRect *target )
-{
-	int ls = VIPS_REGION_LSKIP( from );
-	int ps = VIPS_IMAGE_SIZEOF_PEL( from->im );
-	int nb = from->im->Bands;
-
-	int x, y, z;
-
-	for( y = 0; y < target->height; y++ ) {
-		VipsPel *p = VIPS_REGION_ADDR( from,
-			target->left * 2, (target->top + y) * 2 );
-		VipsPel *q = VIPS_REGION_ADDR( to,
-			target->left, target->top + y );
-
-		/* Process this line of pels.
-		 */
-		switch( from->im->BandFmt ) {
-		case VIPS_FORMAT_UCHAR:
-			SHRINK_TYPE_MEDIAN( unsigned char );  break;
-		case VIPS_FORMAT_CHAR:
-			SHRINK_TYPE_MEDIAN( signed char );  break;
-		case VIPS_FORMAT_USHORT:
-			SHRINK_TYPE_MEDIAN( unsigned short );  break;
-		case VIPS_FORMAT_SHORT:
-			SHRINK_TYPE_MEDIAN( signed short );  break;
-		case VIPS_FORMAT_UINT:
-			SHRINK_TYPE_MEDIAN( unsigned int );  break;
-		case VIPS_FORMAT_INT:
-			SHRINK_TYPE_MEDIAN( signed int );  break;
-		case VIPS_FORMAT_FLOAT:
-			SHRINK_TYPE_MEDIAN( float );  break;
-		case VIPS_FORMAT_DOUBLE:
-			SHRINK_TYPE_MEDIAN( double );  break;
-
-		default:
-			g_assert_not_reached();
-		}
-	}
-}
-
 /* This method is implemented so as to perform well and to always select an
  * output pixel from one of the input pixels. As such we make only the
  * following guarantees:
@@ -1335,6 +1267,8 @@ vips_region_shrink_uncoded_median( VipsRegion *from,
  * IS stable with respect to the initial arrangement of input values
  */
 #define SHRINK_TYPE_MODE( TYPE ) { \
+	int ls = VIPS_REGION_LSKIP( from ); \
+	\
 	for( x = 0; x < target->width; x++ ) { \
 		TYPE *tp = (TYPE *) p; \
 		TYPE *tp1 = (TYPE *) (p + ls); \
@@ -1353,57 +1287,112 @@ vips_region_shrink_uncoded_median( VipsRegion *from,
         		tq[z] = v[index]; \
 		} \
 		\
-		/* Move on two pels in input. \
-		 */ \
 		p += ps << 1; \
 		q += ps; \
 	} \
 }
 
-/* Generate area @target in @to using pixels in @from. Non-complex.
- */
-static void
-vips_region_shrink_uncoded_mode( VipsRegion *from,
-	VipsRegion *to, const VipsRect *target )
-{
-	int ls = VIPS_REGION_LSKIP( from );
-	int ps = VIPS_IMAGE_SIZEOF_PEL( from->im );
-	int nb = from->im->Bands;
-
-	int x, y, z;
-
-	for( y = 0; y < target->height; y++ ) {
-		VipsPel *p = VIPS_REGION_ADDR( from,
-			target->left * 2, (target->top + y) * 2 );
-		VipsPel *q = VIPS_REGION_ADDR( to,
-			target->left, target->top + y );
-
-		/* Process this line of pels.
-		 */
-		switch( from->im->BandFmt ) {
-		case VIPS_FORMAT_UCHAR:
-			SHRINK_TYPE_MODE( unsigned char );  break;
-		case VIPS_FORMAT_CHAR:
-			SHRINK_TYPE_MODE( signed char );  break;
-		case VIPS_FORMAT_USHORT:
-			SHRINK_TYPE_MODE( unsigned short );  break;
-		case VIPS_FORMAT_SHORT:
-			SHRINK_TYPE_MODE( signed short );  break;
-		case VIPS_FORMAT_UINT:
-			SHRINK_TYPE_MODE( unsigned int );  break;
-		case VIPS_FORMAT_INT:
-			SHRINK_TYPE_MODE( signed int );  break;
-		case VIPS_FORMAT_FLOAT:
-			SHRINK_TYPE_MODE( float );  break;
-		case VIPS_FORMAT_DOUBLE:
-			SHRINK_TYPE_MODE( double );  break;
-
-		default:
-			g_assert_not_reached();
-		}
-	}
+#define SHRINK_TYPE_MAX( TYPE ) { \
+	int ls = VIPS_REGION_LSKIP( from ); \
+	\
+	for( x = 0; x < target->width; x++ ) { \
+		TYPE *tp = (TYPE *) p; \
+		TYPE *tp1 = (TYPE *) (p + ls); \
+		TYPE *tq = (TYPE *) q; \
+		\
+		for( z = 0; z < nb; z++ ) { \
+        		tq[z] = VIPS_MAX( \
+					VIPS_MAX( tp[z], tp[z + nb] ), \
+					VIPS_MAX( tp1[z], tp1[z + nb] ) \
+				); \
+		} \
+		\
+		p += ps << 1; \
+		q += ps; \
+	} \
 }
 
+#define SHRINK_TYPE_MIN( TYPE ) { \
+	int ls = VIPS_REGION_LSKIP( from ); \
+	\
+	for( x = 0; x < target->width; x++ ) { \
+		TYPE *tp = (TYPE *) p; \
+		TYPE *tp1 = (TYPE *) (p + ls); \
+		TYPE *tq = (TYPE *) q; \
+		\
+		for( z = 0; z < nb; z++ ) { \
+        		tq[z] = VIPS_MIN( \
+					VIPS_MIN( tp[z], tp[z + nb] ), \
+					VIPS_MIN( tp1[z], tp1[z + nb] ) \
+				); \
+		} \
+		\
+		p += ps << 1; \
+		q += ps; \
+	} \
+}
+
+#define SHRINK_TYPE_NEAREST( TYPE ) { \
+	for( x = 0; x < target->width; x++ ) { \
+		TYPE *tp = (TYPE *) p; \
+		TYPE *tq = (TYPE *) q; \
+		\
+		for( z = 0; z < nb; z++ ) \
+        		tq[z] = tp[z]; \
+		\
+		p += ps << 1; \
+		q += ps; \
+	} \
+}
+
+#define VIPS_REGION_SHRINK( OP ) \
+static void \
+vips_region_shrink_uncoded_ ## OP( VipsRegion *from, \
+	VipsRegion *to, const VipsRect *target ) \
+{ \
+	int ps = VIPS_IMAGE_SIZEOF_PEL( from->im ); \
+	int nb = from->im->Bands; \
+ 	\
+	int x, y, z; \
+ 	\
+	for( y = 0; y < target->height; y++ ) { \
+		VipsPel *p = VIPS_REGION_ADDR( from, \
+			target->left * 2, (target->top + y) * 2 ); \
+		VipsPel *q = VIPS_REGION_ADDR( to, \
+			target->left, target->top + y ); \
+ 		\
+		/* Process this line of pels. \
+		 */ \
+		switch( from->im->BandFmt ) { \
+		case VIPS_FORMAT_UCHAR: \
+			SHRINK_TYPE_ ## OP( unsigned char );  break; \
+		case VIPS_FORMAT_CHAR: \
+			SHRINK_TYPE_ ## OP( signed char );  break; \
+		case VIPS_FORMAT_USHORT: \
+			SHRINK_TYPE_ ## OP( unsigned short );  break; \
+		case VIPS_FORMAT_SHORT: \
+			SHRINK_TYPE_ ## OP( signed short );  break; \
+		case VIPS_FORMAT_UINT: \
+			SHRINK_TYPE_ ## OP( unsigned int );  break; \
+		case VIPS_FORMAT_INT: \
+			SHRINK_TYPE_ ## OP( signed int );  break; \
+		case VIPS_FORMAT_FLOAT: \
+			SHRINK_TYPE_ ## OP( float );  break; \
+		case VIPS_FORMAT_DOUBLE: \
+			SHRINK_TYPE_ ## OP( double );  break; \
+ 		\
+		default: \
+			g_assert_not_reached(); \
+		} \
+	} \
+} 
+
+VIPS_REGION_SHRINK( MAX );
+VIPS_REGION_SHRINK( MIN );
+VIPS_REGION_SHRINK( MODE );
+VIPS_REGION_SHRINK( MEDIAN );
+VIPS_REGION_SHRINK( NEAREST );
+
 /* Generate area @target in @to using pixels in @from. Non-complex.
  */
 static void
@@ -1414,11 +1403,25 @@ vips_region_shrink_uncoded( VipsRegion *from,
 		case VIPS_REGION_SHRINK_MEAN:
 			vips_region_shrink_uncoded_mean( from, to, target );
 			break;
+
 		case VIPS_REGION_SHRINK_MEDIAN:
-			vips_region_shrink_uncoded_median( from, to, target );
+			vips_region_shrink_uncoded_MEDIAN( from, to, target );
 			break;
+
 		case VIPS_REGION_SHRINK_MODE:
-			vips_region_shrink_uncoded_mode( from, to, target );
+			vips_region_shrink_uncoded_MODE( from, to, target );
+			break;
+
+		case VIPS_REGION_SHRINK_MAX:
+			vips_region_shrink_uncoded_MAX( from, to, target );
+			break;
+
+		case VIPS_REGION_SHRINK_MIN:
+			vips_region_shrink_uncoded_MIN( from, to, target );
+			break;
+
+		case VIPS_REGION_SHRINK_NEAREST:
+			vips_region_shrink_uncoded_NEAREST( from, to, target );
 			break;
 
 		default:
@@ -1573,7 +1576,7 @@ vips_region_shrink( VipsRegion *from, VipsRegion *to, const VipsRect *target )
 /* Generate into a region. 
  */
 static int
-vips_region_generate( VipsRegion *reg )
+vips_region_generate( VipsRegion *reg, void *a )
 {
 	VipsImage *im = reg->im;
 
@@ -1656,8 +1659,7 @@ vips_region_prepare( VipsRegion *reg, const VipsRect *r )
 
 	switch( im->dtype ) {
 	case VIPS_IMAGE_PARTIAL:
-		if( vips_region_fill( reg, r, 
-			(VipsRegionFillFn) vips_region_generate, NULL ) )
+		if( vips_region_fill( reg, r, vips_region_generate, NULL ) )
 			return( -1 );
 
 		break;
@@ -1713,7 +1715,7 @@ vips_region_prepare_to_generate( VipsRegion *reg,
 
 	/* Run sequence into reg.
 	 */
-	if( vips_region_generate( reg ) )
+	if( vips_region_generate( reg, NULL ) )
 		return( -1 );
 
 	/* The generate function may not have actually made any pixels ... it
@@ -1735,13 +1737,12 @@ vips_region_prepare_to_generate( VipsRegion *reg,
  * @y: postion of @r in @dest
  *
  * Like vips_region_prepare(): fill @reg with the pixels in area @r. 
- * Unlike vips_region_prepare(), rather than writing the result to @reg, the pixels are
- * written into @dest
- * at offset @x, @y. 
+ *
+ * Unlike vips_region_prepare(), rather than writing the result to @reg, the 
+ * pixels are written into @dest at offset @x, @y. 
  *
  * Also unlike vips_region_prepare(), @dest is not set up for writing for 
- * you with
- * vips_region_buffer(). You can
+ * you with vips_region_buffer(). You can
  * point @dest at anything, and pixels really will be written there. 
  * This makes vips_region_prepare_to() useful for making the ends of 
  * pipelines.
@@ -1896,6 +1897,94 @@ vips_region_prepare_many( VipsRegion **reg, const VipsRect *r )
 	return( 0 );
 }
 
+/** 
+ * vips_region_fetch: (method)
+ * @reg: region to fetch pixels from
+ * @left: area of pixels to fetch
+ * @top: area of pixels to fetch
+ * @width: area of pixels to fetch
+ * @height: area of pixels to fetch
+ *
+ * Generate an area of pixels and return a copy. The result must be freed
+ * with g_free(). The requested area must be completely inside the image.
+ *
+ * This is equivalent to vips_region_prepare(), followed by a memcpy. It is
+ * convenient for language bindings.
+ *
+ * Returns: A copy of the pixel data.
+ */
+VipsPel *
+vips_region_fetch( VipsRegion *region, 
+	int left, int top, int width, int height, size_t *len )
+{
+	VipsRect request;
+	VipsRect image;
+	int y;
+	VipsPel *result;
+	VipsPel *p, *q;
+	size_t skip;
+	size_t line;
+
+	g_assert( width > 0 );
+	g_assert( height > 0 );
+
+	image.left = 0;
+	image.top = 0;
+	image.width = region->im->Xsize;
+	image.height = region->im->Ysize;
+	request.left = left;
+	request.top = top;
+	request.width = width;
+	request.height = height;
+	if( !vips_rect_includesrect( &image, &request ) )
+		return( NULL );
+	if( vips_region_prepare( region, &request ) )
+		return( NULL );
+
+	skip = VIPS_REGION_LSKIP( region );
+	line = VIPS_IMAGE_SIZEOF_PEL( region->im ) * request.width;
+	if( !(result = (VipsPel *) vips_malloc( NULL, line * request.height )) )
+		return( NULL );
+
+	p = VIPS_REGION_ADDR( region, request.left, request.top );
+	q = result;
+	for( y = 0; y < request.height; y++ )  {
+		memcpy( q, p, line ); 
+
+		p += skip;
+		q += line;
+	}
+
+	if( len )
+		*len = request.height * line;
+
+	return( result );
+}
+
+/**
+ * vips_region_width: (method)
+ * @region: fetch width from this
+ *
+ * Returns: Width of the pixels held in region.
+ */
+int
+vips_region_width( VipsRegion *region )
+{
+	return( region->valid.width );
+}	
+
+/**
+ * vips_region_height: (method)
+ * @region: fetch height from this
+ *
+ * Returns: Height of the pixels held in region.
+ */
+int
+vips_region_height( VipsRegion *region )
+{
+	return( region->valid.height );
+}	
+
 /** 
  * vips_region_invalidate: (method)
  * @reg: region to invalidate
@@ -1917,7 +2006,7 @@ vips_region_invalidate( VipsRegion *reg )
 
 #ifdef VIPS_DEBUG
 static void *
-vips_region_dump_all_cb( VipsRegion *region, size_t *alive )
+vips_region_dump_all_cb( VipsRegion *region, size_t *alive, void *b )
 {
 	char str[2048];
 	VipsBuf buf = VIPS_BUF_STATIC( str );
