@@ -33,11 +33,6 @@
 
  */
 
-/* This is a simple wrapper over the old vips7 functions. At some point we
- * should rewrite this as a pure vips8 class and redo the vips7 functions as
- * wrappers over this.
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
@@ -46,7 +41,7 @@
 #include <stdio.h>
 
 #include <vips/vips.h>
-#include <vips/vips7compat.h>
+#include "pmosaicing.h"
 
 typedef struct {
 	VipsOperation parent_instance;
@@ -77,13 +72,13 @@ vips_merge_build( VipsObject *object )
 
 	switch( merge->direction ) { 
 	case VIPS_DIRECTION_HORIZONTAL:
-		if( im_lrmerge( merge->ref, merge->sec, merge->out, 
+		if( vips__lrmerge( merge->ref, merge->sec, merge->out, 
 			merge->dx, merge->dy, merge->mblend ) )
 			return( -1 ); 
 		break;
 
 	case VIPS_DIRECTION_VERTICAL:
-		if( im_tbmerge( merge->ref, merge->sec, merge->out, 
+		if( vips__tbmerge( merge->ref, merge->sec, merge->out, 
 			merge->dx, merge->dy, merge->mblend ) )
 			return( -1 ); 
 		break;
@@ -92,6 +87,17 @@ vips_merge_build( VipsObject *object )
 		g_assert_not_reached();
 	}
 
+	vips__add_mosaic_name( merge->out );
+	if( vips_image_history_printf( merge->out, 
+		"#%s <%s> <%s> <%s> <%d> <%d> <%d>", 
+		merge->direction == VIPS_DIRECTION_HORIZONTAL ?
+			"LRJOIN" : "TBJOIN",
+		vips__get_mosaic_name( merge->ref ), 
+		vips__get_mosaic_name( merge->sec ), 
+		vips__get_mosaic_name( merge->out ), 
+		-merge->dx, -merge->dy, merge->mblend ) ) 
+		return( -1 );
+
 	return( 0 );
 }
 
@@ -131,7 +137,7 @@ vips_merge_class_init( VipsMergeClass *class )
 
 	VIPS_ARG_ENUM( class, "direction", 4, 
 		_( "Direction" ), 
-		_( "Horizontal or vertcial merge" ),
+		_( "Horizontal or vertical merge" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT, 
 		G_STRUCT_OFFSET( VipsMerge, direction ), 
 		VIPS_TYPE_DIRECTION, VIPS_DIRECTION_HORIZONTAL ); 
