@@ -21,7 +21,6 @@
  * 	- deprecate @centre option, it's now always on
  * 	- fix pixel shift
  * 	- speed up the mask construction for uchar/ushort images
- * 	- remove unnecessary round-to-nearest behaviour
  */
 
 /*
@@ -118,14 +117,15 @@ typedef struct _VipsReducev {
 	double voffset;
 
 	/* Precalculated interpolation matrices. int (used for pel
-	 * sizes up to short), and double (for all others).
+	 * sizes up to short), and double (for all others). We go to
+	 * scale + 1 so we can round-to-nearest safely.
 	 */
-	int *matrixi[VIPS_TRANSFORM_SCALE];
-	double *matrixf[VIPS_TRANSFORM_SCALE];
+	int *matrixi[VIPS_TRANSFORM_SCALE + 1];
+	double *matrixf[VIPS_TRANSFORM_SCALE + 1];
 
 	/* And another set for orc: we want 2.6 precision.
 	 */
-	int *matrixo[VIPS_TRANSFORM_SCALE];
+	int *matrixo[VIPS_TRANSFORM_SCALE + 1];
 
 	/* The passes we generate for this mask.
 	 */
@@ -154,7 +154,7 @@ vips_reducev_finalize( GObject *gobject )
 	for( int i = 0; i < reducev->n_pass; i++ )
 		VIPS_FREEF( vips_vector_free, reducev->pass[i].vector );
 	reducev->n_pass = 0;
-	for( int i = 0; i < VIPS_TRANSFORM_SCALE; i++ ) {
+	for( int i = 0; i < VIPS_TRANSFORM_SCALE + 1; i++ ) {
 		VIPS_FREE( reducev->matrixf[i] );
 		VIPS_FREE( reducev->matrixi[i] );
 		VIPS_FREE( reducev->matrixo[i] );
@@ -511,8 +511,12 @@ reducev_notab( VipsReducev *reducev,
 
 	vips_reduce_make_mask( cy, reducev->kernel, reducev->vshrink, y ); 
 
-	for( int z = 0; z < ne; z++ ) 
-		out[z] = reduce_sum<T, double>( in + z, l1, cy, n );
+	for( int z = 0; z < ne; z++ ) {
+		double sum;
+		sum = reduce_sum<T, double>( in + z, l1, cy, n );
+
+		out[z] = VIPS_ROUND_UINT( sum );
+	}
 }
 
 static int
@@ -555,8 +559,9 @@ vips_reducev_gen( VipsRegion *out_region, void *vseq,
 			VIPS_REGION_ADDR( out_region, r->left, r->top + y );
 		const int py = (int) Y;
 		VipsPel *p = VIPS_REGION_ADDR( ir, r->left, py );
-		const int sy = Y * VIPS_TRANSFORM_SCALE;
-		const int ty = sy & (VIPS_TRANSFORM_SCALE - 1);
+		const int sy = Y * VIPS_TRANSFORM_SCALE * 2;
+		const int siy = sy & (VIPS_TRANSFORM_SCALE * 2 - 1);
+		const int ty = (siy + 1) >> 1;
 		const int *cyi = reducev->matrixi[ty];
 		const double *cyf = reducev->matrixf[ty];
 		const int lskip = VIPS_REGION_LSKIP( ir );
@@ -677,8 +682,9 @@ vips_reducev_vector_gen( VipsRegion *out_region, void *vseq,
 		VipsPel *q = 
 			VIPS_REGION_ADDR( out_region, r->left, r->top + y );
 		const int py = (int) Y;
-		const int sy = Y * VIPS_TRANSFORM_SCALE;
-		const int ty = sy & (VIPS_TRANSFORM_SCALE - 1);
+		const int sy = Y * VIPS_TRANSFORM_SCALE * 2;
+		const int siy = sy & (VIPS_TRANSFORM_SCALE * 2 - 1);
+		const int ty = (siy + 1) >> 1;
 		const int *cyo = reducev->matrixo[ty];
 
 #ifdef DEBUG_PIXELS
@@ -739,7 +745,7 @@ vips_reducev_raw( VipsReducev *reducev, VipsImage *in, VipsImage **out )
 	 */
 	if( in->BandFmt == VIPS_FORMAT_UCHAR &&
 		vips_vector_isenabled() ) 
-		for( int y = 0; y < VIPS_TRANSFORM_SCALE; y++ ) {
+		for( int y = 0; y < VIPS_TRANSFORM_SCALE + 1; y++ ) {
 			reducev->matrixo[y] = 
 				VIPS_ARRAY( NULL, reducev->n_point, int ); 
 			if( !reducev->matrixo[y] )
@@ -847,11 +853,11 @@ vips_reducev_build( VipsObject *object )
 	 * by half that distance so that we discard pixels equally
 	 * from left and right. 
 	 */
-	reducev->voffset = (1 + extra_pixels) / 2.0;
+	reducev->voffset = (1 + extra_pixels) / 2.0 - 1;
 
 	/* Build the tables of pre-computed coefficients.
 	 */
-	for( int y = 0; y < VIPS_TRANSFORM_SCALE; y++ ) {
+	for( int y = 0; y < VIPS_TRANSFORM_SCALE + 1; y++ ) {
 		reducev->matrixf[y] = 
 			VIPS_ARRAY( NULL, reducev->n_point, double ); 
 		reducev->matrixi[y] = 
@@ -885,7 +891,7 @@ vips_reducev_build( VipsObject *object )
 	/* Add new pixels around the input so we can interpolate at the edges.
 	 */
 	if( vips_embed( in, &t[1], 
-		0, reducev->n_point / 2 - 1, 
+		0, VIPS_CEIL( reducev->n_point / 2.0 ) - 1, 
 		in->Xsize, in->Ysize + reducev->n_point, 
 		"extend", VIPS_EXTEND_COPY,
 		(void *) NULL ) )
