@@ -51,30 +51,6 @@ save_load() {
 	fi
 }
 
-# is a difference beyond a threshold? return 0 (meaning all ok) or 1 (meaning
-# error, or outside threshold)
-break_threshold() {
-	diff=$1
-	threshold=$2
-	return $(echo "$diff <= $threshold" | bc -l)
-}
-
-# subtract, look for max difference less than a threshold
-test_difference() {
-	before=$1
-	after=$2
-	threshold=$3
-
-	$vips subtract $before $after $tmp/difference.v
-	$vips abs $tmp/difference.v $tmp/abs.v 
-	dif=$($vips max $tmp/abs.v)
-
-	if break_threshold $dif $threshold; then
-		echo "save / load difference is $dif"
-		exit 1
-	fi
-}
-
 # save to the named file in tmp, convert back to vips again, subtract, look
 # for max difference less than a threshold
 test_format() {
@@ -167,20 +143,6 @@ test_saver() {
 	echo "ok"
 }
 
-# test for file format supported
-test_supported() {
-	format=$1
-
-	if $vips $format > /dev/null 2>&1; then
-		result=0
-	else
-		echo "support for $format not configured, skipping test"
-		result=1
-	fi
-
-	return $result
-}
-
 test_format $image v 0
 if test_supported tiffload; then
 	test_format $image tif 0
