@@ -90,6 +90,18 @@
  * 6/1/17
  * 	- fancy geometry strings
  * 	- support VipSize restrictions
+ * 4/5/17
+ * 	- add ! geo modifier
+ * 30/8/17
+ * 	- add --intent
+ * 23/10/17
+ * 	- --size Nx didn't work, argh ... thanks jrochkind 
+ * 3/2/20
+ * 	- add --no-rotate
+ * 	- add --import-profile / --export-profile names
+ * 	- back to -o for output
+ * 29/2/20
+ * 	- deprecate --delete
  */
 
 #ifdef HAVE_CONFIG_H
@@ -118,38 +130,36 @@ static VipsSize size_restriction = VIPS_SIZE_BOTH;
 static char *output_format = "tn_%s.jpg";
 static char *export_profile = NULL;
 static char *import_profile = NULL;
-static gboolean delete_profile = FALSE;
 static gboolean linear_processing = FALSE;
 static gboolean crop_image = FALSE;
+static gboolean no_rotate_image = FALSE;
 static char *smartcrop_image = NULL;
-static gboolean rotate_image = FALSE;
+static char *thumbnail_intent = NULL;
 
 /* Deprecated and unused.
  */
+static gboolean delete_profile = FALSE;
 static gboolean nosharpen = FALSE;
 static gboolean nodelete_profile = FALSE;
 static gboolean verbose = FALSE;
 static char *convolution_mask = NULL;
 static char *interpolator = NULL;
+static gboolean rotate_image = FALSE;
 
 static GOptionEntry options[] = {
 	{ "size", 's', 0, 
 		G_OPTION_ARG_STRING, &thumbnail_size, 
 		N_( "shrink to SIZE or to WIDTHxHEIGHT" ), 
 		N_( "SIZE" ) },
-	{ "output", 'o', G_OPTION_FLAG_HIDDEN, 
+	{ "output", 'o', 0, 
 		G_OPTION_ARG_STRING, &output_format, 
-		N_( "set output to FORMAT" ), 
+		N_( "output to FORMAT" ), 
 		N_( "FORMAT" ) },
-	{ "format", 'f', 0, 
-		G_OPTION_ARG_STRING, &output_format, 
-		N_( "set output format string to FORMAT" ), 
-		N_( "FORMAT" ) },
-	{ "eprofile", 'e', 0, 
+	{ "export-profile", 'e', 0, 
 		G_OPTION_ARG_FILENAME, &export_profile, 
 		N_( "export with PROFILE" ), 
 		N_( "PROFILE" ) },
-	{ "iprofile", 'i', 0, 
+	{ "import-profile", 'i', 0, 
 		G_OPTION_ARG_FILENAME, &import_profile, 
 		N_( "import untagged images with PROFILE" ), 
 		N_( "PROFILE" ) },
@@ -160,13 +170,32 @@ static GOptionEntry options[] = {
 		G_OPTION_ARG_STRING, &smartcrop_image, 
 		N_( "shrink and crop to fill SIZE using STRATEGY" ), 
 		N_( "STRATEGY" ) },
-	{ "rotate", 't', 0, 
-		G_OPTION_ARG_NONE, &rotate_image, 
-		N_( "auto-rotate" ), NULL },
-	{ "delete", 'd', 0, 
+	{ "intent", 'n', 0, 
+		G_OPTION_ARG_STRING, &thumbnail_intent, 
+		N_( "ICC transform with INTENT" ), 
+		N_( "INTENT" ) },
+	{ "delete", 'd', G_OPTION_FLAG_HIDDEN, 
 		G_OPTION_ARG_NONE, &delete_profile, 
-		N_( "delete profile from exported image" ), NULL },
+		N_( "(deprecated, does nothing)" ), NULL },
+	{ "no-rotate", 0, 0, 
+		G_OPTION_ARG_NONE, &no_rotate_image, 
+		N_( "don't auto-rotate" ), NULL },
 
+	{ "format", 'f', G_OPTION_FLAG_HIDDEN, 
+		G_OPTION_ARG_STRING, &output_format, 
+		N_( "set output format string to FORMAT" ), 
+		N_( "FORMAT" ) },
+	{ "eprofile", 0, G_OPTION_FLAG_HIDDEN, 
+		G_OPTION_ARG_FILENAME, &export_profile, 
+		N_( "export with PROFILE" ), 
+		N_( "PROFILE" ) },
+	{ "iprofile", 0, G_OPTION_FLAG_HIDDEN, 
+		G_OPTION_ARG_FILENAME, &import_profile, 
+		N_( "import untagged images with PROFILE" ), 
+		N_( "PROFILE" ) },
+	{ "rotate", 't', G_OPTION_FLAG_HIDDEN, 
+		G_OPTION_ARG_NONE, &rotate_image, 
+		N_( "(deprecated, does nothing)" ), NULL },
 	{ "crop", 'c', G_OPTION_FLAG_HIDDEN, 
 		G_OPTION_ARG_NONE, &crop_image, 
 		N_( "(deprecated, crop exactly to SIZE)" ), NULL },
@@ -243,27 +272,38 @@ thumbnail_process( VipsObject *process, const char *filename )
 {
 	VipsInteresting interesting;
 	VipsImage *image;
+	VipsIntent intent;
 
 	interesting = VIPS_INTERESTING_NONE;
 	if( crop_image )
 		interesting = VIPS_INTERESTING_CENTRE;
 	if( smartcrop_image ) {
 		int n;
-		
+
 		if( (n = vips_enum_from_nick( "vipsthumbnail", 
 			VIPS_TYPE_INTERESTING, smartcrop_image )) < 0 ) 
 			return( -1 ); 
 		interesting = n;
 	}
+	intent = VIPS_INTENT_RELATIVE;
+	if( thumbnail_intent ) {
+		int n;
+
+		if( (n = vips_enum_from_nick( "vipsthumbnail", 
+			VIPS_TYPE_INTENT, thumbnail_intent )) < 0 ) 
+			return( -1 ); 
+		intent = n;
+	}
 
 	if( vips_thumbnail( filename, &image, thumbnail_width, 
 		"height", thumbnail_height, 
 		"size", size_restriction, 
-		"auto_rotate", rotate_image, 
+		"no-rotate", no_rotate_image, 
 		"crop", interesting, 
 		"linear", linear_processing, 
-		"import_profile", import_profile, 
-		"export_profile", export_profile, 
+		"import-profile", import_profile, 
+		"export-profile", export_profile, 
+		"intent", intent, 
 		NULL ) )
 		return( -1 );
 
@@ -290,6 +330,7 @@ thumbnail_parse_geometry( const char *geometry )
 	 */
 
 	const char *p;
+	gboolean had_x;
 
 	/* w or h missing means replace with a huuuge value to prevent 
 	 * reduction or enlargement in that axis.
@@ -304,13 +345,8 @@ thumbnail_parse_geometry( const char *geometry )
 	while( isspace( *p ) )
 		p++;
 	if( isdigit ( *p ) ) {
-		/* We have a number! vipsthumbnail history means that "-s 200"
-		 * means "200x200", not "200xhuge"
-		 */
-		thumbnail_width = thumbnail_height = atoi( p );
+		thumbnail_width = atoi( p );
 
-		/* And skip over it.
-		 */
 		while( isdigit( *p ) )
 			p++;
 	}
@@ -319,8 +355,11 @@ thumbnail_parse_geometry( const char *geometry )
 	 */
 	while( isspace( *p ) )
 		p++;
-	if( *p == 'x' ) 
+	had_x = FALSE;
+	if( *p == 'x' ) {
 		p += 1;
+		had_x = TRUE;
+	}
 	while( isspace( *p ) )
 		p++;
 
@@ -333,7 +372,7 @@ thumbnail_parse_geometry( const char *geometry )
 			p++;
 	}
 
-	/* Get the final < or >.
+	/* Get the final <>!
 	 */
 	while( isspace( *p ) )
 		p++;
@@ -341,6 +380,8 @@ thumbnail_parse_geometry( const char *geometry )
 		size_restriction = VIPS_SIZE_UP;
 	else if( *p == '>' ) 
 		size_restriction = VIPS_SIZE_DOWN;
+	else if( *p == '!' ) 
+		size_restriction = VIPS_SIZE_FORCE;
 	else if( *p != '\0' ||
 		(thumbnail_width == VIPS_MAX_COORD && 
 			thumbnail_height == VIPS_MAX_COORD) ) {
@@ -348,18 +389,36 @@ thumbnail_parse_geometry( const char *geometry )
 		return( -1 );
 	}
 
-	/* If --crop is set, both width and height must be specified,
-	 * since we'll need a complete bounding box to fill.
+	/* If there was no 'x' we have just width. vipsthumbnail history means
+	 * this is a square bounding box.
 	 */
-	if( (crop_image || smartcrop_image) &&
-		(thumbnail_width == VIPS_MAX_COORD ||
-			thumbnail_height == VIPS_MAX_COORD) ) {
-		vips_error( "thumbnail",
-			"both width and height must be given if "
-			"crop is enabled" );
-		return( -1 );
+	if( !had_x ) 
+		thumbnail_height = thumbnail_width;
+
+	/* If force is set and one of width or height isn't set, copy from the
+	 * one that is.
+	 */
+	if( size_restriction == VIPS_SIZE_FORCE ) {
+		if( thumbnail_width == VIPS_MAX_COORD )
+			thumbnail_width = thumbnail_height;
+		if( thumbnail_height == VIPS_MAX_COORD )
+			thumbnail_height = thumbnail_width;
 	}
 
+	/* If --crop is set or force is set, both width and height must be 
+	 * specified, since we'll need a complete bounding box to fill.
+	 */
+	if( crop_image || 
+		smartcrop_image || 
+		size_restriction == VIPS_SIZE_FORCE ) 
+		if( thumbnail_width == VIPS_MAX_COORD ||
+			thumbnail_height == VIPS_MAX_COORD ) {
+			vips_error( "thumbnail",
+				"both width and height must be given if "
+				"crop is enabled" );
+			return( -1 );
+		}
+
 	return( 0 );
 }
 
